/*
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 2000 MIPS Technologies, Inc.  All rights reserved.
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * This is the interface to the remote debugger stub.
 *
 */

#include <linux/serialP.h>
#include <linux/serial_reg.h>

#include <asm/serial.h>
#include <asm/io.h>

static struct serial_state rs_table[RS_TABLE_SIZE] = {
	SERIAL_PORT_DFNS	/* Defined in serial.h */
};

static struct async_struct kdb_port_info = {0};

static __inline__ unsigned int serial_in(struct async_struct *info, int offset)
{
	return (*(volatile unsigned char *)(info->port + KSEG1 + offset) & 0xff);
}

static __inline__ void serial_out(struct async_struct *info, int offset,
				int value)
{
        *(volatile unsigned char *)(info->port + KSEG1 + offset) = 
		(unsigned char)value;
        __asm__("sync\n");
}

static void gdb_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
}

void rs_kgdb_hook(int tty_no) {
	int t;
	struct serial_state *ser = &rs_table[tty_no];

	kdb_port_info.state = ser;
	kdb_port_info.magic = SERIAL_MAGIC;
	kdb_port_info.port = ser->port;
	kdb_port_info.flags = ser->flags;

	/*
	 * Clear all interrupts
	 */
	xilleon_serial_in(&kdb_port_info, UART_LSR);
	xilleon_serial_in(&kdb_port_info, UART_RX);
	xilleon_serial_in(&kdb_port_info, UART_IIR);
	xilleon_serial_in(&kdb_port_info, UART_MSR);

	/*
	 * Now, initialize the UART 
	 */
	xilleon_serial_out(&kdb_port_info, UART_LCR, UART_LCR_WLEN8);	/* reset DLAB */
	if (kdb_port_info.flags & ASYNC_FOURPORT) {
		kdb_port_info.MCR = UART_MCR_DTR | UART_MCR_RTS;
		t = UART_MCR_DTR | UART_MCR_OUT1;
	} else {
		kdb_port_info.MCR 
			= UART_MCR_DTR | UART_MCR_RTS | UART_MCR_OUT2;
		t = UART_MCR_DTR | UART_MCR_RTS;
	}

	kdb_port_info.MCR = t;		/* no interrupts, please */
	xilleon_serial_out(&kdb_port_info, UART_MCR, kdb_port_info.MCR);
	
	/* baud rate is fixed to 115200 */
	t = kdb_port_info.state->baud_base / 115200;
	/* set DLAB */
	xilleon_serial_out(&kdb_port_info, UART_LCR, UART_LCR_WLEN8 | UART_LCR_DLAB);

	xilleon_serial_out(&kdb_port_info, UART_DLL, t & 0xff);/* LS of divisor */
	xilleon_serial_out(&kdb_port_info, UART_DLM, t >> 8);  /* MS of divisor */
	/* reset DLAB */
	xilleon_serial_out(&kdb_port_info, UART_LCR, UART_LCR_WLEN8);
}

int putDebugChar(char c)
{

	if (!kdb_port_info.state) { 	/* need to init device first */
		return 0;
	}

	while ((xilleon_serial_in(&kdb_port_info, UART_LSR) & UART_LSR_THRE) == 0)
		;

	xilleon_serial_out(&kdb_port_info, UART_TX, c);

	return 1;
}

char getDebugChar(void)
{
	if (!kdb_port_info.state) { 	/* need to init device first */
		return 0;
	}

	while (!(xilleon_serial_in(&kdb_port_info, UART_LSR) & 1))
		;

	return(xilleon_serial_in(&kdb_port_info, UART_RX));
}
