/*
 * linux/arch/mips/tc90400/common/tc90400_irq.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
 * Common tx4927 irq handler
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kernel_stat.h>
#include <linux/module.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/timex.h>
#include <linux/slab.h>
#include <linux/random.h>
#include <linux/irq.h>
#include <asm/bitops.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mipsregs.h>
#include <asm/system.h>
#include <asm/tc90400/tc90400.h>

/*
 * DEBUG
 */
#define TC90400_IRQ_CHECK_CP0
#define TC90400_IRQ_CHECK_PIC

#undef  TC90400_IRQ_DEBUG

#ifdef  TC90400_IRQ_DEBUG
#define TC90400_IRQ_NONE        0x00000000

#define TC90400_IRQ_INFO        ( 1 <<  0 )
#define TC90400_IRQ_WARN        ( 1 <<  1 )
#define TC90400_IRQ_EROR        ( 1 <<  2 )

#define TC90400_IRQ_INIT        ( 1 <<  5 )
#define TC90400_IRQ_NEST1       ( 1 <<  6 )
#define TC90400_IRQ_NEST2       ( 1 <<  7 )
#define TC90400_IRQ_NEST3       ( 1 <<  8 )
#define TC90400_IRQ_NEST4       ( 1 <<  9 )

#define TC90400_IRQ_CP0_INIT     ( 1 << 10 )
#define TC90400_IRQ_CP0_STARTUP  ( 1 << 11 )
#define TC90400_IRQ_CP0_SHUTDOWN ( 1 << 12 )
#define TC90400_IRQ_CP0_ENABLE   ( 1 << 13 )
#define TC90400_IRQ_CP0_DISABLE  ( 1 << 14 )
#define TC90400_IRQ_CP0_MASK     ( 1 << 15 )
#define TC90400_IRQ_CP0_ENDIRQ   ( 1 << 16 )

#define TC90400_IRQ_PIC_INIT     ( 1 << 20 )
#define TC90400_IRQ_PIC_STARTUP  ( 1 << 21 )
#define TC90400_IRQ_PIC_SHUTDOWN ( 1 << 22 )
#define TC90400_IRQ_PIC_ENABLE   ( 1 << 23 )
#define TC90400_IRQ_PIC_DISABLE  ( 1 << 24 )
#define TC90400_IRQ_PIC_MASK     ( 1 << 25 )
#define TC90400_IRQ_PIC_ENDIRQ   ( 1 << 26 )

#define TC90400_IRQ_ALL         0xffffffff
#endif

#ifdef TC90400_IRQ_DEBUG
static const u32 tc90400_irq_debug_flag = (TC90400_IRQ_NONE
					  | TC90400_IRQ_INFO
					  | TC90400_IRQ_WARN | TC90400_IRQ_EROR
//                                       | TC90400_IRQ_CP0_INIT
//                                       | TC90400_IRQ_CP0_STARTUP
//                                       | TC90400_IRQ_CP0_SHUTDOWN
//                                       | TC90400_IRQ_CP0_ENABLE
//                                       | TC90400_IRQ_CP0_DISABLE
//                                       | TC90400_IRQ_CP0_MASK
//                                       | TC90400_IRQ_CP0_ENDIRQ
//                                       | TC90400_IRQ_PIC_INIT
//                                       | TC90400_IRQ_PIC_STARTUP
//                                       | TC90400_IRQ_PIC_SHUTDOWN
//                                       | TC90400_IRQ_PIC_ENABLE
//                                       | TC90400_IRQ_PIC_DISABLE
//                                       | TC90400_IRQ_PIC_MASK
//                                       | TC90400_IRQ_PIC_ENDIRQ
//                                       | TC90400_IRQ_INIT
//                                       | TC90400_IRQ_NEST1
//                                       | TC90400_IRQ_NEST2
//                                       | TC90400_IRQ_NEST3
//                                       | TC90400_IRQ_NEST4
    );
#endif

#ifdef TC90400_IRQ_DEBUG
#define TC90400_IRQ_DPRINTK(flag,str...) \
        if ( (tc90400_irq_debug_flag) & (flag) ) \
        { \
           char tmp[100]; \
           sprintf( tmp, str ); \
           printk( "%s(%s:%u)::%s", __FUNCTION__, __FILE__, __LINE__, tmp ); \
        }
#else
#define TC90400_IRQ_DPRINTK(flag,str...)
#endif

/*
 * Forwad definitions for all pic's
 */

static unsigned int tc90400_irq_cp0_startup(unsigned int irq);
static void tc90400_irq_cp0_shutdown(unsigned int irq);
static void tc90400_irq_cp0_enable(unsigned int irq);
static void tc90400_irq_cp0_disable(unsigned int irq);
static void tc90400_irq_cp0_mask_and_ack(unsigned int irq);
static void tc90400_irq_cp0_end(unsigned int irq);

static unsigned int tc90400_irq_pic_startup(unsigned int irq);
static void tc90400_irq_pic_shutdown(unsigned int irq);
static void tc90400_irq_pic_enable(unsigned int irq);
static void tc90400_irq_pic_disable(unsigned int irq);
static void tc90400_irq_pic_mask_and_ack(unsigned int irq);
static void tc90400_irq_pic_end(unsigned int irq);

/*
 * Kernel structs for all pic's
 */

static spinlock_t tc90400_cp0_lock = SPIN_LOCK_UNLOCKED;
static spinlock_t tc90400_pic_lock = SPIN_LOCK_UNLOCKED;

#define TC90400_CP0_NAME "TC90400-CP0"
static struct hw_interrupt_type tc90400_irq_cp0_type = {
	typename:	TC90400_CP0_NAME,
	startup:	tc90400_irq_cp0_startup,
	shutdown:	tc90400_irq_cp0_shutdown,
	enable:		tc90400_irq_cp0_enable,
	disable:	tc90400_irq_cp0_disable,
	ack:		tc90400_irq_cp0_mask_and_ack,
	end:		tc90400_irq_cp0_end,
	set_affinity:	NULL
};

#define TC90400_PIC_NAME "TC90400-PIC"
static struct hw_interrupt_type tc90400_irq_pic_type = {
	typename:	TC90400_PIC_NAME,
	startup:	tc90400_irq_pic_startup,
	shutdown:	tc90400_irq_pic_shutdown,
	enable:		tc90400_irq_pic_enable,
	disable:	tc90400_irq_pic_disable,
	ack:		tc90400_irq_pic_mask_and_ack,
	end:		tc90400_irq_pic_end,
	set_affinity:	NULL
};

#define TC90400_PIC_ACTION(s) { no_action, 0, 0, s, NULL, NULL }
static struct irqaction tc90400_irq_pic_action =
TC90400_PIC_ACTION(TC90400_PIC_NAME);

#define CCP0_STATUS 12
#define CCP0_CAUSE 13

/*
 * Functions for cp0
 */

#define tc90400_irq_cp0_mask(irq) ( 1 << ( irq-TC90400_IRQ_CP0_BEG+8 ) )

static void tc90400_irq_cp0_modify(unsigned cp0_reg, unsigned clr_bits,
	unsigned set_bits)
{
	unsigned long val = 0;

	switch (cp0_reg) {
	case CCP0_STATUS:
		val = read_c0_status();
		break;

	case CCP0_CAUSE:
		val = read_c0_cause();
		break;

	}

	val &= (~clr_bits);
	val |= (set_bits);

	switch (cp0_reg) {
	case CCP0_STATUS:{
			write_c0_status(val);
			break;
		}
	case CCP0_CAUSE:{
			write_c0_cause(val);
			break;
		}
	}

	return;
}

static void __init tc90400_irq_cp0_init(void)
{
	int i;

	TC90400_IRQ_DPRINTK(TC90400_IRQ_CP0_INIT, "beg=%d end=%d\n",
			   TC90400_IRQ_CP0_BEG, TC90400_IRQ_CP0_END);

	for (i = TC90400_IRQ_CP0_BEG; i <= TC90400_IRQ_CP0_END; i++) {
		irq_desc[i].status = IRQ_DISABLED;
		irq_desc[i].action = 0;
		irq_desc[i].depth = 1;
		irq_desc[i].handler = &tc90400_irq_cp0_type;
	}

	return;
}

static unsigned int tc90400_irq_cp0_startup(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_CP0_STARTUP, "irq=%d \n", irq);

#ifdef TC90400_IRQ_CHECK_CP0
	{
		if (irq < TC90400_IRQ_CP0_BEG || irq > TC90400_IRQ_CP0_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	tc90400_irq_cp0_enable(irq);

	return (0);
}

static void tc90400_irq_cp0_shutdown(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_CP0_SHUTDOWN, "irq=%d \n", irq);

#ifdef TC90400_IRQ_CHECK_CP0
	{
		if (irq < TC90400_IRQ_CP0_BEG || irq > TC90400_IRQ_CP0_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	tc90400_irq_cp0_disable(irq);

	return;
}

static void tc90400_irq_cp0_enable(unsigned int irq)
{
	unsigned long flags;

	TC90400_IRQ_DPRINTK(TC90400_IRQ_CP0_ENABLE, "irq=%d \n", irq);

#ifdef TC90400_IRQ_CHECK_CP0
	{
		if (irq < TC90400_IRQ_CP0_BEG || irq > TC90400_IRQ_CP0_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	spin_lock_irqsave(&tc90400_cp0_lock, flags);

	tc90400_irq_cp0_modify(CCP0_STATUS, 0, tc90400_irq_cp0_mask(irq));

	spin_unlock_irqrestore(&tc90400_cp0_lock, flags);

	return;
}

static void tc90400_irq_cp0_disable(unsigned int irq)
{
	unsigned long flags;

	TC90400_IRQ_DPRINTK(TC90400_IRQ_CP0_DISABLE, "irq=%d \n", irq);

#ifdef TC90400_IRQ_CHECK_CP0
	{
		if (irq < TC90400_IRQ_CP0_BEG || irq > TC90400_IRQ_CP0_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	spin_lock_irqsave(&tc90400_cp0_lock, flags);

	tc90400_irq_cp0_modify(CCP0_STATUS, tc90400_irq_cp0_mask(irq), 0);

	spin_unlock_irqrestore(&tc90400_cp0_lock, flags);

	return;
}

static void tc90400_irq_cp0_mask_and_ack(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_CP0_MASK, "irq=%d \n", irq);

#ifdef TC90400_IRQ_CHECK_CP0
	{
		if (irq < TC90400_IRQ_CP0_BEG || irq > TC90400_IRQ_CP0_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	tc90400_irq_cp0_disable(irq);

	return;
}

static void tc90400_irq_cp0_end(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_CP0_ENDIRQ, "irq=%d \n", irq);

#ifdef TC90400_IRQ_CHECK_CP0
	{
		if (irq < TC90400_IRQ_CP0_BEG || irq > TC90400_IRQ_CP0_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	if (!(irq_desc[irq].status & (IRQ_DISABLED | IRQ_INPROGRESS))) {
		tc90400_irq_cp0_enable(irq);
	}

	return;
}

/*
 * Functions for pic
 */

u32 tc90400_irq_pic_addr(int irq)
{
	/* MVMCP -- need to formulize this */
	irq -= TC90400_IRQ_PIC_BEG;
	switch (irq) {
	case 17:
	case 16:
	case 1:
	case 0:
		return (TC90400_INTLEVEL(0));

	case 19:
	case 18:
	case 3:
	case 2:
		return (TC90400_INTLEVEL(1));

	case 21:
	case 20:
	case 5:
	case 4:
		return (TC90400_INTLEVEL(2));

	case 23:
	case 22:
	case 7:
	case 6:
		return (TC90400_INTLEVEL(3));

	case 25:
	case 24:
	case 9:
	case 8:
		return (TC90400_INTLEVEL(4));

	case 27:
	case 26:
	case 11:
	case 10:
		return (TC90400_INTLEVEL(5));

	case 29:
	case 28:
	case 13:
	case 12:
		return (TC90400_INTLEVEL(6));

	case 31:
	case 30:
	case 15:
	case 14:
		return (TC90400_INTLEVEL(7));

	}
	return (0);
}

u32 tc90400_irq_pic_mask(int irq)
{
	/* MVMCP -- need to formulize this */
	irq -= TC90400_IRQ_PIC_BEG;
	switch (irq) {
	case 31:
	case 29:
	case 27:
	case 25:
	case 23:
	case 21:
	case 19:
	case 17:{
			return (0x07000000);
		}
	case 30:
	case 28:
	case 26:
	case 24:
	case 22:
	case 20:
	case 18:
	case 16:{
			return (0x00070000);
		}
	case 15:
	case 13:
	case 11:
	case 9:
	case 7:
	case 5:
	case 3:
	case 1:{
			return (0x00000700);
		}
	case 14:
	case 12:
	case 10:
	case 8:
	case 6:
	case 4:
	case 2:
	case 0:{
			return (0x00000007);
		}
	}
	return (0x00000000);
}

static void tc90400_irq_pic_modify(unsigned pic_reg, unsigned clr_bits,
	unsigned set_bits)
{
	unsigned long val = 0;
	unsigned long tmpval = 0;

	tmpval = TC90400_RD(TC90400_INTMASK); /* backup interrupt mask level */
	TC90400_WR(TC90400_INTMASK, 0x7);     /* all interrupt level mask */

	val = TC90400_RD(pic_reg);
	val &= (~clr_bits);
	val |= (set_bits);
	TC90400_WR(pic_reg, val);

	TC90400_WR(TC90400_INTMASK, tmpval); /* restore interrupt mask level */
	return;
}

static void __init tc90400_irq_pic_init(void)
{
	unsigned long flags;
	int i;

	TC90400_IRQ_DPRINTK(TC90400_IRQ_PIC_INIT, "beg=%d end=%d\n",
			   TC90400_IRQ_PIC_BEG, TC90400_IRQ_PIC_END);

	for (i = TC90400_IRQ_PIC_BEG; i <= TC90400_IRQ_PIC_END; i++) {
		irq_desc[i].status = IRQ_DISABLED;
		irq_desc[i].action = 0;
		irq_desc[i].depth = 2;
		irq_desc[i].handler = &tc90400_irq_pic_type;
	}

	setup_irq(TC90400_IRQ_NEST_PIC_ON_CP0, &tc90400_irq_pic_action);

	spin_lock_irqsave(&tc90400_pic_lock, flags);

	TC90400_WR(TC90400_INTMASK, TC90400_INTMASK_IML(6));	/* irq level mask -- only accept hightest */
	TC90400_WR(TC90400_INTCONT0, TC90400_RD(TC90400_INTCONT0)|TC90400_INTCONT0_ICF);	/* irq enable */

	spin_unlock_irqrestore(&tc90400_pic_lock, flags);

	return;
}

static unsigned int tc90400_irq_pic_startup(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_PIC_STARTUP, "irq=%d\n", irq);

#ifdef TC90400_IRQ_CHECK_PIC
	{
		if (irq < TC90400_IRQ_PIC_BEG || irq > TC90400_IRQ_PIC_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	tc90400_irq_pic_enable(irq);

	return (0);
}

static void tc90400_irq_pic_shutdown(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_PIC_SHUTDOWN, "irq=%d\n", irq);

#ifdef TC90400_IRQ_CHECK_PIC
	{
		if (irq < TC90400_IRQ_PIC_BEG || irq > TC90400_IRQ_PIC_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	tc90400_irq_pic_disable(irq);

	return;
}

static void tc90400_irq_pic_enable(unsigned int irq)
{
	unsigned long flags;

	TC90400_IRQ_DPRINTK(TC90400_IRQ_PIC_ENABLE, "irq=%d\n", irq);

#ifdef TC90400_IRQ_CHECK_PIC
	{
		if (irq < TC90400_IRQ_PIC_BEG || irq > TC90400_IRQ_PIC_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	spin_lock_irqsave(&tc90400_pic_lock, flags);

	tc90400_irq_pic_modify(tc90400_irq_pic_addr(irq), 0,
			      tc90400_irq_pic_mask(irq));

	spin_unlock_irqrestore(&tc90400_pic_lock, flags);

	return;
}

static void tc90400_irq_pic_disable(unsigned int irq)
{
	unsigned long flags;

	TC90400_IRQ_DPRINTK(TC90400_IRQ_PIC_DISABLE, "irq=%d\n", irq);

#ifdef TC90400_IRQ_CHECK_PIC
	{
		if (irq < TC90400_IRQ_PIC_BEG || irq > TC90400_IRQ_PIC_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	spin_lock_irqsave(&tc90400_pic_lock, flags);

	tc90400_irq_pic_modify(tc90400_irq_pic_addr(irq),
			      tc90400_irq_pic_mask(irq), 0);

	spin_unlock_irqrestore(&tc90400_pic_lock, flags);

	return;
}

static void tc90400_irq_pic_mask_and_ack(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_PIC_MASK, "irq=%d\n", irq);

#ifdef TC90400_IRQ_CHECK_PIC
	{
		if (irq < TC90400_IRQ_PIC_BEG || irq > TC90400_IRQ_PIC_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	tc90400_irq_pic_disable(irq);

	return;
}

static void tc90400_irq_pic_end(unsigned int irq)
{
	TC90400_IRQ_DPRINTK(TC90400_IRQ_PIC_ENDIRQ, "irq=%d\n", irq);

#ifdef TC90400_IRQ_CHECK_PIC
	{
		if (irq < TC90400_IRQ_PIC_BEG || irq > TC90400_IRQ_PIC_END) {
			TC90400_IRQ_DPRINTK(TC90400_IRQ_EROR,
					   "bad irq=%d \n", irq);
			panic("\n");
		}
	}
#endif

	if (!(irq_desc[irq].status & (IRQ_DISABLED | IRQ_INPROGRESS))) {
		tc90400_irq_pic_enable(irq);
	}

	return;
}

/*
 * Main init functions
 */
void __init tc90400_irq_init(void)
{
	extern asmlinkage void tc90400_irq_handler(void);

	TC90400_IRQ_DPRINTK(TC90400_IRQ_INIT, "-\n");

	TC90400_IRQ_DPRINTK(TC90400_IRQ_INIT, "=Calling tc90400_irq_cp0_init()\n");
	tc90400_irq_cp0_init();

	TC90400_IRQ_DPRINTK(TC90400_IRQ_INIT, "=Calling tc90400_irq_pic_init()\n");
	tc90400_irq_pic_init();

	TC90400_IRQ_DPRINTK(TC90400_IRQ_INIT,
			   "=Calling set_except_vector(tc90400_irq_handler)\n");
	set_except_vector(0, tc90400_irq_handler);

	TC90400_IRQ_DPRINTK(TC90400_IRQ_INIT, "+\n");

	return;
}

int tc90400_irq_nested(void)
{
	int sw_irq = 0;
	u32 level2;

	TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST1, "-\n");

	level2 = TC90400_RD(TC90400_INTVEC);
	TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST2, "=level2a=0x%x\n", level2);

	if ((level2 & TC90400_INTVEC_FLAG) == 0) {
		level2 &= TC90400_INTVEC_CAUSE_MASK;
		TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST3, "=level2b=0x%x\n", level2);

		sw_irq = TC90400_IRQ_PIC_BEG + level2;
		TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST3, "=sw_irq=%d\n", sw_irq);

#ifdef CONFIG_TOSHIBA_TC90400GDRS
		if (sw_irq == TC90400_IRQ_EXT0) { /* FPGA on chip board */
			TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST4, "=irq-%d\n",
					   sw_irq);

			{
				sw_irq = tc90400gdrs_irq_nested(sw_irq);
			}

			TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST4, "=irq+%d\n",
					   sw_irq);
		}
		if (sw_irq == TC90400_IRQ_EXT1) { /* FPGA on main board */
			TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST4, "=irq-%d\n",
					   sw_irq);

			{
				sw_irq = tc90400gdrs_irq_nested(sw_irq);
			}

			TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST4, "=irq+%d\n",
					   sw_irq);
		}
#endif
	}

	TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST2, "=sw_irq=%d\n", sw_irq);

	TC90400_IRQ_DPRINTK(TC90400_IRQ_NEST1, "+\n");

	return (sw_irq);
}

void tc90400_irq_spurious(void)
{
	printk("TC90400 SPURIOUS INTERRUPT!!!\n");
}
