/*
 * linux/arch/mips/tc90400/common/tc90400_setup.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
 * linux/arch/mips/tx4927/common/tx4927_setup.c
 *
 * common tx4927 setup stuff
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kernel_stat.h>
#include <linux/module.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/timex.h>
#include <linux/slab.h>
#include <linux/random.h>
#include <linux/irq.h>
#include <asm/bitops.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mipsregs.h>
#include <asm/system.h>
#include <asm/time.h>
#include <asm/tc90400/tc90400.h>

#undef DEBUG
#undef TIMER_CHECK

void __init tc90400_setup(void);
void __init tc90400_time_init(void);
void __init tc90400_timer_setup(struct irqaction *irq);
void dump_cp0(char *key);
void (*__wbflush) (void);

int tc90400_cpu_clock = 100000000;	/* 100MHz */


static void tc90400_write_buffer_flush(void)
{
	__asm__ __volatile__
	    ("sync\n\t" "nop\n\t" "loop: bc0f loop\n\t" "nop\n\t");
}

unsigned long measureFreq(void)
{
	int n, omcr;
	u32 cpuclock = 0;

	omcr = (TC90400_RD(TC90400_CG_OMCR) & 0x00000007);
	
	n = (omcr & 0x00000003) + 2;
	if( omcr & 0x00000004 )  {
		cpuclock = (TC90400_XT_CLCK_30 * 24) / ( 2 * n);
	}
	else {
		cpuclock = (TC90400_XT_CLCK_27 * 24) / ( 2 * n);
	}
	return cpuclock;
}

void __init tc90400_setup(void)
{
	board_time_init = tc90400_time_init;
	board_timer_setup = tc90400_timer_setup;
	__wbflush = tc90400_write_buffer_flush;

        /* cpu clock */
	tc90400_cpu_clock = measureFreq();

#ifdef CONFIG_TOSHIBA_TC90400EVB
	{
		extern void tc90400_evb_setup(void);
		tc90400_evb_setup();
	}
#endif
#ifdef CONFIG_TOSHIBA_TC90400GDRS
	{
		extern void tc90400_gdrs_setup(void);
		tc90400_gdrs_setup();
	}
#endif
#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
	{
		extern void tc90400_qfp_board01_setup(void);
		tc90400_qfp_board01_setup();
	}
#endif

	return;
}


void __init tc90400_time_init(void)
{
	u32	temp;

#ifdef TIMER_CHECK
	 /* How to check it is different in the kind of the board */
        timer_interval_check_init();
#endif
	/* TC90400 TMR 0 Init */
	TC90400_WR(TC90400_TMTCR(0),
		(TC90400_TMTCR_CCDE_ENABLE  | TC90400_TMTCR_CRE_ENABLE |
		 TC90400_TMTCR_CCS_INTERNAL | TC90400_TMTCR_TMODE_IT));
	TC90400_WR(TC90400_TMTISR(0), 0);
	TC90400_WR(TC90400_TMCCDR(0), 0);	/* Div 2 */
	temp = ((tc90400_cpu_clock / 2) / 2);	/* E-BUS CLK */
	temp = (temp / 2) / HZ;
	TC90400_WR(TC90400_TMCPRA(0), temp);

	mips_counter_frequency = tc90400_cpu_clock / 2;

#ifdef CONFIG_KGDB
	{
		printk("Calling breakpoint() -- start remote kgdb\n");
		set_debug_traps();
		breakpoint();
		printk("Calling breakpoint() -- done\n");
	}
#endif

	return;
}

static void tc90400_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	/* Acknowledge and reset the interrupt */
	TC90400_WR(TC90400_TMTISR(0), 0);

#ifdef TIMER_CHECK
	 /* How to check it is different in the kind of the board */
        timer_interval_check();
#endif

#if 0 /* for CPU internal timer not used */
	timer_interrupt(irq, NULL, regs);
#else
	do_timer(regs);
	local_timer_interrupt(0, NULL, regs);
#endif
}

void __init tc90400_timer_setup(struct irqaction *irq)
{
	u32	temp;

        /* set timer handler */
	irq->handler = &tc90400_timer_interrupt;
	setup_irq(TC90400_IRQ_TMR0, irq);

	/* Enable Interrupt */
	temp = (TC90400_TMITMR_TIIE_ENABLE | TC90400_TMITMR_TZCE_ENABLE);
	TC90400_WR(TC90400_TMITMR(0), temp);

	/* Start Counter */
	temp = (TC90400_RD(TC90400_TMTCR(0)) & (~TC90400_TMTCR_CRE_ENABLE));
	TC90400_WR(TC90400_TMTCR(0), temp);
	TC90400_WR(TC90400_TMTCR(0), (temp | TC90400_TMTCR_TCE_RUNS));

	return;
}



#ifdef DEBUG
void print_cp0(char *key, int num, char *name, u32 val)
{
	printk("%s cp0:%02d:%s=0x%08x\n", key, num, name, val);
	return;
}

void
dump_cp0(char *key)
{
	if (key == NULL)
		key = "";

	print_cp0(key, 0, "INDEX   ", read_c0_index());
	print_cp0(key, 2, "ENTRYLO1", read_c0_entrylo0());
	print_cp0(key, 3, "ENTRYLO2", read_c0_entrylo1());
	print_cp0(key, 4, "CONTEXT ", read_c0_context());
	print_cp0(key, 5, "PAGEMASK", read_c0_pagemask());
	print_cp0(key, 6, "WIRED   ", read_c0_wired());
	//print_cp0(key, 8, "BADVADDR",  read_c0_badvaddr());
	print_cp0(key, 9, "COUNT   ", read_c0_count());
	print_cp0(key, 10, "ENTRYHI ", read_c0_entryhi());
	print_cp0(key, 11, "COMPARE ", read_c0_compare());
	print_cp0(key, 12, "STATUS  ", read_c0_status());
	print_cp0(key, 13, "CAUSE   ", read_c0_cause() & 0xffff87ff);
	print_cp0(key, 16, "CONFIG  ", read_c0_config());
	return;
}
	
void print_pic(char *key, u32 reg, char *name)
{
	printk("%s pic:0x%08x:%s=0x%08x\n", key, reg, name,
	       TC90400_RD(reg));
	return;
}


void dump_pic(char *key)
{
	if (key == NULL)
		key = "";

	print_pic(key, TC90400_INTCONT0, "INTCONT0 ");
	print_pic(key, TC90400_INTCONT1, "INTCONT1 ");
	print_pic(key, TC90400_INTCONT2, "INTCONT2 ");

	print_pic(key, TC90400_INTLEVEL(0), "INTLVL0  ");
	print_pic(key, TC90400_INTLEVEL(1), "INTLVL1  ");
	print_pic(key, TC90400_INTLEVEL(2), "INTLVL2  ");
	print_pic(key, TC90400_INTLEVEL(3), "INTLVL3  ");
	print_pic(key, TC90400_INTLEVEL(4), "INTLVL4  ");
	print_pic(key, TC90400_INTLEVEL(5), "INTLVL5  ");
	print_pic(key, TC90400_INTLEVEL(6), "INTLVL6  ");
	print_pic(key, TC90400_INTLEVEL(7), "INTLVL7  ");

	print_pic(key, TC90400_INTMASK, "INTMASK  ");
	print_pic(key, TC90400_INTSTSC, "INTSTSC  ");
	print_pic(key, TC90400_INTSTS,  "INTSTS   ");
	print_pic(key, TC90400_INTVEC,  "INTVEC   ");

	return;
}


void print_addr(char *hdr, char *key, u32 addr)
{
	printk("%s %s:0x%08x=0x%08x\n", hdr, key, addr, TC90400_RD(addr));
	return;
}


void dump_180(char *key)
{
	u32 i;

	for (i = 0x80000180; i < 0x80000180 + 0x80; i += 4) {
		print_addr("180", key, i);
	}
	return;
}


void dump_eh0(char *key)
{
	int i;
	extern unsigned long exception_handlers[];

	for (i = (int) exception_handlers;
	     i < (int) (exception_handlers + 20); i += 4) {
		print_addr("eh0", key, i);
	}

	return;
}

void pk0(void)
{
	volatile u32 val;

	__asm__ __volatile__("ori %0, $26, 0":"=r"(val)
	    );
	printk("k0=[0x%08x]\n", val);
}
#endif
