/*
 * linux/arch/mips/tc90400/eva_board/eva_board_setup.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
 * Toshiba rbtx4927 specific setup
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 * Copyright (C) 1996, 1997, 2001  Ralf Baechle
 * Copyright (C) 2000 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *   glonnon@ridgerun.com, skranz@ridgerun.com, stevej@ridgerun.com
 *
 * Copyright 2001 MontaVista Software Inc.
 * Author: jsun@mvista.com or jsun@junsun.net
 *
 * Copyright 2002 MontaVista Software Inc.
 * Author: Michael Pruznick, michael_pruznick@mvista.com
 *
 * Copyright (C) 2000-2001 Toshiba Corporation 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/timex.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/pci.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/time.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <asm/tc90400/tc90400evb.h>

#undef TOSHIBA_TC90400EVB_SETUP_DEBUG

#ifdef TOSHIBA_TC90400EVB_SETUP_DEBUG
#define TOSHIBA_TC90400EVB_SETUP_NONE        0x00000000

#define TOSHIBA_TC90400EVB_SETUP_INFO        ( 1 <<  0 )
#define TOSHIBA_TC90400EVB_SETUP_WARN        ( 1 <<  1 )
#define TOSHIBA_TC90400EVB_SETUP_EROR        ( 1 <<  2 )

#define TOSHIBA_TC90400EVB_SETUP_EFWFU       ( 1 <<  3 )
#define TOSHIBA_TC90400EVB_SETUP_SETUP       ( 1 <<  4 )
#define TOSHIBA_TC90400EVB_SETUP_TIME_INIT   ( 1 <<  5 )
#define TOSHIBA_TC90400EVB_SETUP_TIMER_SETUP ( 1 <<  6 )

#define TOSHIBA_TC90400EVB_SETUP_ALL         0xffffffff
#endif

#ifdef TOSHIBA_TC90400EVB_SETUP_DEBUG
static const u32 tc90400evb_setup_debug_flag =
   (TOSHIBA_TC90400EVB_SETUP_NONE | TOSHIBA_TC90400EVB_SETUP_INFO |
    TOSHIBA_TC90400EVB_SETUP_WARN | TOSHIBA_TC90400EVB_SETUP_EROR |
    TOSHIBA_TC90400EVB_SETUP_EFWFU | TOSHIBA_TC90400EVB_SETUP_SETUP |
    TOSHIBA_TC90400EVB_SETUP_TIME_INIT | TOSHIBA_TC90400EVB_SETUP_TIMER_SETUP);
#endif

#ifdef TOSHIBA_TC90400EVB_SETUP_DEBUG
#define TOSHIBA_TC90400EVB_SETUP_DPRINTK(flag,str...) \
        if ( (tc90400evb_setup_debug_flag) & (flag) ) \
        { \
           char tmp[100]; \
           sprintf( tmp, str ); \
           printk( "%s(%s:%u)::%s", __FUNCTION__, __FILE__, __LINE__, tmp ); \
        }
#else
#define TOSHIBA_TC90400EVB_SETUP_DPRINTK(flag,str...)
#endif

/* These functions are used for rebooting or halting the machine*/
extern void toshiba_tc90400evb_restart(char *command);
extern void toshiba_tc90400evb_halt(void);
extern void toshiba_tc90400evb_power_off(void);

extern char *prom_getcmdline(void);

static int tc90400_ccfg_toeon = 1;

#ifdef CONFIG_PCI
#undef TC90400_SUPPORT_COMMAND_IO
#undef  TC90400_SUPPORT_PCI_66

unsigned long mips_pci_io_base;
unsigned long mips_pci_io_size;
unsigned long mips_pci_mem_base;
unsigned long mips_pci_mem_size;
/* for legacy I/O, PCI I/O PCI Bus address must be 0 */
unsigned long mips_memory_upper;
void tc90400_pci_setup(void);
#endif

char *toshiba_name = "";

/* HP display controll */
#define HDISP_CTRL_REG (TC90400_HPDISP_BASE+(0x30*2)+0x1) /* 16bit bus, big-endian offset */
#define HDISP_CHAR_REG (TC90400_HPDISP_BASE+(0x38*2)+0x1) /* 16bit bus, big-endian offset */
#define hdsp_outb(a,v)	(*((volatile unsigned char *)(a))=(v))


void hdisp_cmd(char cmd)
{
	hdsp_outb(HDISP_CTRL_REG, cmd);
}

void hdisp_putchar(char data, char pos)
{
	hdsp_outb(HDISP_CHAR_REG + (pos * 2), data);
}

void hdisp_print(char *str)
{
	int i;
	
	for(i = 0; i < 8 && *str != 0; i++)
	{
		hdisp_putchar(*str, i);
		str++;
	}
}


#ifdef CONFIG_PCI
void tc90400_pci_setup(void)
{
	extern unsigned int tc90400_get_mem_size(void);

	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2, "-\n");

	mips_memory_upper = tc90400_get_mem_size() << 20;
	mips_memory_upper += KSEG0;
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2,
				       "0x%08lx=mips_memory_upper\n",
				       mips_memory_upper);
	mips_pci_io_base = TC90400_PCIIO;
	mips_pci_io_size = TC90400_PCIIO_SIZE;
	mips_pci_mem_base = TC90400_PCIMEM;
	mips_pci_mem_size = TC90400_PCIMEM_SIZE;

	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_base\n",
				       mips_pci_io_base);
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_size\n",
				       mips_pci_io_size);
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_base\n",
				       mips_pci_mem_base);
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_size\n",
				       mips_pci_mem_size);
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2,
				       "0x%08lx=mips_io_port_base",
				       mips_io_port_base);

	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_PCI2, "+\n");
}
#endif /* CONFIG_PCI */

void toshiba_tc90400evb_restart(char *command)
{
	printk(KERN_NOTICE "System Rebooting...\n");

	cli();
	flush_cache_all();
	write_c0_wired(0);
	__asm__ __volatile__("jr\t%0"::"r"(0xbfc00000));

	/* no return */
}

void toshiba_tc90400evb_halt(void)
{
	printk(KERN_NOTICE "System Halted\n");
	cli();
	while (1) {
		asm_wait();
	}
	/* no return */
}

void toshiba_tc90400evb_power_off(void)
{
	toshiba_tc90400evb_halt();
	/* no return */
}

void __init tc90400_evb_setup(void)
{
	vu32 cp0_config;
	char *argptr;
        unsigned long temp_reg;

	printk("CPU is %s\n", toshiba_name);

	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       "-\n");

	/* f/w leaves this on at startup */
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       ":Clearing STO_ERL.\n");
	clear_c0_status(ST0_ERL);

	/* enable caches -- HCP5 does this, pmon does not */
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       ":Enabling TX49_CONF_IC,TX49_CONF_DC.\n");
	cp0_config = read_c0_config();
	cp0_config = cp0_config & ~(TX49_CONF_IC | TX49_CONF_DC);
	write_c0_config(cp0_config);

#ifdef TOSHIBA_TC90400EVB_SETUP_DEBUG
	{
		extern void dump_cp0(char *);
		dump_cp0("toshiba_tc90400_early_fw_fixup");
	}
#endif

	/* setup irq stuff */
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       ":Setting up tc90400 pic.\n");
	TC90400_WR(TC90400_INTCONT1, TC90400_INTCONT1_I2C1); /* i2c1 edge sense */
	TC90400_WR(TC90400_INTCONT2, TC90400_INTCONT2_I2C0); /* i2c0 edge sense */
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       "+\n");
	/* setup share pin */
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       ":Setting up tc90400 pic.\n");
	TC90400_WR(TC90400_CFSH3, (TC90400_RD(TC90400_CFSH3) & ~(TC90400_CFSH3_FABPID))); /* disable all bidirection into input */
        temp_reg = TC90400_CFSH0_I2C0|TC90400_CFSH0_I2C1; /* enable I2C0,1 */
	TC90400_WR(TC90400_CFSH0, (TC90400_RD(TC90400_CFSH0)|temp_reg));
        temp_reg = (TC90400_CFSH1_INT0 | TC90400_CFSH1_SIO0 | TC90400_CFSH1_SIO1 | TC90400_CFSH1_SIO1SEND); /* enable INT0, SIO0,1 */
	TC90400_WR(TC90400_CFSH1, (TC90400_RD(TC90400_CFSH1)|temp_reg));
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       "+\n");

	/* print to HP display */
	hdisp_print("TC90400");

	mips_io_port_base = KSEG1 + TC90400EVB_ISA_IO_OFFSET;
	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       ":mips_io_port_base=0x%08lx\n",
				       mips_io_port_base);

	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       ":Resource\n");
	ioport_resource.start = 0;
	ioport_resource.end = 0xffffffff;
	iomem_resource.start = 0;
	iomem_resource.end = 0xffffffff;


	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       ":ResetRoutines\n");
	_machine_restart = toshiba_tc90400evb_restart;
	_machine_halt = toshiba_tc90400evb_halt;
	_machine_power_off = toshiba_tc90400evb_power_off;

#ifdef CONFIG_PCI
	/* CCFG */
	/* enable Timeout BusError */
	if (tc90400_ccfg_toeon)
		TC90400_WR(TC90400_GBTOUT, TC90400_GBTOUT_ON);

	tc90400_pci_setup();
#endif

#ifdef CONFIG_SERIAL_TC90400_CONSOLE
        argptr = prom_getcmdline();
        if (strstr(argptr, "console=") == NULL) {
                strcat(argptr, " console=ttyS0,38400");
        }
#endif

#ifdef CONFIG_IP_PNP
        argptr = prom_getcmdline();
        if (strstr(argptr, "ip=") == NULL) {
                strcat(argptr, " ip=any");
        }
#endif

#ifdef CONFIG_CS89x0
        argptr = prom_getcmdline();
        if (strstr(argptr, "MAC=") == NULL) {
                strcat(argptr, " MAC=00:00:39:83:02:0c");
        }
#endif

#ifdef CONFIG_ROOT_NFS
        argptr = prom_getcmdline();
        if (strstr(argptr, "root=") == NULL) {
                strcat(argptr, " root=/dev/nfs rw");
        }
#endif

	TOSHIBA_TC90400EVB_SETUP_DPRINTK(TOSHIBA_TC90400EVB_SETUP_SETUP,
				       "+\n");
}

/* tick(TMR0) timer check */
void timer_interval_check_init(void)
{
        return;
}

void timer_interval_check(void)
{
        return;
}
