/*
 * linux/arch/mips/tc90400/gdrs_board/gdrs_board_setup.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
 * Toshiba rbtx4927 specific setup
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 * Copyright (C) 1996, 1997, 2001  Ralf Baechle
 * Copyright (C) 2000 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *   glonnon@ridgerun.com, skranz@ridgerun.com, stevej@ridgerun.com
 *
 * Copyright 2001 MontaVista Software Inc.
 * Author: jsun@mvista.com or jsun@junsun.net
 *
 * Copyright 2002 MontaVista Software Inc.
 * Author: Michael Pruznick, michael_pruznick@mvista.com
 *
 * Copyright (C) 2000-2001 Toshiba Corporation 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/timex.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/pci.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/time.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <asm/tc90400/tc90400gdrs.h>

#undef TOSHIBA_TC90400GDRS_SETUP_DEBUG

#ifdef TOSHIBA_TC90400GDRS_SETUP_DEBUG
#define TOSHIBA_TC90400GDRS_SETUP_NONE        0x00000000

#define TOSHIBA_TC90400GDRS_SETUP_INFO        ( 1 <<  0 )
#define TOSHIBA_TC90400GDRS_SETUP_WARN        ( 1 <<  1 )
#define TOSHIBA_TC90400GDRS_SETUP_EROR        ( 1 <<  2 )

#define TOSHIBA_TC90400GDRS_SETUP_EFWFU       ( 1 <<  3 )
#define TOSHIBA_TC90400GDRS_SETUP_SETUP       ( 1 <<  4 )
#define TOSHIBA_TC90400GDRS_SETUP_TIME_INIT   ( 1 <<  5 )
#define TOSHIBA_TC90400GDRS_SETUP_TIMER_SETUP ( 1 <<  6 )

#define TOSHIBA_TC90400GDRS_SETUP_ALL         0xffffffff
#endif

#ifdef TOSHIBA_TC90400GDRS_SETUP_DEBUG
static const u32 toshiba_tc90400gdrs_setup_debug_flag =
   (TOSHIBA_TC90400GDRS_SETUP_NONE | TOSHIBA_TC90400GDRS_SETUP_INFO |
    TOSHIBA_TC90400GDRS_SETUP_WARN | TOSHIBA_TC90400GDRS_SETUP_EROR |
    TOSHIBA_TC90400GDRS_SETUP_EFWFU | TOSHIBA_TC90400GDRS_SETUP_SETUP |
    TOSHIBA_TC90400GDRS_SETUP_TIME_INIT | TOSHIBA_TC90400GDRS_SETUP_TIMER_SETUP);
#endif

#ifdef TOSHIBA_TC90400GDRS_SETUP_DEBUG
#define TOSHIBA_TC90400GDRS_SETUP_DPRINTK(flag,str...) \
        if ( (toshiba_tc90400gdrs_setup_debug_flag) & (flag) ) \
        { \
           char tmp[100]; \
           sprintf( tmp, str ); \
           printk( "%s(%s:%u)::%s", __FUNCTION__, __FILE__, __LINE__, tmp ); \
        }
#else
#define TOSHIBA_TC90400GDRS_SETUP_DPRINTK(flag,str...)
#endif

/* These functions are used for rebooting or halting the machine*/
extern void toshiba_tc90400gdrs_restart(char *command);
extern void toshiba_tc90400gdrs_halt(void);
extern void toshiba_tc90400gdrs_power_off(void);

extern char *prom_getcmdline(void);

static int tc90400_ccfg_toeon = 1;

#ifdef CONFIG_PCI
#undef TC90400_SUPPORT_COMMAND_IO
#undef  TC90400_SUPPORT_PCI_66

unsigned long mips_pci_io_base;
unsigned long mips_pci_io_size;
unsigned long mips_pci_mem_base;
unsigned long mips_pci_mem_size;
/* for legacy I/O, PCI I/O PCI Bus address must be 0 */
unsigned long mips_memory_upper;
void tc90400_pci_setup(void);
#endif

char *toshiba_name = "";

#ifdef CONFIG_PCI
void tc90400_pci_setup(void)
{
	extern unsigned int tc90400_get_mem_size(void);

	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2, "-\n");

	mips_memory_upper = tc90400_get_mem_size() << 20;
	mips_memory_upper += KSEG0;
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2,
				       "0x%08lx=mips_memory_upper\n",
				       mips_memory_upper);
	mips_pci_io_base = TC90400_PCIIO;
	mips_pci_io_size = TC90400_PCIIO_SIZE;
	mips_pci_mem_base = TC90400_PCIMEM;
	mips_pci_mem_size = TC90400_PCIMEM_SIZE;

	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_base\n",
				       mips_pci_io_base);
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_size\n",
				       mips_pci_io_size);
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_base\n",
				       mips_pci_mem_base);
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_size\n",
				       mips_pci_mem_size);
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2,
				       "0x%08lx=mips_io_port_base",
				       mips_io_port_base);

	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_PCI2, "+\n");
}
#endif /* CONFIG_PCI */

void toshiba_tc90400gdrs_restart(char *command)
{
	printk(KERN_NOTICE "System Rebooting...\n");

	cli();
	flush_cache_all();
	write_c0_wired(0);
	__asm__ __volatile__("jr\t%0"::"r"(0xbfc00000));

	/* no return */
}

void toshiba_tc90400gdrs_halt(void)
{
	printk(KERN_NOTICE "System Halted\n");
	cli();
	while (1) {
		asm_wait();
	}
	/* no return */
}

void toshiba_tc90400gdrs_power_off(void)
{
	toshiba_tc90400gdrs_halt();
	/* no return */
}

int dpsw_check(void)
{
        unsigned long dpsw;
#ifdef CONFIG_TOSHIBA_TC90400_QFP
	dpsw = (TC90400_RD08(TC90400GDRS_FPGA0_DIPSW) & 0x3);
#else
	dpsw = (TC90400_RD16(TC90400GDRS_FPGA0_DIPSW) & 0x3);
#endif
        return dpsw;
}

void __init tc90400_gdrs_setup(void)
{
	vu32 cp0_config;
	char *argptr;
        unsigned long temp_reg;
	unsigned long cfini;

	printk("CPU is %s\n", toshiba_name);

	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       "-\n");
        cfini = TC90400_RD(TC90400_CFINI);
#ifndef CONFIG_TOSHIBA_TC90400_REV1
        /* Check TX49timer interrupt */
        if (!(cfini & TC90400_CFINI_TX49_TMRINT)) { /* 0:enable */
            printk(KERN_WARNING "WARNING, TX49 Timer Interrupt Enable!!\n");
        }
#endif /* #ifndef CONFIG_TOSHIBA_TC90400_REV1 */
 
	/* f/w leaves this on at startup */
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       ":Clearing STO_ERL.\n");
	clear_c0_status(ST0_ERL);

	/* enable caches */
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       ":Enabling TX49_CONF_IC,TX49_CONF_DC.\n");
	cp0_config = read_c0_config();
	cp0_config = cp0_config & ~(TX49_CONF_IC | TX49_CONF_DC);
	write_c0_config(cp0_config);

#ifdef TOSHIBA_TC90400GDRS_SETUP_DEBUG
	{
		extern void dump_cp0(char *);
		dump_cp0("tc90400_gdrs_setup");
	}
#endif
	/* setup share pin */
        temp_reg = TC90400_CFSH0_I2C0 | TC90400_CFSH0_I2C1;
	TC90400_WR(TC90400_CFSH0, (TC90400_RD(TC90400_CFSH0)|temp_reg));
        temp_reg = (TC90400_CFSH1_INT0|TC90400_CFSH1_INT1|TC90400_CFSH1_SIO0);
	TC90400_WR(TC90400_CFSH1, (TC90400_RD(TC90400_CFSH1)|temp_reg));

#ifdef CONFIG_TOSHIBA_TC90400_QFP
        if (!(cfini & TC90400_CFINI_EBIF_8BITS)) { /* 0:16bit */
            printk(KERN_WARNING "WARNING, EBIF bit width selection 16bit!!\n");
        }
        /* Main board LED6-9 ON */
        TC90400GDRS_WR08(TC90400GDRS_FPGA1_LED69,0x0f);
        /* PIO16 reset control for I2C */
	TC90400GDRS_WR08(TC90400GDRS_FPGA0_PIO1623_DIR,TC90400_RD08(TC90400GDRS_FPGA0_PIO1623_DIR)|0x01); /* PIO16 OUTPUT */
	TC90400GDRS_WR08(TC90400GDRS_FPGA0_PIO1623_OUT,TC90400_RD08(TC90400GDRS_FPGA0_PIO1623_OUT)|0x01); /* PIO16 High */
        /* FPGA revision */
        printk("ChipBoard Chip&FPGA Revision:0x%02x\n",
	       TC90400_RD08(TC90400GDRS_FPGA0_REVISION));
        printk("MainBoard Type & Revision:0x%02x\n",
                TC90400_RD08(TC90400GDRS_FPGA1_BASE));
        /* FPGA0 interrupt mask */
	TC90400GDRS_WR08(TC90400GDRS_FPGA0_INT_MASK,0);
	TC90400GDRS_WR08(TC90400GDRS_FPGA0_INT_LEVEL,0x0); /* ALL:L */
	TC90400GDRS_WR08(TC90400GDRS_FPGA0_INT_CLEAR,0);
        /* FPGA1 interrupt mask */
	TC90400GDRS_WR08(TC90400GDRS_FPGA1_INT_MASK1,0);
	TC90400GDRS_WR08(TC90400GDRS_FPGA1_INT_MASK2,0);
	TC90400GDRS_WR08(TC90400GDRS_FPGA1_INT_CLEAR1,0);
	TC90400GDRS_WR08(TC90400GDRS_FPGA1_INT_CLEAR2,0);
#else
        if (cfini & TC90400_CFINI_EBIF_8BITS) { /* 1:8bit */
            printk(KERN_WARNING "WARNING, EBIF bit width selection 8bit!!\n");
        }
        /* Main board LED6-9 ON */
        TC90400GDRS_WR16(TC90400GDRS_FPGA1_LED69,0x0f);
        /* PIO16 reset control for I2C */
	TC90400GDRS_WR16(TC90400GDRS_FPGA0_PIO1623_DIR,TC90400_RD16(TC90400GDRS_FPGA0_PIO1623_DIR)|0x01); /* PIO16 OUTPUT */
	TC90400GDRS_WR16(TC90400GDRS_FPGA0_PIO1623_OUT,TC90400_RD16(TC90400GDRS_FPGA0_PIO1623_OUT)|0x01); /* PIO16 High */
        /* FPGA revision */
        printk("ChipBoard Chip&FPGA Revision:0x%02x\n",
	       TC90400_RD16(TC90400GDRS_FPGA0_REVISION)&0x00ff);
        printk("MainBoard Type & Revision:0x%04x\n",
                TC90400_RD16(TC90400GDRS_FPGA1_BASE));
        /* FPGA0 interrupt mask */
	TC90400GDRS_WR16(TC90400GDRS_FPGA0_INT_MASK,0);
	TC90400GDRS_WR16(TC90400GDRS_FPGA0_INT_LEVEL,0x0); /* ALL:L */
	TC90400GDRS_WR16(TC90400GDRS_FPGA0_INT_CLEAR,0);
        /* FPGA1 interrupt mask */
	TC90400GDRS_WR16(TC90400GDRS_FPGA1_INT_MASK,0);
	TC90400GDRS_WR16(TC90400GDRS_FPGA1_INT_CLEAR,0);
#endif /* #ifdef CONFIG_TOSHIBA_TC90400_QFP */

	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       "+\n");


	mips_io_port_base = KSEG1 + TC90400GDRS_ISA_IO_OFFSET;
	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       ":mips_io_port_base=0x%08lx\n",
				       mips_io_port_base);

	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       ":Resource\n");
	ioport_resource.start = 0;
	ioport_resource.end = 0xffffffff;
	iomem_resource.start = 0;
	iomem_resource.end = 0xffffffff;


	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       ":ResetRoutines\n");
	_machine_restart = toshiba_tc90400gdrs_restart;
	_machine_halt = toshiba_tc90400gdrs_halt;
	_machine_power_off = toshiba_tc90400gdrs_power_off;

#ifdef CONFIG_PCI
	/* CCFG */
	/* enable Timeout BusError */
	if (tc90400_ccfg_toeon)
		TC90400_WR(TC90400_GBTOUT, TC90400_GBTOUT_ON);

	tc90400_pci_setup();
#endif

#ifdef CONFIG_SERIAL_TC90400_CONSOLE
        argptr = prom_getcmdline();
        if (strstr(argptr, "console=") == NULL) {
                strcat(argptr, " console=ttyS0,38400");
        }
#endif

#ifdef CONFIG_IP_PNP
        argptr = prom_getcmdline();
        if (strstr(argptr, "ip=") == NULL) {
                strcat(argptr, " ip=any");
        }
#endif

#ifdef CONFIG_ROOT_NFS
        argptr = prom_getcmdline();
        if (strstr(argptr, "root=") == NULL) {
                strcat(argptr, " root=/dev/nfs rw");
        }
#endif

	TOSHIBA_TC90400GDRS_SETUP_DPRINTK(TOSHIBA_TC90400GDRS_SETUP_SETUP,
				       "+\n");
}

/* tick(TMR0) timer check */
void timer_interval_check_init(void)
{
#ifdef CONFIG_TOSHIBA_TC90400_QFP
 	TC90400GDRS_WR08(TC90400GDRS_FPGA0_PIO07_DIR,TC90400_RD08(TC90400GDRS_FPGA0_PIO07_DIR)|0x01); /* PIO0 OUTPUT */
#else
 	TC90400GDRS_WR16(TC90400GDRS_FPGA0_PIO07_DIR,TC90400_RD16(TC90400GDRS_FPGA0_PIO07_DIR)|0x01); /* PIO0 OUTPUT */
#endif
}

void timer_interval_check(void)
{
        static unsigned char i = 0;
        /* PIO0 ON/OFF */
#ifdef CONFIG_TOSHIBA_TC90400_QFP
	TC90400GDRS_WR08(TC90400GDRS_FPGA0_PIO07_OUT,(i%2));
#else
	TC90400GDRS_WR16(TC90400GDRS_FPGA0_PIO07_OUT,(i%2));
#endif
	i++;
}
