/*
 * linux/arch/mips/zboot/common/tc90400_uart.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/******************************************************************************
 * (C) Copyright    Toshiba Corporation 2004
 ******************************************************************************
 * PROJECT              :   CTODLinuxMisc
 * AUTHOR               :   Vadivelan.M.
 * MODULE NAME          :   CTODLinuxCompKernel
 * APPROVED BY          :   Not  Yet Approved
 * DATE                 :   05/11/04
 * DEVELOPED BY         :   SoCrates Software India Pvt Ltd.
 * DESCRIPTION          :   This file contains the source code for initializing
 *                          the serial controller and writing characters to the
 *                          transmit FIFO.
 * ****************************************************************************
*/
#include "../include/tc90400.h"


#define reg_address(_address_)	(*((volatile unsigned long *)(_address_))) 
/* Serial Registers */
#define SILCR	        (TC90400_SIO0_REG+0x00)
#define SIDICR	        (TC90400_SIO0_REG+0x04)
#define SIDISR	        (TC90400_SIO0_REG+0x08)
#define SICISR	        (TC90400_SIO0_REG+0x0c)
#define SIFCR	        (TC90400_SIO0_REG+0x10)
#define SIFLCR	        (TC90400_SIO0_REG+0x14)
#define SIBGR	        (TC90400_SIO0_REG+0x18)
#define SITFIFO	        (TC90400_SIO0_REG+0x1c)
#define SIRFIFO	        (TC90400_SIO0_REG+0x20)

void serial_init(void)
{
	unsigned int uiCount;
    
	reg_address(SILCR) = (unsigned long)(0x0020);
	reg_address(SIBGR) = (unsigned long)(0x0021); /* 384kbps */

	reg_address(SIFCR) |= (unsigned long)(0x0007); /* reset */

    /*  wait loop  */
    for (uiCount = 0; uiCount < 0x10000; uiCount++);

   	/* Clear FIFO reset */ 	
	reg_address(SIFCR) = 0; 
    
	/* No interrupts enabled*/
	reg_address(SIDICR) = 0; 

	reg_address(SIFLCR) = 0;

	/* set Receive and transmit request trigger level to 1 byte & 1 byte resp.*/
	reg_address(SIFCR) |= (unsigned long)(0x0000);
	reg_address(SIDISR) = 0;

	/* Making sure that Status Change Interrupt Status Register (SICISR) is
	 cleared*/
	reg_address(SICISR) = 0;

    return ;
}
void tc90400_putc(unsigned char ch)
{
	if (ch == '\n') {

	/* wait till Transmit FIFO is empty*/
	while (!((reg_address(SIDISR) >> 8) & 1));
	
	/* Transmit data*/
	reg_address(SITFIFO) = '\r'; 

	}   
	/* wait till Transmit FIFO is empty*/
	while (!((reg_address(SIDISR) >> 8) & 1));

	/* Transmit data*/
	reg_address(SITFIFO) = ch; 

	return;
}

void serial_putc(void *com_port, unsigned char ch)
{
	tc90400_putc(ch);
	return;
}

unsigned char
serial_getc(void *com_port)
{
    return '\n';
}

int
serial_tstc(void *com_port)
{
    return 1;
}




