/*
 * linux/arch/mips/zboot/include/tc90400.h : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#if !defined(_TC90400_H_)
#define _TC90400_H_

#define K1BASE 0xa0000000

/**************** MPIF registers ****************/
#define MPIF_BASE_ADDR		(K1BASE + 0x1e020000)

#define MPIF_ERCCR0_ADDR	(MPIF_BASE_ADDR + 0x0000)
#define MPIF_ERCCR1_ADDR	(MPIF_BASE_ADDR + 0x0004)
#define MPIF_ERCCR2_ADDR	(MPIF_BASE_ADDR + 0x0008)
#define MPIF_ERCCR3_ADDR	(MPIF_BASE_ADDR + 0x000c)

/**************** ZTEST registers ****************/
#define ZTEST_BASE_ADDR		(K1BASE + 0x1E01C000)

#define ZTEST_CFSH0_ADDR 	(ZTEST_BASE_ADDR + 0x0004)
#define ZTEST_CFSH1_ADDR 	(ZTEST_BASE_ADDR + 0x0008)
#define ZTEST_CFSH2_ADDR 	(ZTEST_BASE_ADDR + 0x000C)
#define ZTEST_CFSH3_ADDR	(ZTEST_BASE_ADDR + 0x002C)


/****************** CG registers *****************/
#define CG_BASE_ADDR		(K1BASE + 0x17500000)

#define CG_STCOCR_ADDR		(CG_BASE_ADDR + 0x0000)
#define CG_COCR_ADDR		(CG_BASE_ADDR + 0x0004)
#define CG_OMCR_ADDR		(CG_BASE_ADDR + 0x0008)
#define CG_IVCCCR_ADDR		(CG_BASE_ADDR + 0x000c)
#define CG_VCOCR_ADDR		(CG_BASE_ADDR + 0x0010)
#define CG_AOSCCR_ADDR		(CG_BASE_ADDR + 0x0014)
#define CG_IAOSCFCR_ADDR	(CG_BASE_ADDR + 0x0018)
#define CG_VDCXOCR0_ADDR	(CG_BASE_ADDR + 0x001c)
#define CG_VDCXOCR1_ADDR	(CG_BASE_ADDR + 0x0020)
#define CG_ADCXOCR0_ADDR	(CG_BASE_ADDR + 0x0024)
#define CG_ADCXOCR1_ADDR	(CG_BASE_ADDR + 0x0028)
#define CG_INTMCR_ADDR		(CG_BASE_ADDR + 0x0048)


/**************** MBUS registers *****************/
#define MBUS_BASE_ADDR		(K1BASE + 0x17420000)

#define MBUS_BEM_ADDR		(MBUS_BASE_ADDR + 0x0000)
#define MBUS_BEA_ADDR		(MBUS_BASE_ADDR + 0x0008)
#define MBUS_BEC_ADDR		(MBUS_BASE_ADDR + 0x0010)
#define MBUS_AMPMAXLAT_ADDR	(MBUS_BASE_ADDR + 0x0020)
#define MBUS_ASPMAXLAT_ADDR	(MBUS_BASE_ADDR + 0x0028)
#define MBUS_GFXMAXLAT_ADDR	(MBUS_BASE_ADDR + 0x0030)
#define MBUS_TSPMAXLAT_ADDR	(MBUS_BASE_ADDR + 0x0038)
#define MBUS_RNDPRI_ADDR	(MBUS_BASE_ADDR + 0x0040)
#define MBUS_MEPACON_ADDR	(MBUS_BASE_ADDR + 0x0080)


/***************** VMMS registers ****************/
#define VMMS_BASE_ADDR          (K1BASE + 0x17410000)

#define VMMS_GBUSMAXLAT_ADDR	(VMMS_BASE_ADDR + 0x0200)
#define VMMS_MBUSMAXLAT_ADDR    (VMMS_BASE_ADDR + 0x0204)
#define VMMS_MCMAXLAT_ADDR      (VMMS_BASE_ADDR + 0x0208)
#define VMMS_INTRAMAXLAT_ADDR   (VMMS_BASE_ADDR + 0x020C)
#define VMMS_RNDPRI_ADDR        (VMMS_BASE_ADDR + 0x0210)
#define VMMS_MEMCONFIG_ADDR     (VMMS_BASE_ADDR + 0x0300)
#define VMMS_REFMODE_ADDR       (VMMS_BASE_ADDR + 0x0308)
#define VMMS_MEMACCHAR_ADDR     (VMMS_BASE_ADDR + 0x030C)
#define VMMS_ACTMODE_ADDR       (VMMS_BASE_ADDR + 0x0310)
#define VMMS_COMSET_ADDR        (VMMS_BASE_ADDR + 0x0314)
#define VMMS_RDQWAIT_ADDR       (VMMS_BASE_ADDR + 0x0318)
#define VMMS_GRAT00_ADDR        (VMMS_BASE_ADDR + 0x0400)
#define VMMS_GRAT01_ADDR        (VMMS_BASE_ADDR + 0x0404)
#define VMMS_GRAT02_ADDR        (VMMS_BASE_ADDR + 0x0408)
#define VMMS_GRAT03_ADDR        (VMMS_BASE_ADDR + 0x040C)
#define VMMS_IOMODE_ADDR        (VMMS_BASE_ADDR + 0x0410)
#define VMMS_CLMODE_ADDR        (VMMS_BASE_ADDR + 0x0414)
#define VMMS_DLLMODE_ADDR       (VMMS_BASE_ADDR + 0x0418)

#define VMMS_MIPADR		(VMMS_BASE_ADDR + 0x0504)
#define VMMS_MCAADR		(VMMS_BASE_ADDR + 0x0508)


/***************** SIO registers ****************/
#define TC90400_SIO0_REG	(K1BASE + 0x1E010000)
#define TC90400_SIO1_REG	(K1BASE + 0x1E010040)
#define TC90400_SIO2_REG	(K1BASE + 0x1E010080)


/*********** Parallel I/O Port register **********/
#define TC90400_XPIODO_ADDR	(K1BASE + 0x1E008000)
#define TC90400_XPIODI_ADDR	(K1BASE + 0x1E008004)
#define TC90400_XPIODIR_ADDR	(K1BASE + 0x1E008008)


#endif	/* _TC90400_H_ */
