/*
 * linux/arch/mips/zboot/include/tc90400gdrs.h : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#if !defined(_TC90400GDRS_H_)
#define _TC90400GDRS_H_

/* memory mapping  mode:DSW[7:6]="10" is not support */
#define CE0_ADDR	0x1f000000
#define CE1_ADDR	0x18000000
#define CE2_ADDR	0x19000000
#define CE3_ADDR	0x19000000

/* external input clock */
#define XT_CLCK	27000000

/***************** Register default values ******************/

/* EBIF register */
#define RBA_MASK		0x07f00000

#define RBA(VAL)		( (VAL & RBA_MASK))	/* ROM control base address */
#define RPM(VAL)		( (VAL & 0x03) << 18)	/* ROM control page mode page size */
#define RPWT(VAL)		( (VAL & 0x03) << 16)	/* ROM control page mode wait time */ 
#define RWT(VAL)		( (VAL & 0x0f) << 12)	/* ROM control normal mode wait time */
#define RCS(VAL)		( (VAL & 0x0f) << 8)	/* ROM controlchannel size 0=1MB 1=2MB 2=4MB 3=8MB 4=16MB 5=32MB 6=64MB 7=128MB */
#define BUS16(VAL)		( (VAL & 0x01) << 7)	/* ROM control bus width 0=8bit 1=16bit */
#define RDY(VAL)		( (VAL & 0x01) << 6)	/* ROM control ready input active */
#define RBC(VAL)		( (VAL & 0x01) << 5)	/* ROM byte control */
#define RME(VAL)		( (VAL & 0x01) << 3)	/* ROM control master enable */
#define RSHWT(VAL)		( (VAL & 0x07) << 0)	/* ROM control setup/hold wait time */

#ifdef CONFIG_TOSHIBA_TC90400_QFP

/* ----- QFP ----- */
#if (BOOT == 1)		/* booting from NOR FLASH */
  #ifdef CONFIG_MTD_NAND_TC90400NDFMC
    #define ERCCR0_DEFAULT (RBA(CE0_ADDR)|RPM(0)|RPWT(0)|RWT(6)|RCS(4)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(0))
    #define ERCCR1_DEFAULT 0	/* Used NAND FLASH */
    #define ERCCR2_DEFAULT (RBA(CE2_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(0)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
    #define ERCCR3_DEFAULT (RBA(CE1_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(4)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
  #else	/* #ifdef CONFIG_MTD_NAND_TC90400NDFMC */
    #define ERCCR0_DEFAULT (RBA(CE0_ADDR)|RPM(0)|RPWT(0)|RWT(6)|RCS(4)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(0))
    #define ERCCR1_DEFAULT (RBA(CE1_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(4)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
    #define ERCCR2_DEFAULT (RBA(CE2_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(0)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
    #define ERCCR3_DEFAULT (RBA(CE3_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(0)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
  #endif	/* #ifdef CONFIG_MTD_NAND_TC90400NDFMC */
#else	/* #if (BOOT == 1) : booting from NAND FLASH */
    #define ERCCR0_DEFAULT 0	/* Used NAND FLASH */
    #define ERCCR1_DEFAULT (RBA(CE1_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(4)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
    #define ERCCR2_DEFAULT (RBA(CE2_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(0)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
    #define ERCCR3_DEFAULT (RBA(CE3_ADDR)|RPM(0)|RPWT(0)|RWT(2)|RCS(0)|BUS16(0)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
#endif	/* #if (BOOT == 1) */

#else /* BGA(#2) */

/* ----- BGA ----- */
#if (BOOT == 1)		/* booting from NOR FLASH */
  #ifdef CONFIG_MTD_NAND_TC90400NDFMC
    #define ERCCR0_DEFAULT (RBA(CE0_ADDR)|RPM(0)|RPWT(0)|RWT(8)|RCS(4)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(0))
    #define ERCCR1_DEFAULT 0	/* Used NAND FLASH */
    #define ERCCR2_DEFAULT (RBA(CE2_ADDR)|RPM(0)|RPWT(1)|RWT(4)|RCS(0)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(2))
    #define ERCCR3_DEFAULT (RBA(CE1_ADDR)|RPM(0)|RPWT(0)|RWT(1)|RCS(4)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
  #else	/* #ifdef CONFIG_MTD_NAND_TC90400NDFMC */
    #define ERCCR0_DEFAULT (RBA(CE0_ADDR)|RPM(0)|RPWT(0)|RWT(8)|RCS(4)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(0))
    #define ERCCR1_DEFAULT (RBA(CE1_ADDR)|RPM(0)|RPWT(0)|RWT(1)|RCS(4)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
    #define ERCCR2_DEFAULT (RBA(CE2_ADDR)|RPM(0)|RPWT(0)|RWT(1)|RCS(0)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
    #define ERCCR3_DEFAULT (RBA(CE3_ADDR)|RPM(0)|RPWT(1)|RWT(4)|RCS(0)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(2))
  #endif	/* #ifdef CONFIG_MTD_NAND_TC90400NDFMC */
#else	/* #if (BOOT == 1) : booting from NAND FLASH */
  #define ERCCR0_DEFAULT 0	/* Used NAND FLASH */
  #define ERCCR1_DEFAULT (RBA(CE1_ADDR)|RPM(0)|RPWT(0)|RWT(1)|RCS(4)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
  #define ERCCR2_DEFAULT (RBA(CE2_ADDR)|RPM(0)|RPWT(0)|RWT(1)|RCS(0)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(1))
  #define ERCCR3_DEFAULT (RBA(CE3_ADDR)|RPM(0)|RPWT(1)|RWT(4)|RCS(0)|BUS16(1)|RDY(0)|RBC(0)|RME(1)|RSHWT(2))
#endif	/* #if (BOOT == 1) */

#endif /* CONFIG_TOSHIBA_TC90400_QFP */

/* ZTEST register */
#define _ZTEST_CFSH3_DEFAULT_VAL  	0x00000000

/***************** CHIP BOARD FPGA registers ****************/
#define TC90400_CHIP_FPGA       (CE1_ADDR + 0xa0000000 + 0x00008000)
#define TC90400_PIO_DIR03       (TC90400_CHIP_FPGA+0x24)
#define TC90400_PIO_DO03        (TC90400_CHIP_FPGA+0x20)


#endif	/* _TC90400GDRS_H_ */
