/*
 * linux/arch/mips/zboot/tc90400/head.S : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
 * arch/mips/kernel/head.S
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 1995 Waldorf Electronics
 * Written by Ralf Baechle and Andreas Busse
 * Copyright (C) 1995 - 1999 Ralf Baechle
 * Copyright (C) 1996 Paul M. Antoine
 * Modified for DECStation and hence R3000 support by Paul M. Antoine
 * Further modifications by David S. Miller and Harald Koerfgen
 * Copyright (C) 1999 Silicon Graphics, Inc.
 *
 * Head.S contains the MIPS exception handler and startup code.
 *
 **************************************************************************
 *  9 Nov, 2000.
 *  Added Cache Error exception handler and SBDDP EJTAG debug exception.
 *
 *  Kevin Kissell, kevink@mips.com and Carsten Langgaard, carstenl@mips.com
 *  Copyright (C) 2000 MIPS Technologies, Inc.  All rights reserved.
 **************************************************************************
 */
#include <linux/config.h>
#include <linux/threads.h>

#include <asm/asm.h>
#include <asm/cacheops.h>
#include <asm/mipsregs.h>
#include <asm/offset.h>
#include <asm/cachectl.h>
#include <asm/regdef.h>

#include "../include/tc90400.h"
#ifdef CONFIG_TOSHIBA_TC90400GDRS
#include "../include/tc90400gdrs.h"
#endif
#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
#include "../include/tc90400qfp_board01.h"
#endif


#define IndexInvalidate_I       0x00
#define IndexWriteBack_D        0x01

/*--- USB Controller Reg. Value ---*/
/*
 * HcCommandStatus (cmdstatus) register masks
 */
#define OHCI_OCR                0x08
#define OHCI_HCR                1
/* pre-shifted values for HCFS */
#define OHCI_USB_RESET          0
#define OHCI_USB_OPER           0x0080
/* roothub.status bits */
#define RH_HS_LPSC              0x00010000

/* macro for 32 bit register write */
#define W_REG_32(PAR1, PAR2) \
	la	t0, PAR1; \
	li	t1, PAR2; \
	sw	t1, 0x0(t0) 
		
/* macro for LED access */
#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
#define LEDON(PAR) \
        la	t8, TC90400_XPIODO_ADDR; \
        lw      t9, 8(t8); \
        or      t9, _PIO_LEDMASK_VAL; \
        sw      t9, 8(t8); \
        lw      t9, 0(t8); \
        and     t9, ~_PIO_LEDMASK_VAL; \
        li      t7, PAR; \
        sll     t7, t7,13; \
        or      t9, t7; \
        sw      t9, 0(t8)
#endif /* CONFIG_TOSHIBA_TC90400QFP_BOARD01 */

#ifdef CONFIG_TOSHIBA_TC90400GDRS
#ifdef CONFIG_TOSHIBA_TC90400_QFP
#define LEDON(PAR) \
        la	t8, TC90400_PIO_DO03; \
        lb      t9, 4(t8); \
        or      t9, 0x0F;\
        sb      t9, 4(t8); \
        lb      t9, 0(t8); \
        and     t9, ~0x0F; \
        or      t9, PAR; \
        sb      t9, 0(t8)
#else
#define LEDON(PAR) \
        la	t8, TC90400_PIO_DO03; \
        lh      t9, 4(t8); \
        or      t9, 0x0F;\
        sh      t9, 4(t8); \
        lh      t9, 0(t8); \
        and     t9, ~0x0F; \
        or      t9, PAR; \
        sh      t9, 0(t8)
#endif /* CONFIG_TOSHIBA_TC90400_QFP */
#endif /* CONFIG_TOSHIBA_TC90400GDRS */

/* delay macro */	
#define GO_WAIT(PAR) \
	li	t2, 0; \
	li	t3, 0; \
	li	t4, PAR; \
	1: \
	addi	t2, t2, 1; \
	bne	t2, t4, 1b; \
	nop	


	.set noreorder
	.cprestore
	LEAF(start)
start:
	/* Here's the TC90400 specific stuff */

	/* set up ZTEST pin share */
	la	t0, ZTEST_CFSH3_ADDR
	li	t1, _ZTEST_CFSH3_DEFAULT_VAL
	sw	t1, 0x00(t0)
#ifdef CONFIG_TOSHIBA_TC90400_QFP
	la	t0, ZTEST_CFSH0_ADDR
	lw	t1, 0x00(t0)
	li	t2, 0x00400000		/* Enable USB port1 (USB Power) */
        or     	t1, t1, t2
	sw	t1, 0x00(t0)
#endif /* CONFIG_TOSHIBA_TC90400_QFP */

	/* LINUX expects MPIF properly set up */
        la      t0, MPIF_ERCCR0_ADDR
        li      t1, ERCCR0_DEFAULT	/* CS0 */
        sw      t1, 0x00(t0)

        li      t1, ERCCR1_DEFAULT	/* CS1 */
        sw      t1, 0x04(t0)

        li      t1, ERCCR2_DEFAULT	/* CS2 */
        sw      t1, 0x08(t0)

        li      t1, ERCCR3_DEFAULT	/* CS3 */
        sw      t1, 0x0c(t0)

	LEDON(1)

#ifdef CONFIG_TOSHIBA_TC90400_UPDATE
	/* DualBoot proc */
        W_REG_32 (0xbc008008,   OHCI_OCR)       /* Ownership Change Request */
        GO_WAIT(4218)                           /* wait 10msec */

        W_REG_32 (0xbc008004,   0)              /* Reset USB */
        W_REG_32 (0xbc008008,   OHCI_HCR)       /* Reset HC */
        GO_WAIT(4)                              /* wait 10usec */

        LEDON(2)
        W_REG_32 (0xbc008004,   OHCI_USB_RESET) /* Reset USB */
        GO_WAIT(4218)                           /* wait 10msec */


        LEDON(3)
        W_REG_32 (0xbc008004,   OHCI_USB_OPER)  /* start controller operation */
        W_REG_32 (0xbc008050,   RH_HS_LPSC)     /* roothub status */
        li      t0, 0x1fe			/* wait 1msec * 510 */
        li      t1, 1
5:
        GO_WAIT(421)                            /* (wait 1msec) */
        subu    t0, t0, t1
        bne     zero, t0, 5b
        nop     
        nop

        LEDON(4)
        la      t0, 0xbc008054                  /* HC RootHub port status reg. addr. */
        lw      t1, 0(t0)                       /* get rootHub port status */
        GO_WAIT(4218)                           /* wait 10msec */

        andi    t1, t1, 0x0001                  /* Check device inserted bit */
        bne     zero, t1, 4f                    /* jump detect USB device */
        nop
        nop
        LEDON(7)
        la      k0, MASTER_IPL_ADDR             /* ZBOOT IPL Addr. on NorFlash */
        jr      k0                              /* Change IPL */
        nop
        nop

4:
#endif  /* CONFIG_TOSHIBA_TC90400_UPDATE */

#if (BOOT == 1)	/* booting from NOR FLASH */
	/*
         * We're booting from NOR FLASH and no PMON
         * is installed so we have to setup VMMS
         */
	
	/* Initialize the memory controller */
#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
	W_REG_32 (VMMS_MEMCONFIG_ADDR,		0x00000005)	/* SDRAM size & organization - 256 -> 512Mbits, x16 (060713)*/
#else
	W_REG_32 (VMMS_MEMCONFIG_ADDR,		0x00000005)	/* SDRAM size & organization - 512Mbits, x16 */
#endif
	W_REG_32 (VMMS_MEMACCHAR_ADDR,		0x0003057f)	/* DDR SDRAM AC settings */
  	W_REG_32 (VMMS_RDQWAIT_ADDR,		0x00000003)   	/* Set read data wait cycle CAS Latency to 2.5 */
	W_REG_32 (VMMS_IOMODE_ADDR,   		0x00000000)	/* Enable the SSTL2 IO cell */
  	W_REG_32 (VMMS_CLMODE_ADDR,    		0x00000002)   	/* CAS latency to 2.5 */
  	W_REG_32 (VMMS_DLLMODE_ADDR,   		0x00000001)   	/* Set lock address mode to "internal" and set the DLL */
	
	/* Setup of gear ratio value
		 DDR 333 (162MHz)
		 GRAT00=79
		 GRAT01=23
		 GRAT02=36
		 GRAT03=92 */
  	W_REG_32 (VMMS_GRAT00_ADDR,    		0x0000004f)   /* Set gear ratio for "read data strobe" */
  	W_REG_32 (VMMS_GRAT01_ADDR,    		0x00000017)   /* Set gear ratio for "read data strobe mask" */
  	W_REG_32 (VMMS_GRAT02_ADDR,    		0x00000024)   /* Set gear ratio for "clk90wd" */
  	W_REG_32 (VMMS_GRAT03_ADDR,    		0x0000005c)   /* Set gear ratio for "clk90rd" */
	
	GO_WAIT(40)	/* wait 123 clk cycles */

  	/* Initialize the External DDR-SDRAM */
  	W_REG_32 (VMMS_ACTMODE_ADDR,   		0x00000001)	/* Set DDR-SDRAM IF access mode to "external" */
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00dc0000)   	/* Command - NOP and CKE "high" */
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00080400)   	/* Command - PRECHARGE ALL */
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00010000)   	/* Enable the DLL */
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00000163)   	/* Initialy reset DLL */

 	GO_WAIT(10)	/* wait 33 clk cycles */
	
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00080400)	/* Command - PRECHARGE ALL */
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00100000)	/* Command - AUTO REFRESH */
	
	GO_WAIT(1)	/* wait 6 clk cycles */
	
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00100000)   	/* Command - AUTO REFRESH */
  	W_REG_32 (VMMS_COMSET_ADDR,    		0x00000063)   	/* Set the DLL */
  	W_REG_32 (VMMS_ACTMODE_ADDR,   		0x00000000)   	/* Set IF access mode to "normal" */
  	W_REG_32 (VMMS_REFMODE_ADDR,   		0x00000104)   	/* Auto refresh set to 2 times, refresh cycles = 256*/

	W_REG_32 (MBUS_AMPMAXLAT_ADDR,		0x0000003f)
  	W_REG_32 (MBUS_ASPMAXLAT_ADDR,		0x0000003f)
  	W_REG_32 (MBUS_GFXMAXLAT_ADDR,		0x0000000f)
  	W_REG_32 (MBUS_TSPMAXLAT_ADDR,		0x0000003f)
  	W_REG_32 (MBUS_RNDPRI_ADDR,		0x84441122)
	
	/* Set bus arbitration */
  	W_REG_32 (VMMS_GBUSMAXLAT_ADDR,		0x0000003f)	/* GBUS I/F */
  	W_REG_32 (VMMS_MBUSMAXLAT_ADDR, 	0x0000003f)   	/* MBUS I/F */
  	W_REG_32 (VMMS_MCMAXLAT_ADDR,   	0x00000008)   	/* MCIF */
  	W_REG_32 (VMMS_INTRAMAXLAT_ADDR,	0x0000000f)   	/* INTRAIF */
  	W_REG_32 (VMMS_RNDPRI_ADDR,      	0x88444421)   	/* Priority */
		
	/* Setup the MEP area */
	la	t0, VMMS_MIPADR	/* MeP initialization Proguram address offset */
  	li    	t1, 0x00800000        	/* write to MIPADR the value 0x00800000 */
  	sw    	t1, 0(t0)	
	la    	t0, VMMS_MCAADR		/* MeP cache address offset */
  	li    	t1, 0x00A00000		/* write to MCAADR the value 0x00A00000 */
  	sw    	t1, 0(t0)
	
  	/* Dummy read to finalize the VMMS setting */
  	la	t0, 0x80000000
  	li    	t1, 0x0
	sw	t1, 0(t0)
	nop
	sw	t1, 4(t0)
	nop
	lw	t1, 0(t0)
	nop
	lw	t1, 4(t0)
	nop

#else	/* (BOOT==1) */

	/*
         * if booting from NAND FLASH VMMS is already set up by IPL code
         * and we're already running from DDR RAM. Setting up VMMS
         * again would kill ourselves
         */

#endif	/* (BOOT==1) */

	LEDON(2)

	/* clear arguments */
	li	a0, 0x00
	li	a1, 0x00
	li	a2, 0x00
	li	a3, 0x00
	
	/* TC90400 specific end */	

	bal	locate
	nop
locate:
	la	s8, FLASH_LOAD_ADDR	/* 0xbfc00000 */
	la	sp, (.stack + 8192)

	move	s0, a0		/* Save boot rom start args */
	move	s1, a1
	move	s2, a2
	move	s3, a3

	la	a0, start	/* Where we were linked to run */

	move	a1, s8
	la	a2, _edata
	subu	t1, a2, a0
	srl	t1, t1, 2

	/* copy text section */
	li	t0, 0
1:	lw	v0, 0(a1)
	nop
	sw	v0, 0(a0)
	xor	t0, t0, v0
	addu	a0, 4
	bne	a2, a0, 1b
	addu	a1, 4

	/* Clear BSS */
	la	a0, _edata
	la	a2, _end
2:	sw	zero, 0(a0)
	bne	a2, a0, 2b
	addu	a0, 4

	LEDON(4)

	/* push the D-Cache and invalidate I-Cache */
	li	k0, 0x80000000  # start address
	li	k1, 0x80002000  # end address (8KB I-Cache&D-Cache)
	subu	k1, 128

1:
	.set mips3
	cache	IndexWriteBack_D, 0(k0)
	cache	IndexWriteBack_D, 32(k0)
	cache	IndexWriteBack_D, 64(k0)
	cache	IndexWriteBack_D, 96(k0)
	cache	IndexInvalidate_I, 0(k0)
	cache	IndexInvalidate_I, 32(k0)
	cache	IndexInvalidate_I, 64(k0)
	cache	IndexInvalidate_I, 96(k0)
	.set mips0

	bne	k0, k1, 1b
	addu	k0, k0, 128
	/* done */

	LEDON(3)

	move	a0, s8		     /* load address */
	move	a1, t1               /* length in words */
	move	a2, t0               /* checksum */
	move	a3, sp

	la	ra, 1f
	la	k0, decompress_kernel
	jr	k0
	nop
1:

	move	a0, s0
	move	a1, s1
	move	a2, s2
	move	a3, s3
	li	k0, KERNEL_ENTRY
	jr	k0
	nop
3:
	b 3b
	END(start)

	LEAF(udelay)
udelay:
	END(udelay)


	LEAF(FlushCache)
	li	k0, 0x80000000  # start address
	li	k1, 0x80002000  # end address (8KB I-Cache&D-Cache)
	subu	k1, 128

1:
	.set mips3
	cache	IndexWriteBack_D, 0(k0)
	cache	IndexWriteBack_D, 32(k0)
	cache	IndexWriteBack_D, 64(k0)
	cache	IndexWriteBack_D, 96(k0)
	cache	IndexInvalidate_I, 0(k0)
	cache	IndexInvalidate_I, 32(k0)
	cache	IndexInvalidate_I, 64(k0)
	cache	IndexInvalidate_I, 96(k0)
	.set mips0

	bne	k0, k1, 1b
	addu	k0, k0, 128
	jr	ra
	nop
	END(FlushCache)

	.comm .stack,4096*2,4
