/*
 * linux/drivers/char/tc90400_drv.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * User-space enhance driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your potion) any lator version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/mm.h>
#include <linux/wait.h>
#include <asm/irq.h>
#include <asm/uaccess.h>

#include <asm/tc90400/tc90400.h>
#include <asm/tc90400/tc90400_drv.h>

#define DRIVER_VERSION  "TC90400 user-space enhance driver 1.5\n"

#define DBG_TC90400		0
#define DBG_COMMAND             1
#define DEBUG_USE               0

#if DBG_TC90400
#define DBG_TC90400DRV_PRINT(x...) {printk("[TC90400_DRV: "); printk(x); printk("]\n");}
#else
#define DBG_TC90400DRV_PRINT(x...)
#endif



/****************************************/
/*     Interrupt handle driver          */
/****************************************/

/* declaration wait queues */
/* DECLARE_WAIT_QUEUE_HEAD(intdrv_wait[*]); */
static	wait_queue_head_t intdrv_wait[16] = {
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[0]),  /* dummy */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[1]),  /* IRQ15  GMBR */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[2]),  /* IRQ16  GFX */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[3]),  /* IRQ17  VIOU */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[4]),  /* IRQ9   EXTINT0(QFP_BOARD01) */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[5]),  /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[6]),  /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[7]),  /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[8]),  /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[9]),  /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[10]), /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[11]), /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[12]), /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[13]), /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[14]),  /* reserved */
  __WAIT_QUEUE_HEAD_INITIALIZER(intdrv_wait[15])  /* reserved */
};

/* Association for device minor no. and IRQ */
static unsigned int  minor_irq[16] = {
  0,   /* dummy */
  TC90400_IRQ_GMBR,  /* 1: IRQ15  GMBR */
  TC90400_IRQ_GFX,   /* 2: IRQ16  GFX */
  TC90400_IRQ_VIOU,  /* 3: IRQ17  VIOU */

#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
  TC90400_IRQ_EXT0,  /* 4: IRQ9   EXT0 */
#if DEBUG_USE
  TC90400_IRQ_TMR1,  /* 5: IRQ37  Timer ch.1 --- debug use ---*/
  TC90400_IRQ_TMR2,  /* 6: IRQ38  Timer ch.2 --- debug use ---*/
#else
  0,0,
#endif
  0,0,0,0,0,0,0,0,0
#else /* CONFIG_TOSHIBA_TC90400QFP_BOARD01 */
#if DEBUG_USE
  TC90400_IRQ_EXT2,  /* 4: IRQ11  EXT2 --- debug use ---*/
  TC90400_IRQ_TMR1,  /* 5: IRQ37  Timer ch.1 --- debug use ---*/
  TC90400_IRQ_TMR2,  /* 6: IRQ38  Timer ch.2 --- debug use ---*/
#else
  0,0,0,
#endif /* DEBUG_USE */
  0,0,0,0,0,0,0,0,0
#endif /* CONFIG_TOSHIBA_TC90400QFP_BOARD01 */
};


#define  _INT_NONE           0
#define  _INT_OCCOUR         1
#define  _INT_ABORT          2
#define  _INT_OCCOUR_ACCEPT  3
#define  _INT_ABORT_ACCEPT   4   


static unsigned int  irq_sts[16] = { _INT_NONE };



static void tc90400_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
    DBG_TC90400DRV_PRINT("interrupt %d !", (int)dev_id);
#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
    if (irq == TC90400_IRQ_EXT0) {
      /* Interrupt edge clear */
      TC90400_WR(TC90400_INTSTSC,(TC90400_INTSTSC_EDGECLEAR|TC90400_INTSTSC_EICLR1));
    }
#endif
    irq_sts[(int)dev_id] = _INT_OCCOUR;
    wake_up(&intdrv_wait[(int)dev_id]);
    disable_irq(irq);
}


static unsigned int tc90400_int_poll(struct file *file, poll_table *wait)
{
  unsigned int retval;

  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
  
  DBG_TC90400DRV_PRINT("POLL: interrupt Driver MINOR(%d)", minor);

  poll_wait(file, &intdrv_wait[minor], wait);
  
  disable_irq(minor_irq[minor]);
  DBG_TC90400DRV_PRINT("poll status:%d", irq_sts[minor]);

  if (_INT_OCCOUR == irq_sts[minor]) {
    irq_sts[minor] = _INT_OCCOUR_ACCEPT;
    retval = POLLIN | POLLRDNORM;
  }
  else if (_INT_ABORT == irq_sts[minor]) {
    irq_sts[minor] = _INT_ABORT_ACCEPT;
    retval = POLLIN | POLLRDNORM;
  }
  else {
    irq_sts[minor] = _INT_NONE;
    retval = TC90400_OK;
  }
  enable_irq(minor_irq[minor]);
  return retval;
}

static ssize_t tc90400_int_read(struct file *file,char *buf,size_t count,loff_t *ppos)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
  DECLARE_WAITQUEUE(wait, current);
  ssize_t retval;
  int flags;

  DBG_TC90400DRV_PRINT("READ: interrupt Driver MINOR(%d)", minor);

  add_wait_queue(&intdrv_wait[minor], &wait);
  current->state = TASK_INTERRUPTIBLE;
  do {
    DBG_TC90400DRV_PRINT("read intr status:%d", irq_sts[minor]);
    if ((_INT_OCCOUR_ACCEPT == irq_sts[minor]) ||
	(_INT_OCCOUR        == irq_sts[minor])) {
      retval = TC90400_OK;
      break;
    }
    if ((_INT_ABORT_ACCEPT == irq_sts[minor]) ||
	(_INT_ABORT        == irq_sts[minor])) {
      retval = _INT_ABORT;
      break;
    }
    if (file->f_flags & O_NONBLOCK) {
      retval = -EAGAIN;
      break;
    }
    if (signal_pending(current)) {
      retval = -ERESTARTSYS;
      break;
    }
    schedule();
  } while (1);

  DBG_TC90400DRV_PRINT("read ret(%d)", retval);
  save_flags(flags);
  cli();
  irq_sts[minor] = _INT_NONE;
  current->state = TASK_RUNNING;
  remove_wait_queue(&intdrv_wait[minor], &wait);
  restore_flags(flags);

  return retval;
}

static int tc90400_int_ioctl(struct inode *inode,struct file *file,unsigned int cmd,
		            unsigned long arg)
{
  int flags;
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);

  switch (cmd) {
  case IOCTL_INTDRV_INTENABLE:
    DBG_TC90400DRV_PRINT("IOCTL_INTDRV_INTENABLE Driver");

    wbflush();       /* flush TXcore's write-buffer */

    enable_irq(minor_irq[minor]);
    break;
  case IOCTL_INTDRV_INTDISABLE:
    DBG_TC90400DRV_PRINT("IOCTL_INTDRV_INTDISABLE Driver");
    disable_irq(minor_irq[minor]);
    break;
  case IOCTL_INTDRV_CANCEL:
    DBG_TC90400DRV_PRINT("IOCTL_INTDRV_CANCEL Driver");
    save_flags(flags);
    cli();
    /* set status flag */
    irq_sts[minor] = _INT_ABORT;
    /* Now do the rest of the actions */
    wake_up_interruptible(&intdrv_wait[minor]);
    restore_flags(flags);
    break;
  default:
    printk(KERN_ERR "tc90400_ioctl : interrupt device invalid operation %d.\n", cmd);
    return -EPERM;
  }
  return TC90400_OK;
}

static int tc90400_int_open(struct inode *inode, struct file *file)
{
#define ISRNAME "tc90400_interrupt_  "

  int  retval, flags;
  unsigned int minor = MINOR(inode->i_rdev);
  char  *name = ISRNAME;

  DBG_TC90400DRV_PRINT("OPEN: interrupt Driver MINOR(%d)", minor);

  if (minor_irq[minor] == 0) {
    printk(KERN_ERR "tc90400 : Device %d does not define IRQ.\n", minor);
    return -EIO;
  }

#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
  if (minor_irq[minor] == TC90400_IRQ_EXT0) {
    /* Set IRC Control Reg. to EXT0 falling edge detect */
    TC90400_WR(TC90400_INTCONT1, TC90400_RD(TC90400_INTCONT1)|TC90400_INTCONT1_EXT0_F);
    /* Interrupt edge clear */
    TC90400_WR(TC90400_INTSTSC,(TC90400_INTSTSC_EDGECLEAR|TC90400_INTSTSC_EICLR1));
  }
#endif

  /* entry interrupt request */
  name[sizeof(ISRNAME)-3] = '0' | (minor / 10);
  name[sizeof(ISRNAME)-2] = '0' | (minor % 10);
  DBG_TC90400DRV_PRINT("    interrupt Driver name :  %s", name);

  wbflush();       /* flush TXcore's write-buffer */
  
  save_flags(flags);
  cli();
  irq_sts[minor] = _INT_NONE;
  if(request_irq(minor_irq[minor], tc90400_interrupt, SA_INTERRUPT, name, (char*)minor)) {
    printk(KERN_ERR "tc90400 : IRQ %d is not free.\n", minor_irq[minor]);
    retval = -EIO;
  } else {
    disable_irq(minor_irq[minor]);
    wake_up(&intdrv_wait[minor]);
    retval = TC90400_OK;  
  }
  restore_flags(flags);
  return retval;
}

static int tc90400_int_release(struct inode *inode, struct file *file)
{
  unsigned int minor = MINOR(inode->i_rdev);

  DBG_TC90400DRV_PRINT("CLOSE: interrupt Driver MINOR(%d)",minor);
  disable_irq(minor_irq[minor]);
  free_irq(minor_irq[minor] , (char*)minor);
  return TC90400_OK;
}

/****************************************/
/*     Memory map driver                */
/****************************************/

/* Memory map parameters */
static tMapParm map_parm[] = {
   {"SDRAM", 
    TC90400_SDRAM_START_PHYS, TC90400_SDRAM_START_OFF, TC90400_SDRAM_SIZE}
  ,{"MBUS", 
    TC90400_MBUS_START_PHYS, TC90400_MBUS_START_OFF, TC90400_MBUS_SIZE}
  ,{"EXTB1", 
    TC90400_EXTB1_START_PHYS, TC90400_EXTB1_START_OFF, TC90400_EXTB1_SIZE}
  ,{"ZTEST", 
    TC90400_ZTEST_START_PHYS, TC90400_ZTEST_START_OFF, TC90400_ZTEST_SIZE}
  ,{"EXTB2", 
    TC90400_EXTB2_START_PHYS, TC90400_EXTB2_START_OFF, TC90400_EXTB2_SIZE}
};

#define LEN_MAP_PARM     (sizeof(map_parm)/sizeof(tMapParm))
    

static int tc90400_mem_mmap(struct file * file, struct vm_area_struct * vma)
{
    unsigned long off;
    unsigned long size;
    unsigned long phys_addr;
    int   i;

    DBG_TC90400DRV_PRINT("MMAP: memory Driver");
    DBG_TC90400DRV_PRINT("vm_mm:%p", vma->vm_mm);
    DBG_TC90400DRV_PRINT("vm_start:%p", (void*)vma->vm_start);
    DBG_TC90400DRV_PRINT("vm_end:%p", (void*)vma->vm_end);
    DBG_TC90400DRV_PRINT("vm_next:%p", vma->vm_next);
    DBG_TC90400DRV_PRINT("vm_page_prot:%lx", (long)vma->vm_page_prot.pgprot);
    DBG_TC90400DRV_PRINT("vm_flags:%p", (void*)vma->vm_flags);
    DBG_TC90400DRV_PRINT("vm_next_share:%p", vma->vm_next_share);
    DBG_TC90400DRV_PRINT("vm_pprev_share:%p", vma->vm_pprev_share);
    DBG_TC90400DRV_PRINT("vm_ops:%p", vma->vm_ops);
    DBG_TC90400DRV_PRINT("vm_pgoff:%p", (void*)vma->vm_pgoff);
    DBG_TC90400DRV_PRINT("vm_file:%p", vma->vm_file);
    DBG_TC90400DRV_PRINT("vm_raend:%p", (void*)vma->vm_raend);
    DBG_TC90400DRV_PRINT("vm_private_data:%p", vma->vm_private_data);
    
    off = vma->vm_pgoff << PAGE_SHIFT;
    size = vma->vm_end - vma->vm_start;
    DBG_TC90400DRV_PRINT(" [offset   %08x]", off);
    DBG_TC90400DRV_PRINT(" [size     %08x]", size);

    for (i=0; i < LEN_MAP_PARM; i++) {
      DBG_TC90400DRV_PRINT(" Area name : %s", map_parm[i].pAreaName);
      DBG_TC90400DRV_PRINT(" Start phys %08x", map_parm[i].startPhys);
      DBG_TC90400DRV_PRINT(" Start off  %08x", map_parm[i].startOff);
      DBG_TC90400DRV_PRINT(" Area size  %08x", map_parm[i].areaSize);
      if ((off >= map_parm[i].startOff)
	  && (off < (map_parm[i].startOff + map_parm[i].areaSize))) {
	if ((off + size) > (map_parm[i].startOff + map_parm[i].areaSize)) {
	  printk(KERN_ERR "tc90400_mem: ERROR invalid %s size %lx\n", map_parm[i].pAreaName, size);
	  return -EPERM;
	}
	phys_addr = (off - map_parm[i].startOff) + map_parm[i].startPhys;
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);    

	DBG_TC90400DRV_PRINT("Remap %s.", map_parm[i].pAreaName);
	DBG_TC90400DRV_PRINT("off:%p", (void*)off);
	DBG_TC90400DRV_PRINT("size:%lx",size);
	DBG_TC90400DRV_PRINT("phys_addr:%p", (void*)phys_addr);
	DBG_TC90400DRV_PRINT("vma->vm_page_prot:%lx",(long)vma->vm_page_prot.pgprot);
	DBG_TC90400DRV_PRINT("Exec io_remap_page_range");
	if (io_remap_page_range(vma->vm_start, phys_addr, vma->vm_end - vma->vm_start, vma->vm_page_prot)){
	  printk(KERN_ERR "ERROR io_remap_page_range\n");
	  return -EAGAIN;
	}
	break;
      } else {
	continue;
      }
    }
    if (i >= LEN_MAP_PARM) {
      /* out of range */
      printk(KERN_ERR "tc90400_mem: ERROR invalid offset %lx\n", off);
      return -EPERM;
    }

    return TC90400_OK;
}


static int tc90400_mem_open(struct inode *inode, struct file *file)
{
    DBG_TC90400DRV_PRINT("OPEN: memory Driver");
    return TC90400_OK;  
}

static int tc90400_mem_release(struct inode *inode, struct file *file)
{
    DBG_TC90400DRV_PRINT("CLOSE: memory Driver");
    return TC90400_OK;
}


/************************************/
/*         Root functions           */
/************************************/

static int tc90400_ioctl(struct inode *inode,struct file *file,unsigned int cmd,
			  unsigned long arg)
{
  int  i;
  unsigned int data[2];
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);

  if (!(TC90400_ENH_MMAP_MINOR <= minor && minor <= TC90400_ENH_INT_MINOR_MAX)) {
    printk(KERN_ERR "tc90400 ioctl : ERROR invalid minor no. %d\n", minor);
    return -EPERM;
  }

  switch (cmd) {
#if DBG_COMMAND
  case IOCTL_INTDRV_READ_KSEG:  /* read memory as long. */
    DBG_TC90400DRV_PRINT("IOCTL_INTDRV_READ_KSEG Driver");
    if(copy_from_user(data, (long *)arg, sizeof(data)))
      return -EFAULT;
    data[1] = *(long *)(data[0]);
    DBG_TC90400DRV_PRINT("read addr=%p data=0x%08x",(void*)data[0],data[1]);
    if(copy_to_user((long *)arg, data, sizeof(data)))
      return -EFAULT;
    break; 
  case IOCTL_INTDRV_WRITE_KSEG:  /* write memory as long. */
    DBG_TC90400DRV_PRINT("IOCTL_INTDRV_WRITE_KSEG Driver");
    if(copy_from_user(data, (long *)arg, sizeof(data)))
      return -EFAULT;
    DBG_TC90400DRV_PRINT("write addr=%p data=0x%08x",(void*)data[0],data[1]);
    *(long *)(data[0]) = (long)data[1];
    if(copy_to_user((long *)arg, data, sizeof(data)))
      return -EFAULT;
    break; 
#if DBG_TC90400 | DBG_COMMAND
  case IOCTL_INTDRV_DUMP_INFO:
    printk("IOCTL_INTDRV_DUMP_INFO Driver --- irq_sts[1..15]\n");
    for (i=1; i<16; i++) {
      printk("  (%d)%d", i, irq_sts[i]);
      if ((i == 8) || (i == 15))    printk("\n");
    }
    break; 
#endif /* DBG_TC90400 | DBG_COMMAND */
#endif /* DBG_COMMAND */
  default:
    if (TC90400_ENH_INT_MINOR_MIN <= minor && minor <= TC90400_ENH_INT_MINOR_MAX) {
      return tc90400_int_ioctl(inode, file, cmd, arg);
    } else {
      printk(KERN_ERR "tc90400 ioctl : ERROR invalid operation %d.\n", cmd);
      return -EPERM;
    }
  }
  return TC90400_OK;
}


static int tc90400_mmap(struct file * file, struct vm_area_struct * vma)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
  
  if (TC90400_ENH_MMAP_MINOR == minor) {
    return tc90400_mem_mmap( file, vma);
  } else
  printk(KERN_ERR "tc90400 mmap : ERROR invalid minor no. %d\n", minor);
  return -EPERM;
}

static unsigned int tc90400_poll(struct file *file, poll_table *wait)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
  
  if (TC90400_ENH_INT_MINOR_MIN <= minor && minor <= TC90400_ENH_INT_MINOR_MAX) {
    return tc90400_int_poll(file, wait);
  }
  printk(KERN_ERR "tc90400 poll : ERROR invalid minor no. %d\n", minor);
  return -EPERM;
}

static ssize_t tc90400_read(struct file *file,char *buf,size_t count,loff_t *ppos)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);

  if (TC90400_ENH_INT_MINOR_MIN <= minor && minor <= TC90400_ENH_INT_MINOR_MAX) {
    return tc90400_int_read(file, buf, count, ppos);
  }
  printk(KERN_ERR "tc90400 read : ERROR invalid minor no. %d\n", minor);
  return -EPERM;
}

static int tc90400_open(struct inode *inode, struct file *file)
{
  unsigned int minor = MINOR(inode->i_rdev);
  
  DBG_TC90400DRV_PRINT("OPEN:TC90400 Driver MINOR(%d)",minor);
  
  if (TC90400_ENH_MMAP_MINOR == minor) {
    return tc90400_mem_open(inode, file);
  } else if (TC90400_ENH_INT_MINOR_MIN <= minor && minor <= TC90400_ENH_INT_MINOR_MAX) {
    return tc90400_int_open(inode, file);
  }
  printk(KERN_ERR "tc90400 open : ERROR invalid minor no. %d\n", minor);
  return -EPERM;
}

static int tc90400_release(struct inode *inode, struct file *file)
{
  unsigned int minor = MINOR(inode->i_rdev);
  
  DBG_TC90400DRV_PRINT("CLOSE:TC90400 Driver MINOR(%d)",minor);
  
  if (TC90400_ENH_MMAP_MINOR == minor) {
    return tc90400_mem_release(inode, file);
  } else if (TC90400_ENH_INT_MINOR_MIN <= minor && minor <= TC90400_ENH_INT_MINOR_MAX) {
    return tc90400_int_release(inode, file);
  }
  printk(KERN_ERR "tc90400 close : ERROR invalid minor no. %d\n", minor);
  return -EPERM;
}


/***** define : Tc90400 device driver file operations *****/
struct file_operations tc90400_fops =
{
	owner:		THIS_MODULE,
	open:		tc90400_open,
	release:	tc90400_release,
	mmap:           tc90400_mmap,
	ioctl:		tc90400_ioctl,
	read:		tc90400_read,
	poll:		tc90400_poll,
};


int __init
tc90400_init(void)
{
    int rc;

    printk(KERN_INFO DRIVER_VERSION);
    
    rc = register_chrdev(TC90400_ENH_MAJOR, TC90400_ENH_DEV, &tc90400_fops);

    if (rc) {
        printk (KERN_ERR "tc90400 : register_chrdev failed\n");
        goto err_out;
    }

    return TC90400_OK;
    
err_out:
    return rc;
}

module_init(tc90400_init);


#ifdef MODULE
void
tc90400_cleanup_module(void)
{
    unregister_chrdev(TC90400_ENH_MAJOR, TC90400_ENH_DEV);
} /* tc90400_cleanup_module */

/* Module entry-points */

module_exit(tc90400_cleanup_module);


#else /* MODULE */

/* called by linux/init/main.c to parse command line options */

void
tc90400_drv_setup(char *str, int *ints)
{
    
} /* tc90400_setup */

#endif /* MODULE */

