/*
 * linux/drivers/char/tc90400_pd.c : Reference No : 43PJ292-200512110-QFP_BOARD01-001
 *
 * Power Down driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your potion) any lator version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/wait.h>
#include <asm/asm.h>
#include <asm/mipsregs.h>
#include <asm/irq.h>
#include <asm/uaccess.h>

#include <asm/tc90400/tc90400.h>
#include <asm/tc90400/tc90400_pd.h>

#define DRIVER_VERSION  "TC90400 power-down driver 0.02\n"

#define DBG_TC90400PD		0
#define DBG_TC90400PD_LED	0

#if DBG_TC90400PD
#define DBG_TC90400PD_PRINT(x...) {printk("[TC90400_PD: "); printk(x); printk("]\n");}
#else
#define DBG_TC90400PD_PRINT(x...)
#endif

#define WAIT_MACRO(x) \
  __asm__ __volatile__( \
    "   .set    noreorder	                           \n" \
    "   li      $8,  0x0   # Initialization of lapsed time \n" \
    "   li      $9,  0x0                                   \n" \
    "   or      $9,  $9, %0 # Set wait time                \n" \
    "1: addi    $8,  $8, 0x1                               \n" \
    "   bne     $8,  $9, 1b                                \n" \
    "   nop                                                \n" \
    "   .set    reorder 	                           \n" \
   : :"r" (x) :"$8","$9") /* $8=$t0,$9=$t1 */

#if DBG_TC90400PD_LED
#define DBG_LED(PAR) \
  __asm__ __volatile__( \
    "   .set    noreorder	                          \n" \
    "   la	$13, 0xbe008000 # XPIO register           \n" \
    "   lw      $15, 8($13)     # PIO bit13-16 0:On,1:Off \n" \
    "   or      $15, 0x0001e000                           \n" \
    "   sw      $15, 8($13)                               \n" \
    "   lw      $15, 0($13)                               \n" \
    "   or      $15, 0x0001e000                           \n" \
    "   ori     $14, %0, 0                                \n" \
    "   sll     $14, $14,13                               \n" \
    "   xor     $15, $14                                  \n" \
    "   sw      $15, 0($13)                               \n" \
    "   nop                                               \n" \
    "   .set    reorder 	                          \n" \
    : :"r" (PAR) ) /* $13=$t5,$14=$t6,$15=$t7 */
#else
#define DBG_LED(PAR)
#endif

static void power_down(void) __attribute__ ((__section__ (".text.halt")));

unsigned long intlevel_save[INTLVL_REGNUM];
unsigned long pd_open_flg = 0;

static void tc90400pd_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
    DBG_TC90400PD_PRINT("Poower-Down Interrupt %d !", irq);
#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
    if (irq == TC90400_IRQ_EXT0) {
      /* Interrupt edge clear */
      TC90400_WR(TC90400_INTSTSC,(TC90400_INTSTSC_EDGECLEAR|TC90400_INTSTSC_EICLR1));
    }
#endif
}

static void tc90400_irc_intlevel(unsigned long enable)
{
  unsigned int i;
  unsigned long tmpval = 0;

  tmpval = TC90400_RD(TC90400_INTMASK); /* backup interrupt mask level */
  TC90400_WR(TC90400_INTMASK, 0x7);     /* all interrupt level mask */

  for (i = 0; i < INTLVL_REGNUM; i++) {
    if (!enable) {
      intlevel_save[i] = TC90400_RD(TC90400_INTLEVEL(i));
      TC90400_WR(TC90400_INTLEVEL(i), 0);
    }
    else {
      TC90400_WR(TC90400_INTLEVEL(i), intlevel_save[i]);
    }
    DBG_TC90400PD_PRINT("INTLEVEL[%d]:0x%08x",i,intlevel_save[i]);
  }

  TC90400_WR(TC90400_INTMASK, tmpval); /* restore interrupt mask level */
}

static inline void lock_icache(unsigned long addr, unsigned long size)
{
  unsigned long end;

  addr  = addr & ~0x1f;
  end   = addr + size;

  __asm__ __volatile__(
		       "    .set    mips3	   \n"
		       "    .set    noreorder	   \n"
		       "    sync   	           \n" /* flush write-buffer */
		       "    add     $13, %0, 0     \n"
		       "    ori     $14, %1, 0     \n"
		       "1:  addiu   $12, $0, 0x4   \n"
		       "    mtc0    $12, $28	   \n" /* load $28:TagLow */
		       "    cache   0x8, 0x3($13)  \n" /* Index Store Tag */
		       "    cache   0x14, 0x3($13) \n" /* Fill */
		       "    add     $13, $13, 0x20 \n"
		       "    bne     $13, $14, 1b   \n"
		       "    nop                    \n"
		       "    .set    reorder	   \n"
		       "    .set    mips0	   \n"
		       : :"r"(addr),"r"(end)
		       ); /* $12=$t4,$13=$t5,$14=$t6 */

}

static inline void unlock_icache(unsigned long addr, unsigned long size)
{
  unsigned long end;

  addr  = addr & ~0x1f;
  end   = addr + size;

  __asm__ __volatile__(
		       "    .set    mips3	   \n"
		       "    .set    noreorder	   \n"
		       "    ori     $13,  %0, 0    \n"
		       "    ori     $14,  %1, 0    \n"
		       "1:  addiu   $12, $0, 0x0   \n"
		       "    mtc0    $12, $28 	   \n" /* load $28:TagLow */
		       "    cache   0x8, 0x3($13)  \n" /* Index Store Tag */
		       "    add     $13, $13, 0x20 \n"
		       "    bne     $13, $14, 1b   \n"
		       "    nop                    \n"
		       "    .set    reorder	   \n"
		       "    .set    mips0	   \n"
		       : :"r"(addr),"r"(end)
		       ); /* $12=$t4,$13=$t5,$14=$t6 */

}

static void power_down(void)
{
  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7500004        \n" /* CG_COCR */
		       "    li      $10, 0x0000e000        \n" /* Disable all clocks except EBUS-peripheral,VMMS,VMMS-DDR clock */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff);
		       DBG_LED(0x2);

  __asm__ __volatile__(
		       "    .set    mips3	          \n"
		       "    .set    noreorder	          \n"
		       "1:  la      $11,  0xb7410304      \n" /* VMMS_MEMBANK */
		       "    lw      $10,  0x0($11)        \n"
		       "    and     $10,  $10, 0x00000001 \n"
		       "    bne     $10,  0x0, 1b         \n" /* Loop until bit[0] is 0*/
		       "    nop                           \n"
		       "    la      $11, 0xb7410308       \n" /* VMMS_REFMODE */
		       "    li      $10, 0x00000001       \n" /* Put DDR into self-refresh mode */
		       "    sw      $10, 0x0($11)         \n"
		       "    la      $11, 0xb7410310       \n" /* VMMS_ACTMODE */
		       "    li      $10, 0x00000001       \n"
		       "    sw      $10, 0x0($11)         \n"
		       "    la      $11, 0xb7410314       \n" /* VMMS_COMSET */
		       "    li      $10, 0x00900000       \n"
		       "    sw      $10, 0x0($11)         \n"
		       "    .set    reorder	          \n"
		       "    .set    mips0	          \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
  		       WAIT_MACRO(0xffff);
		       DBG_LED(0x3);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7500004        \n" /* CG_COCR */
		       "    li      $10, 0x00008000        \n" /* Disable all clocks except EBUS peripheral clock */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff);
		       DBG_LED(0x4);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7500008        \n" /* CG_OMCR */
		       "    li      $10, 0x000000C0        \n" /* Enable Main PLL Bypass and  Main PLL */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff);
		       DBG_LED(0x5);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7500008        \n" /* CG_OMCR */
		       "    li      $10, 0x00000080        \n" /* Enable ONLY Main PLL Bypass */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff);
		       DBG_LED(0x5);

	
  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    wait                           \n" /* Put TX49 into Halt mode */
		       "    nop                            \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0  	           \n"
		       );

	
  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7500008        \n" /* CG_OMCR */
		       "    li      $10, 0x000000C0        \n" /* Re-enable Main PLL */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff);
		       DBG_LED(0x5);

  __asm__ __volatile__(
		       "    .set    reorder	           \n"
		       "    .set    mips0  	           \n"
		       "    la      $11, 0xb7500008        \n" /* CG_OMCR */
		       "    li      $10, 0x00000040        \n" /* Disable Main PLL Bypass */
 		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0  	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff);
		       DBG_LED(0x6);
	
  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7500004        \n" /* CG_COCR */
		       "    li      $10, 0x0000e000        \n" /* Enable EBUS-peripheral,VMMS,VMMS-DDR clock */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff); /* needs to be a large wait(?) */
		       DBG_LED(0x7);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7410314        \n" /* VMMS_COMSET */
		       "    li      $10, 0x00dc0000        \n" /* Take DDR out of self-refresh mode */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
  		       WAIT_MACRO(0xffff);
		       DBG_LED(0x8);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7410314        \n" /* VMMS_COMSET */
		       "    li      $10, 0x00100000        \n"
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
                       WAIT_MACRO(0xffff);
		       DBG_LED(0x9);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7410310        \n" /* VMMS_ACTMODE */
		       "    li      $10, 0x00000000        \n"
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
  		       WAIT_MACRO(0xffff);
		       DBG_LED(0xa);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7410308        \n" /* VMMS_REFMODE */
		       "    li      $10, 0x00000104        \n"
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
                       WAIT_MACRO(0xffff);
		       DBG_LED(0xb);

  __asm__ __volatile__(
		       "    .set    mips3	           \n"
		       "    .set    noreorder	           \n"
		       "    la      $11, 0xb7500004        \n" /* CG_COCR */
		       "    li      $10, 0x1fffffff        \n" /* Re-enable all clocks */
		       "    sw      $10, 0x0($11)          \n"
		       "    .set    reorder	           \n"
		       "    .set    mips0	           \n"
		       : : : "$10", "$11"
		       ); /* $10=$t3, $11=$t4 */
		       WAIT_MACRO(0xffff); /* needs to be a large wait(?) */
		       DBG_LED(0xc);

}

asmlinkage int tc90400pd_halt(unsigned long irq)
{
  unsigned long sts, tmp;

  /* Enable irq */
  DBG_TC90400PD_PRINT("Enable irq:%d", irq);
#ifdef CONFIG_TOSHIBA_TC90400QFP_BOARD01
  if (irq == TC90400_IRQ_EXT0) {
    /* Set IRC Control Reg. to EXT0 falling edge detect */
    TC90400_WR(TC90400_INTCONT1, TC90400_RD(TC90400_INTCONT1)|TC90400_INTCONT1_EXT0_F);
    /* Interrupt edge clear */
    TC90400_WR(TC90400_INTSTSC,(TC90400_INTSTSC_EDGECLEAR|TC90400_INTSTSC_EICLR1));
  }
#endif
  if (request_irq(irq, tc90400pd_interrupt, SA_INTERRUPT, "TC90400PD", 0)) {
    printk(KERN_ERR "tc90400pd:IRQ %d is not free\n",(unsigned int)irq);
    return -EIO;
  }

  DBG_TC90400PD_PRINT("Mask C0_STATUS");
  /* Mask Interrupt STATUS(IMbit) */
  sts  = read_c0_status();
  tmp  = sts & 0x0000ff01;
  sts &= 0xffff00fe; /* status reg. IM: all mask, IE:0 */
  write_c0_status(sts);

  /* lock to instruction cache */
  lock_icache((unsigned long)&power_down, 0x800);
  DBG_LED(0x1);

  /* halt mode */
  power_down();

  /* unlock to instruction cache */
  DBG_LED(0xd);
  unlock_icache((unsigned long)&power_down, 0x800);

  DBG_LED(0xe);
  DBG_TC90400PD_PRINT("Unmask C0_STATUS (0x%x)", tmp);
  /* Unmask Interrupt STATUS(IMbit) */
  sts  = read_c0_status();
  sts |= tmp; /* status reg. return to the previous value */
  write_c0_status(sts);

  DBG_TC90400PD_PRINT("Disable irq:%d", irq);
  /* Disable irq */
  disable_irq(irq);
  free_irq(irq , 0);

  return 0;
}


/************************************/
/*            functions             */
/************************************/

static int tc90400pd_ioctl(struct inode *inode, struct file *file,
			   unsigned int cmd, unsigned long arg)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
  unsigned long data;
  int ret;

  if (TC90400_PD_MINOR != minor) {
    printk(KERN_ERR "tc90400pd ioctl:ERROR invalid minor no. %d\n", minor);
    return -EPERM;
  }

  switch (cmd) {
  case IOCTL_PD_HALT:
    DBG_TC90400PD_PRINT("IOCTL_PD_HALT Driver irq:%d",arg);
    ret = tc90400pd_halt(arg);
    if (ret < 0) return ret;
    break;
  case IOCTL_PD_COCR_W:
    DBG_TC90400PD_PRINT("IOCTL_PD_COCR_W write 0x%x", arg);
    TC90400_WR(TC90400_CG_COCR, arg);
    break;
  case IOCTL_PD_COCR_R:
    DBG_TC90400PD_PRINT("IOCTL_PD_COCR_R ");
    data = TC90400_RD(TC90400_CG_COCR);
    DBG_TC90400PD_PRINT(" COCR:0x%08x\n", (unsigned int)data);
    if (put_user(data, (unsigned long *)arg)) {
      return -EFAULT;
    }
    break;
  case IOCTL_PD_INT_MASK:
    DBG_TC90400PD_PRINT("IOCTL_PD_INT_MASK");
    tc90400_irc_intlevel(0); /* interrupt all disable */
    break;
  case IOCTL_PD_INT_UNMASK:
    DBG_TC90400PD_PRINT("IOCTL_PD_INT_UNMASK");
    tc90400_irc_intlevel(1); /* return to save value */
    break;
  case IOCTL_PD_RSTCR:
    DBG_TC90400PD_PRINT("IOCTL_PD_RSTCR write 0x%x",~PD_RSTCR_VALUE);
    TC90400_WR(TC90400_CG_RSTCR, ~PD_RSTCR_VALUE);
    break;
  default:
    printk(KERN_ERR "tc90400pd ioctl:ERROR invalid operation %d.\n", cmd);
    return -EPERM;
  }
  return TC90400PD_OK;
}


static int tc90400pd_open(struct inode *inode, struct file *file)
{
  unsigned int minor = MINOR(inode->i_rdev);
  
  if (TC90400_PD_MINOR != minor) {
    printk(KERN_ERR "tc90400pd open:ERROR invalid minor no. %d\n", minor);
    return -EPERM;
  }
  /* check open flag */
  if (pd_open_flg > 0) {
    printk(KERN_ERR "tc90400pd open:ERROR driver busy\n");
    return -EBUSY;
  }

  DBG_TC90400PD_PRINT("OPEN:TC90400 PD Driver MINOR(%d)",minor);
  pd_open_flg++;
  return TC90400PD_OK;
}


static int tc90400pd_release(struct inode *inode, struct file *file)
{
  unsigned int minor = MINOR(inode->i_rdev);
  
  if (TC90400_PD_MINOR != minor) {
    printk(KERN_ERR "tc90400pd close:ERROR invalid minor no. %d\n", minor);
    return -EPERM;
  }

  DBG_TC90400PD_PRINT("CLOSE:TC9040 PD Driver MINOR(%d)",minor);
  pd_open_flg = 0;
  return TC90400PD_OK;
}


/***** define : Tc90400 power-down driver file operations *****/
struct file_operations tc90400pd_fops =
{
	owner:		THIS_MODULE,
	open:		tc90400pd_open,
	release:	tc90400pd_release,
	ioctl:		tc90400pd_ioctl,
};


int __init tc90400pd_init(void)
{
    int rc;

    printk(KERN_INFO DRIVER_VERSION);
    
    rc = register_chrdev(TC90400_PD_MAJOR, TC90400_PD_DEV, &tc90400pd_fops);

    if (rc) {
        printk(KERN_ERR "tc90400pd init:ERROR register_chrdev failed\n");
        goto err_out;
    }
    return TC90400PD_OK;
    
err_out:
    return rc;
}

module_init(tc90400pd_init);


#ifdef MODULE
void
tc90400pd_cleanup_module(void)
{
    unregister_chrdev(TC90400_PD_MAJOR, TC90400_PD_DEV);
}

/* Module entry-points */

module_exit(tc90400pd_cleanup_module);


#else /* MODULE */

/* called by linux/init/main.c to parse command line options */

void
tc90400pd_setup(char *str, int *ints)
{
    
}

#endif /* MODULE */

