/*
 * linux/drivers/char/tc90400_pio.c : Reference No : 43PJ292-20050819-QFP_BOARD01-000
 *
 * Internal I2C driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#include <linux/module.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>
#include <linux/init.h>
#include <asm/irq.h>

#include <asm/tc90400/tc90400.h>
#include <asm/tc90400/tc90400_pio.h>


#define DRIVER_VERSION  "TC90400 PIO driver 0.0\n"

#define PIO_IOCTL_DBG		1
#define DBG_TC90400		0

#if DBG_TC90400
#define DBG_TC90400_PRINT(x...) {printk("[TC90400_PIO: "); printk(x); printk("]\n");}
#else
#define DBG_TC90400_PRINT(x...)
#endif

/* PIO HW table */
static struct pio_dev_t {
  unsigned int minor;
  char         *name;
  int          open;
} pio_dev = {
  PIO_MISC_MINOR,      /* minor */
  "tc90400-pio",       /* name  */
  0,                   /* open flag */
};


static int pio_open(struct inode *inode, struct file *file)
{

  DBG_TC90400_PRINT("OPEN");

  unsigned int minor = MINOR(inode->i_rdev);
  
  if (minor != pio_dev.minor) {
    printk(KERN_DEBUG "TC90400PIO open: illegal minor no. %d\n", minor);
    return -ENODEV;
  }

  if (pio_dev.open > 0) {
    return -EBUSY;
  }
  pio_dev.open++;
  return 0;  
}


static int pio_release(struct inode *inode, struct file *file)
{
  DBG_TC90400_PRINT("CLOSE");

  unsigned int minor = MINOR(inode->i_rdev);
  
  if (minor != pio_dev.minor) {
    printk(KERN_DEBUG "TC90400PIO release: illegal minor no. %d\n", minor);
    return -ENODEV;
  }
  pio_dev.open = 0;

  return 0;
}


static int pio_ioctl(struct inode *inode, struct file *filp,
			  unsigned int cmd, unsigned long arg)
{
  unsigned int data[2];
  unsigned int tmp;
  unsigned int minor = MINOR(inode->i_rdev);
	
  if (minor != pio_dev.minor) {
    printk(KERN_DEBUG "TC90400PIO ioctl: illegal minor no. %d\n", minor);
    return -ENODEV;
  }

  switch(cmd){
  case IOCTL_PIO_IN:
    data[0] = TC90400_RD(TC90400_XPIODI);
    if (put_user(data[0], (unsigned int *)arg)) {
      return -EFAULT;
    }
    break;
  case IOCTL_PIO_OUT:
    if(copy_from_user(data, (long *)arg, sizeof(data)))
      return -EFAULT;
    tmp = TC90400_RD(TC90400_XPIODIR);
    TC90400_WR(TC90400_XPIODIR, tmp | data[0]);
    tmp = TC90400_RD(TC90400_XPIODO);
    if (data[1])
      TC90400_WR(TC90400_XPIODO, tmp | data[0]);
    else
      TC90400_WR(TC90400_XPIODO, tmp & ~data[0]);
    break;
#ifdef PIO_IOCTL_DBG
  case IOCTL_PIO_DIR_R:
    data[0] = TC90400_RD(TC90400_XPIODIR);
    if (put_user(data[0], (unsigned int *)arg)) {
      return -EFAULT;
    }
    break;
  case IOCTL_PIO_DIR_W:
    TC90400_WR(TC90400_XPIODIR, arg);
    break;
  case IOCTL_PIO_OUT_R:
    data[0] = TC90400_RD(TC90400_XPIODO);
    if (put_user(data[0], (unsigned int *)arg)) {
      return -EFAULT;
    }
    break;
#endif
  default:
    DBG_TC90400_PRINT("IOCTL_PIO_nothing:TC90400 PIO Driver");
    return -EINVAL;
  }

  return 0;
}


/***** define : PIO device driver file operations *****/
struct file_operations pio_fops =
{
	owner:		THIS_MODULE,
	open:		pio_open,
	ioctl:          pio_ioctl,
	release:	pio_release,
};

static struct miscdevice pio_miscdev = {
	minor:		PIO_MISC_MINOR,
	name:		"TC90400_PIO",
	fops:		&pio_fops,
};


static int __init pio_init(void)
{
  int rc;

  printk(KERN_INFO DRIVER_VERSION);

  rc = misc_register(&pio_miscdev);
  if (rc) {
    printk (KERN_ERR "TC90400 pio: misc_register failed\n");
    return rc;
  }
  //  TC90400_WR(TC90400_CFSH0, (TC90400_RD(TC90400_CFSH0)|0x10)); /* for DEBUG */

  return 0;
}


static void __exit pio_exit(void)
{
  misc_deregister(&pio_miscdev);

}

/* Module entry-points */
module_init(pio_init);
module_exit(pio_exit);
