/*
 * linux/drivers/i2c/i2c-adap-tc90400.c : Reference No : 43PJ292-20051019-QFP_BOARD01-000
 *
 * Internal I2C driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
   ---------------------------------------------------------------------------
   Hai-Pao Fan, MontaVista Software, Inc.
   hpfan@mvista.com or source@mvista.com

   Copyright 2001 MontaVista Software Inc.

   ----------------------------------------------------------------------------
   This file was highly leveraged from i2c-elektor.c, which was created
   by Simon G. Vogl and Hans Berglund:

 
     Copyright (C) 1995-97 Simon G. Vogl
                   1998-99 Hans Berglund

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			     */
/* ------------------------------------------------------------------------- */

/* With some changes from Kytiki <kmalkki@cc.hut.fi> and even
   Frodo Looijaard <frodol@dds.nl> */


#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/irq.h>
#include <asm/io.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-tc90400.h>
#include <linux/i2c-adap-tc90400.h>
#include "i2c-tc90400.h"


/* ----- global defines -----------------------------------------------	*/
/*#define TC90400_I2C_DEBUG*/

#if defined(TC90400_I2C_DEBUG)
#define DEB_START	printk(__FUNCTION__": start\n");
#define DEB_END		printk(__FUNCTION__": end\n");
#define DEB_SNAP(x...)	printk(__FUNCTION__": "/*##*/x);
#define DEB_SNAP2(x...)	printk(/*##*/x);
#else
#define DEB_START
#define DEB_END
#define DEB_SNAP(x...)
#define DEB_SNAP2(x...)
#endif

#define DEF_TIMEOUT 16

/* ----- global variables ---------------------------------------------	*/
static int clock = 0;
static int own   = 0;

/* static int i2c_debug=0; */
static struct i2c_tc90400 gpi[2];
static wait_queue_head_t i2c_wait[2];
static int i2c_pending[2];

/* ----- local functions ----------------------------------------------	*/

static void i2c_tc90400_seti2c(int ctl, unsigned char val)
{

DEB_SNAP2("    Write 0x%02x to 0x%x(0x%x)\n",(unsigned short)val, ctl, KSEG1ADDR(ctl))

  *(volatile u32 *)(ctl) = (volatile u32)val;
}

static unsigned char i2c_tc90400_geti2c(int ctl)
{
  volatile u32 val;

  val = *(volatile u32 *)(ctl);

DEB_SNAP2("    Read 0x%02x to 0x%x(0x%x)\n",(unsigned short)val, ctl, KSEG1ADDR(ctl))
  return (val);
}

/* Return our slave address.  This is the address
 * put on the I2C bus when another master on the bus wants to address us
 * as a slave
 */
static int i2c_tc90400_getown(struct i2c_algo_tc90400_data *adap)
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START

  return (gpi->i2c_own);
}


static int i2c_tc90400_getclock(struct i2c_algo_tc90400_data *adap)
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START

  return (gpi->i2c_clk);
}


static int i2c_tc90400_setclock(struct i2c_algo_tc90400_data *adap, int value)
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START
  if ((value < 1) || (value > 6)) {
    printk(KERN_DEBUG "Set default baudrate value\n");
    if (!adap->ch)
      gpi->i2c_clk = DEFAULT_CLOCK_CH0;
    else
      gpi->i2c_clk = DEFAULT_CLOCK_CH1;
  }
  else {
    gpi->i2c_clk = value;
  }
  return 0;
}


/* Put this process to sleep.  We will wake up when the
 * I2C controller interrupts.
 */
static void i2c_tc90400_waitforpin(struct i2c_algo_tc90400_data *adap, long timeout) {

  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START

  if(!timeout){
    timeout = 2;
  }

  /* If interrupts are enabled (which they are), then put the process to
   * sleep.  This process will be awakened by two events -- either the
   * the I2C peripheral interrupts or the timeout expires. 
   * If interrupts are not enabled then delay for a reasonable amount 
   * of time and return.
   */
  if (gpi->i2c_irq > 0) {
    cli();
    if (i2c_pending[adap->ch] == 0) {
      interruptible_sleep_on_timeout(&i2c_wait[adap->ch], timeout*HZ );
    } else {
      i2c_pending[adap->ch] = 0;
    }
    sti();
  } else {
    udelay(100);
  }
DEB_END
}

/*
 * wait for bus free
 */
static int i2c_tc90400_waitforbb(struct i2c_algo_tc90400_data *adap, long timeout)
{
  short status;

DEB_START

  status = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));

  while (timeout-- && (status & I2CCR1_BUSBUSY)) {
#if 1 /* Is this method better? */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(1);
#else
    udelay(1000); /* How much is this? */
#endif
    status = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));
  }

  if (timeout<=0) {
    printk(KERN_ERR "i2c_tc90400_waitforbb:Timeout, device is busy\n");
    /*adap->reset(adap);*/
  }
  return(timeout<=0);
}

/*
 * wait for ack of transaction 
 */
static int i2c_tc90400_waitforack(struct i2c_algo_tc90400_data *adap, unsigned short trx, long timeout) {

  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;
  short status;

DEB_START

  status = ~(trx << 6);
 
  while (timeout-- && ((status & I2CCR1_TRX) != (trx << 6))) {
    adap->waitforpin(adap,2);
    status = gpi->i2c_sts;
  }

  if (timeout <= 0) {
    printk(KERN_ERR "i2c_tc90400_waitforack:Timeout!\n");
    return(-1);
  } else {
    return(0);
  }
}

/*
 * wait for LRB=H
 */
static int i2c_tc90400_waitforlrb(struct i2c_algo_tc90400_data *adap, long timeout)
{
  short status;

DEB_START

  status = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));

  while (timeout-- && !(status & I2CCR1_LRB)) {
#if 1 /* Is this method better? */
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(1);
#else
    udelay(1000); /* How much is this? */
#endif
    status = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));
  }

  if (timeout<=0) {
    printk(KERN_ERR "i2c_tc90400_waitforlrb:Timeout, SDA:L\n");
  }
  return(timeout<=0);
}

/*
 * generate start condition
 */
static void i2c_tc90400_start(struct i2c_algo_tc90400_data *adap)
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START

  /* generate start condition */
  spin_lock(&gpi->lock);
  gpi->skip_intr = 1;
  spin_unlock(&gpi->lock);

  i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
		     (I2C_CMD | I2CCR1_PIN | I2CCR1_BUSBUSY |I2CCR1_TRX));
}

/*
 * generate stop condition
 */
static void i2c_tc90400_stop(struct i2c_algo_tc90400_data *adap)
{
  short val;

DEB_START

  val = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));

  if (val & I2CCR1_BUSBUSY) {
    i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
		       (I2C_CMD | I2CCR1_PIN | I2CCR1_TRX));
  }
}

/*
 * generate stop condition
 */
static void i2c_tc90400_stop_for_restart(struct i2c_algo_tc90400_data *adap)
{
DEB_START
  i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
		     (I2CCR1_SBIM_I2C | I2CCR1_PIN));

#if 0 /* This check is moved after interrupt handler completion */
  adap->waitforbb(adap,DEF_TIMEOUT);

  status = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));

  while (timeout-- && !(status & I2CCR1_LRB)) {
    udelay(1000); /* How much is this? */
    status = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));
  }
  if (timeout<=0) {
    printk(KERN_ERR "Timeout, device is busy\n");
    /* adap->reset(adap); */
  }

  udelay(10); /* How much is this? */
#endif
  udelay(10); /* How much is this? */

}
static void i2c_tc90400_reset(struct i2c_algo_tc90400_data *adap)
{
  short val;

DEB_START
  /* exec software reset */
  i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch), I2CCR1_RESET1|I2CCR1_SBIM_I2C);
  i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch), I2CCR1_RESET2|I2CCR1_SBIM_I2C);

  /* wait for reset completion */
  do {
    val = i2c_tc90400_geti2c(TC90400_I2CCR0(adap->ch));
    mdelay(1);
  } while (!(val & I2CCR0_SWRMON));
}

static int i2c_tc90400_master_send(struct i2c_algo_tc90400_data *adap)
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START
DEB_SNAP("##### tx_size:0x%x, tx_count:0x%x, restart:0x%x\n"
	 ,gpi->tx_size, gpi->tx_count, gpi->restart)

  spin_lock(&gpi->tx_lock);
  if (gpi->tx_count < gpi->tx_size) {
    /* write 1 word data */
    i2c_tc90400_seti2c(TC90400_I2CDBR(adap->ch),
		       gpi->tx_buf[gpi->tx_count]);

    gpi->tx_count++;
    spin_unlock(&gpi->tx_lock);
    return gpi->tx_count;
  } else {
    /* stop condition */
    spin_unlock(&gpi->tx_lock);
    if( !gpi->restart ){
      i2c_tc90400_stop(adap);
    }else{
      i2c_tc90400_stop_for_restart(adap);
    }
    /* signal to exit transmit */
    wake_up_interruptible(&i2c_wait[adap->ch]);
    complete(&gpi->tx_comp);
  }
  return -1;
}


static int i2c_tc90400_master_recv(struct i2c_algo_tc90400_data *adap)
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START
DEB_SNAP("##### skip_intr:0x%x, rx_size:0x%x, rx_count:0x%x, restart:0x%x\n"
	 ,gpi->skip_intr, gpi->rx_size, gpi->rx_count, gpi->restart)

  spin_lock(&gpi->lock);
  if (gpi->skip_intr == 1) {
    /* next is last data */
    if ((gpi->rx_size - gpi->rx_count) == 1) {
      /* set no ACK mode */
      i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			 (I2C_CNT0 | i2c_tc90400_getclock(adap)));
    }
    /* dummy read */
    i2c_tc90400_geti2c(TC90400_I2CDBR(adap->ch));

    gpi->skip_intr = 0;
    spin_unlock(&gpi->lock);
    return 0;
  }
  spin_unlock(&gpi->lock);
  spin_lock(&gpi->rx_lock);
  if (gpi->rx_size > gpi->rx_count) {
    if ((gpi->rx_size - gpi->rx_count) == 2) {
      /* set no ACK mode */
      i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			 (I2C_CNT0 | i2c_tc90400_getclock(adap)));
    } else if ((gpi->rx_size - gpi->rx_count) == 1) {
      /* set 1bit transfer clock */
      i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			 (I2C_CNT0 | I2CCR0_BC_1 | i2c_tc90400_getclock(adap)));
    }
    /* read 1 word data */
    gpi->rx_buf[gpi->rx_count] = i2c_tc90400_geti2c(TC90400_I2CDBR(adap->ch));
    gpi->rx_count++;
    spin_unlock(&gpi->rx_lock);
    return gpi->rx_count;
  }

  /* stop condition */
  spin_unlock(&gpi->rx_lock);
  if( !gpi->restart ){
    i2c_tc90400_stop(adap);
  }else{
    i2c_tc90400_stop_for_restart(adap);
  }
  wake_up_interruptible(&i2c_wait[adap->ch]);
  complete(&gpi->rx_comp);
  return -1;
}

static void mst_handler(struct i2c_algo_tc90400_data *adap, unsigned short val) 
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;
  int flags;

DEB_START
  if (val & I2CCR1_PIN) {
    return;
  }

  save_flags(flags);
  cli();

  gpi->i2c_sts = val;
DEB_SNAP2("master mode--")
  if (val & I2CCR1_TRX) {       /* transmit mode */
DEB_SNAP2("transmit mode--")
    if (val & I2CCR1_LRB) {	/* last received bit(0:ack) */
DEB_SNAP2("nack--\n")
      if (gpi->tx_count) {
        gpi->tx_count--; /* for increment in master_send */
      }
      if ( !gpi->restart ){ /* need to check */
	i2c_tc90400_stop(adap);
      }else{
	i2c_tc90400_stop_for_restart(adap);
      }
      /* signal to exit transmit */
      wake_up_interruptible(&i2c_wait[adap->ch]);
      if(gpi->trns_mode){
	complete(&gpi->rx_comp);
      }else {
	complete(&gpi->tx_comp);
      }
    }
    /* TC90400_TENBIT */
    else if (gpi->tenbit_addr) {
      if (gpi->trns_mode)
	i2c_tc90400_stop_for_restart(adap);
      wake_up_interruptible(&i2c_wait[adap->ch]);
    }
    /* TC90400_TENBIT */
    else {
DEB_SNAP2("ack--\n")
      if(i2c_tc90400_master_send(adap) < 0) {
      }
    }
  }
  else {	/* receive mode */
DEB_SNAP2("receive mode--\n")
    if (i2c_tc90400_master_recv(adap) < 0) {
    }
  }
  i2c_pending[adap->ch] = (int)1;

  restore_flags(flags);
DEB_END
}

static void slv_handler(struct i2c_algo_tc90400_data *adap, unsigned short val) 
{
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;
  int flags, mode;
  unsigned char sum, i;

DEB_START
  if (gpi->trns_mode < 0) { /* not the transfer mode */
    /* only interrupt clear */
    i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
		       (I2CCR1_SBIM_I2C | I2CCR1_PIN));
    return;
  }
  if (val & I2CCR1_PIN) {
    return;
  }

  save_flags(flags);
  cli();

  gpi->i2c_sts = val;
  mode = gpi->trns_mode;

DEB_SNAP2("slave mode--")
  if (val & I2CCR1_AL) { /* Detect arbitration loss */
    printk(KERN_ERR "Arbitration loss occurred! ch%d cr1:0x%x\n",adap->ch,val);
    /* It shouldn't happen not to be the multi-master mode.  */
    /* You must examine it if the multi-master mode is used. */
    gpi->trx_status = SLV_ALERROR;
    i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
		       (I2CCR1_SBIM_I2C | I2CCR1_PIN));
    if(gpi->trns_mode){
      complete(&gpi->rx_comp);
    }
    else {
      complete(&gpi->tx_comp);
    }
    restore_flags(flags);
    return;
  }

  if (val & I2CCR1_AAS) { /* Receive slave address */
    if (!gpi->trx_status) { /* normal status */
      if (val & I2CCR1_LRB) {  /* no return Ack */
DEB_SNAP("aas&lrb:0x%x\n", val) 
        /* only interrupt clear */
        i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
			   (I2CCR1_SBIM_I2C | I2CCR1_PIN));
      }
      else { /* return Ack */
	if (val & I2CCR1_TRX) { /* transmit mode */
DEB_SNAP2("aas transmit mode--\n")
          gpi->trx_status = SLV_NEXTWRITE;
          wake_up_interruptible(&gpi->rx_wait);
	}
	else { /* receive mode */
DEB_SNAP2("aas receiver mode--\n")
          gpi->trx_status = SLV_NEXTREAD;
          wake_up_interruptible(&gpi->rx_wait);
	}
      }
    }
    else { /* illegal status */
DEB_SNAP2("aas illegal--\n")
#if 1
      if ((gpi->trx_status == SLV_READ) && (!gpi->rx_count)) {
#if 1
	printk(KERN_DEBUG "retry slave-address sts:%d cr1:0x%x\n",gpi->trx_status,val);
#endif
	if (val & I2CCR1_TRX) { /* transmit mode */
          gpi->trx_status = SLV_NEXTWRITE;
          wake_up_interruptible(&gpi->rx_wait);
	}
	else { /* receive mode */
	  /* read dummy data */
	  i2c_tc90400_geti2c(TC90400_I2CDBR(adap->ch));
	}
      }
#endif
      else {
#if 0 /* not change ACKbit */
	/* Set no ACK mode */
	i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			   (I2C_CNT0|i2c_tc90400_getclock(adap)));
#endif
	/* Clear interrupt */
	i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
			   (I2CCR1_SBIM_I2C | I2CCR1_PIN));
	printk(KERN_ERR "Unexpected slave-address interruption sts:%d cr1:0x%x\n",gpi->trx_status,val);
	gpi->trns_mode = -1; /* transfer end */
	gpi->trx_status = SLV_SEQERROR;
	if (mode) {
	  wake_up_interruptible(&gpi->rx_wait);
	}
	else {
	  wake_up_interruptible(&gpi->tx_wait);
	}
      }
    }
  }

  else {
    if (val & I2CCR1_TRX) { /* transmit mode */
      spin_lock(&gpi->tx_lock);
      if (gpi->trx_status == SLV_WRITE) {
	if (val & I2CCR1_LRB) { /* transmit end (no ack) */
DEB_SNAP2("transmit mode NOACK--\n")
#if 0 /* not change ACKbit */
          /* Set no ACK mode */
          i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			     (I2C_CNT0|i2c_tc90400_getclock(adap)));
          /* Clear interrupt */
          i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
			     (I2CCR1_SBIM_I2C | I2CCR1_PIN));
#endif
	  gpi->trns_mode = -1; /* transfer end */
	  gpi->trx_status = SLV_WRITEEND;
          wake_up_interruptible(&gpi->tx_wait);
	}
	else { /* transmit continue (ack) */
DEB_SNAP2("transmit mode ACK--\n")
DEB_SNAP("##### tx_size:0x%x, tx_count:0x%x\n" ,gpi->tx_size, gpi->tx_count)
          if (gpi->tx_count < gpi->tx_size) {
	    /* Write 1 word data */
	    i2c_tc90400_seti2c(TC90400_I2CDBR(adap->ch),
			       gpi->tx_buf[gpi->tx_count]);
	    gpi->tx_count++;
	    gpi->trx_status = SLV_WRITE;
	  }
	  else { /* no transmit data */
	    gpi->trx_status = SLV_UNDERRFLOW;
	    wake_up_interruptible(&gpi->tx_wait);
	  }
	}
      }
      else { /* illegal status */
DEB_SNAP2("tx illegal--\n")
#if 0 /* not change ACKbit */
        /* Set no ACK mode */
        i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			   (I2C_CNT0|i2c_tc90400_getclock(adap)));
#endif
        /* Clear interrupt */
        i2c_tc90400_seti2c(TC90400_I2CCR1(adap->ch),
			   (I2CCR1_SBIM_I2C | I2CCR1_PIN));
	printk(KERN_ERR "Unexpected TX interruption sts:%d cr1:0x%x\n",gpi->trx_status,val);
	gpi->trns_mode = -1; /* transfer end */
	gpi->trx_status = SLV_SEQERROR;
	if (mode) {
	  wake_up_interruptible(&gpi->rx_wait);
	}
	else {
	  wake_up_interruptible(&gpi->tx_wait);
	}
      }
      spin_unlock(&gpi->tx_lock);
    }
    else { /* receive mode */
DEB_SNAP2("receive mode--\n")
DEB_SNAP("##### rx_size:0x%x, rx_count:0x%x\n" ,gpi->rx_size, gpi->rx_count)
      spin_lock(&gpi->rx_lock);
      if ((gpi->trx_status & SLV_READ) == SLV_READ) {
#if 0 /* not change ACKbit */
	if ((gpi->rx_size-gpi->rx_count) <= 1) {
	  /* Set no ACK mode */
	  i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			     (I2C_CNT0|i2c_tc90400_getclock(adap)));
	}
#endif
	val = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));
	if (!(val & I2CCR1_BUSBUSY) || ((gpi->rx_size-gpi->rx_count)==1)) { /* bus free or last byte */
	  gpi->trns_mode = -1; /* transfer end */
	  if (gpi->trx_status & SLV_CHECKSUM) {
	    gpi->trx_status = SLV_CHECKSUM;
	  }
	  else {
	    gpi->trx_status = SLV_READEND;
	  }
	  wake_up_interruptible(&gpi->rx_wait);
	}
	else {
	  /* Read 1 word data */
	  gpi->rx_buf[gpi->rx_count] = i2c_tc90400_geti2c(TC90400_I2CDBR(adap->ch));
	  gpi->rx_count++;
	  if ((gpi->rx_size-gpi->rx_count)==1) { /* check sum */
	    for (i=0,sum=0; i<(gpi->rx_count-1); i++) {
	      sum += (unsigned char)gpi->rx_buf[i];
	    }
	    if (sum != (unsigned char)gpi->rx_buf[gpi->rx_count-1]) {
	      /* check sum error */
	      gpi->trx_status |= SLV_CHECKSUM;
	      /* GPIO3_1 -> H */
	      TC90400_WR(TC90400_XPIODIR,(TC90400_RD(TC90400_XPIODIR)|0x02000000)); /* direction output */
	      TC90400_WR(TC90400_XPIODO,TC90400_RD(TC90400_XPIODO)|0x02000000); /* output L->H */
	    }
	  }
	}
      }
      else { /* illegal status */
DEB_SNAP2("rx illegal--\n")
#if 0 /* not change ACKbit */
        /* Set no ACK mode */
        i2c_tc90400_seti2c(TC90400_I2CCR0(adap->ch),
			   (I2C_CNT0|i2c_tc90400_getclock(adap)));
#endif
	/* Read dummy data */
	i2c_tc90400_geti2c(TC90400_I2CDBR(adap->ch));
	printk(KERN_ERR "Unexpected RX interruption sts:%d cr1:0x%x\n",gpi->trx_status,val);
	gpi->trns_mode = -1; /* transfer end */
	gpi->trx_status = SLV_SEQERROR;
	if (mode) {
	  wake_up_interruptible(&gpi->rx_wait);
	}
	else {
	  wake_up_interruptible(&gpi->tx_wait);
	}
      }
      spin_unlock(&gpi->rx_lock);
    }
  }

  restore_flags(flags);
DEB_END
}

static void i2c_tc90400_handler(int this_irq, void *dev, struct pt_regs *regs) 
{
  struct i2c_algo_tc90400_data *adap = dev;
  unsigned long clear_no;
  unsigned short cr1;

  cr1 = i2c_tc90400_geti2c(TC90400_I2CCR1(adap->ch));

  /* Interrupt Clear */
  clear_no = this_irq - TC90400_IRQ_PIC_BEG -16;
  TC90400_WR(TC90400_INTSTSC, (TC90400_INTSTSC_EDGECLEAR|clear_no)<<16);

  if (cr1 & I2CCR1_MST) {
    mst_handler(adap, cr1); /* Master Interrupt */
  }
  else {
    slv_handler(adap, cr1); /* Slave Interrupt */
  }
}



/* Lock the region of memory where I/O registers exist.  Request our
 * interrupt line and register its associated nhandler.
 */
static int i2c_hw_resrc_init(struct i2c_adapter *i2c_adap)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->data;
  struct i2c_tc90400 *gpi = (struct i2c_tc90400 *)adap->data;

DEB_START

  if( check_region(gpi->i2c_base, TC90400_I2C_IO_SIZE) < 0 )
    return -ENODEV;
  else {
    if( adap->ch == 0 )
      request_region(gpi->i2c_base, TC90400_I2C_IO_SIZE
		     , "i2c ch.0 (i2c bus adapter)" );
    else
      request_region(gpi->i2c_base, TC90400_I2C_IO_SIZE
		     , "i2c ch.1 (i2c bus adapter)" );
  }

  /* register interrupt routine */
   i2c_pending[adap->ch] = 0;

   if (gpi->i2c_irq > 0) {
     if (request_irq(gpi->i2c_irq, i2c_tc90400_handler, 0, "TC90400 I2C", adap) < 0) {
       gpi->i2c_irq = 0;
     }
   }

   return 0;
}


static void i2c_tc90400_release(int ch)
{

DEB_START

  if (gpi[ch].i2c_irq > 0) {
    disable_irq(gpi[ch].i2c_irq);
    free_irq(gpi[ch].i2c_irq, 0);
  }
  release_region( gpi[ch].i2c_base, TC90400_I2C_IO_SIZE );
}


static int i2c_tc90400_reg(struct i2c_client *client)
{

DEB_START

  return 0;
}


static int i2c_tc90400_unreg(struct i2c_client *client)
{

DEB_START

  return 0;
}


static void i2c_tc90400_inc_use(struct i2c_adapter *adap)
{

DEB_START

#ifdef MODULE
  MOD_INC_USE_COUNT;
#endif
}


static void i2c_tc90400_dec_use(struct i2c_adapter *adap)
{

DEB_START

#ifdef MODULE
  MOD_DEC_USE_COUNT;
#endif
}

/* ------------------------------------------------------------------------
 * Encapsulate the above functions in the correct operations structure.
 * This is only done when more than one hardware adapter is supported.
 */

static struct i2c_algo_tc90400_data i2c_tc90400_data[2] = {
	{
		NULL,				/* void *data */
		i2c_tc90400_seti2c,
		i2c_tc90400_geti2c,
		i2c_tc90400_getown,
		i2c_tc90400_getclock,
		i2c_tc90400_setclock,
		i2c_tc90400_waitforpin,
		i2c_tc90400_waitforbb,
		i2c_tc90400_waitforack,
		i2c_tc90400_waitforlrb,
		i2c_tc90400_start,
		i2c_tc90400_stop,
		i2c_tc90400_reset,
		i2c_tc90400_master_send,
		i2c_tc90400_master_recv,
		80, 80, 100, 0,			/* udelay,mdelay,timeout,ch */
	},{
		NULL,				/* void *data */
		i2c_tc90400_seti2c,
		i2c_tc90400_geti2c,
		i2c_tc90400_getown,
		i2c_tc90400_getclock,
		i2c_tc90400_setclock,
		i2c_tc90400_waitforpin,
		i2c_tc90400_waitforbb,
		i2c_tc90400_waitforack,
		i2c_tc90400_waitforlrb,
		i2c_tc90400_start,
		i2c_tc90400_stop,
		i2c_tc90400_reset,
		i2c_tc90400_master_send,
		i2c_tc90400_master_recv,
		80, 80, 100, 1,			/* udelay,mdelay,timeout,ch */
	}
};
static struct i2c_adapter i2c_tc90400_ops[2] = {
	{
		"TC90400 I2C adapter(CH0)",
		I2C_HW_TC90400,
		NULL,
		&i2c_tc90400_data[0],
		i2c_tc90400_inc_use,
		i2c_tc90400_dec_use,
		i2c_tc90400_reg,
		i2c_tc90400_unreg,
	},{
		"TC90400 I2C adapter(CH1)",
		I2C_HW_TC90400,
		NULL,
		&i2c_tc90400_data[1],
		i2c_tc90400_inc_use,
		i2c_tc90400_dec_use,
		i2c_tc90400_reg,
		i2c_tc90400_unreg,
	}
};


/* Called when the module is loaded.  This function starts the
 * cascade of calls up through the heirarchy of i2c modules (i.e. up to the
 *  algorithm layer and into to the core layer)
 */
static int __init i2c_tc90400_init(void) 
{

  struct i2c_tc90400 *pi2c;
  int ch;
  struct i2c_algo_tc90400_data *i2c_data;

DEB_START
  printk(KERN_INFO "Initialize TC90400 I2C adapter module\n");

  for (ch = 0; ch < NUM_OF_CH; ch++) {
    pi2c = &gpi[ch];
    i2c_data = &i2c_tc90400_data[ch];

    if(ch==0){
      pi2c->i2c_base = DEFAULT_BASE_CH0;
    }else{
      pi2c->i2c_base = DEFAULT_BASE_CH1;
    }

    if(ch==0){
      pi2c->i2c_irq = DEFAULT_IRQ_CH0;
    }else{
      pi2c->i2c_irq = DEFAULT_IRQ_CH1;
    }

    if (clock == 0){
      if(ch==0){
	pi2c->i2c_clk = DEFAULT_CLOCK_CH0;
      }else{
	pi2c->i2c_clk = DEFAULT_CLOCK_CH1;
      }
    }else{
      pi2c->i2c_clk = clock;
    }

    if (own == 0){
      if(ch==0){
	pi2c->i2c_own = DEFAULT_OWN_CH0;
      }else{
	pi2c->i2c_own = DEFAULT_OWN_CH1;
      }
    }else{
      pi2c->i2c_own = own;
    }

    pi2c->restart = 0;
    /* not the transfer mode */
    pi2c->trns_mode = -1;
    /* TC90400_TENBIT */
    pi2c->tenbit_addr = 0;

    i2c_data->data = (void *)pi2c;

    init_waitqueue_head(&i2c_wait[ch]);
    init_waitqueue_head(&pi2c->tx_wait);
    init_waitqueue_head(&pi2c->rx_wait);

    spin_lock_init(&pi2c->lock);
    spin_lock_init(&pi2c->tx_lock);
    spin_lock_init(&pi2c->rx_lock);

    i2c_tc90400_ops[ch].data = i2c_data;
    if (ch == 0) {
#ifdef CONFIG_I2C_TC90400_CH0_S
      i2c_tc90400_ops[ch].ms_mode = 1; /* slave mode */
#else
      i2c_tc90400_ops[ch].ms_mode = 0; /* master mode */
#endif
    }
    else {
#ifdef CONFIG_I2C_TC90400_CH1_S
      i2c_tc90400_ops[ch].ms_mode = 1; /* slave mode */
#else
      i2c_tc90400_ops[ch].ms_mode = 0; /* master mode */
#endif
    }

    if (i2c_hw_resrc_init(&i2c_tc90400_ops[ch]) == 0) {
      if (i2c_tc90400_add_bus(&i2c_tc90400_ops[ch]) < 0){
	return -ENODEV;
      }
    } else {
      return -ENODEV;
    }
    printk(KERN_INFO " found device at %#x irq %d.\n", 
	   pi2c->i2c_base, pi2c->i2c_irq);

    init_completion(&pi2c->tx_comp);
    init_completion(&pi2c->rx_comp);
  }

  return 0;
}


static void i2c_tc90400_exit(void)
{
  int i;

DEB_START

  for(i=0; i<NUM_OF_CH; i++){
    i2c_tc90400_del_bus(&i2c_tc90400_ops[i]);
    i2c_tc90400_release(i);
  }
}

EXPORT_NO_SYMBOLS;

/* If modules is NOT defined when this file is compiled, then the MODULE_*
 * macros will resolve to nothing
 */
MODULE_AUTHOR("MontaVista Software <www.mvista.com>");
MODULE_DESCRIPTION("I2C-Bus adapter routines for TC90400 I2C bus adapter");
MODULE_LICENSE("GPL");

MODULE_PARM(clock, "i");
MODULE_PARM(own, "i");
/* MODULE_PARM(i2c_debug,"i"); */


/* Called when module is loaded or when kernel is intialized.
 * If MODULES is defined when this file is compiled, then this function will
 * resolve to init_module (the function called when insmod is invoked for a
 * module).  Otherwise, this function is called early in the boot, when the
 * kernel is intialized.  Check out /include/init.h to see how this works.
 */
module_init(i2c_tc90400_init);

/* Resolves to module_cleanup when MODULES is defined. */
module_exit(i2c_tc90400_exit);
