/*
 * linux/drivers/i2c/i2c-algo-tc90400.c : Reference No : 43PJ292-20051019-QFP_BOARD01-000
 *
 * Internal I2C driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
   -------------------------------------------------------------------------
   Hai-Pao Fan, MontaVista Software, Inc.
   hpfan@mvista.com or source@mvista.com

   Copyright 2000 MontaVista Software Inc.

   ---------------------------------------------------------------------------
   This file was highly leveraged from i2c-algo-pcf.c, which was created
   by Simon G. Vogl and Hans Berglund:


     Copyright (C) 1995-1997 Simon G. Vogl
                   1998-2000 Hans Berglund

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.			     */
/* ------------------------------------------------------------------------- */

/* With some changes from Kyti Mkki <kmalkki@cc.hut.fi> and 
   Frodo Looijaard <frodol@dds.nl> ,and also from Martin Bailey
   <mbailey@littlefeet-inc.com> */


/* #include <linux/config.h> */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/ioport.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <asm/irq.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-tc90400.h>
#include <linux/i2c-adap-tc90400.h>
#include "i2c-tc90400.h"

/* ----- global defines ----------------------------------------------- */
/*#define TC90400_I2C_DEBUG*/

#if defined(TC90400_I2C_DEBUG)
#define DEB_START	printk(__FUNCTION__": start\n");
#define DEB_END		printk(__FUNCTION__": end\n");
#define DEB_SNAP(x...)	printk(__FUNCTION__": "/*##*/x);
#define DEB_SNAP2(x...)	printk(/*##*/x);
#else
#define DEB_START
#define DEB_END
#define DEB_SNAP(x...)
#define DEB_SNAP2(x...)
#endif

/* debug the protocol by showing transferred bits */
#define DEF_TIMEOUT 16

/* debugging - slow down transfer to have a look at the data .. 		*/
/* I use this with two leds&resistors, each one connected to sda,scl 	*/
/* respectively. This makes sure that the algorithm works. Some chips   */
/* might not like this, as they have an internal timeout of some mils	*/


/* ----- global variables ---------------------------------------------	*/

/* module parameters: */
/* static int i2c_debug=0; */

/* --- setting states on the bus with the right timing: ---------------	*/
#define TC90400_I2C_DATA(adap)		((struct i2c_tc90400 *)(adap->data))

#define i2c_outw(adap, reg, val)	adap->seti2c(reg, val)
#define i2c_inw(adap, reg)		adap->geti2c(reg)

#define i2c_getclk(adap)		adap->getclock(adap)
#define i2c_setclk(adap, val)		adap->setclock(adap,val)
#define i2c_getown(adap)		adap->getown(adap)
#define i2c_status(adap)		(TC90400_I2C_DATA(adap)->i2c_sts)
#define i2c_irq(adap)		        (TC90400_I2C_DATA(adap)->i2c_irq)
#define i2c_set_restart(adap)		(TC90400_I2C_DATA(adap)->restart = 1)
#define i2c_clr_restart(adap)		(TC90400_I2C_DATA(adap)->restart = 0)
/* TC90400_TENBIT */
#define i2c_set_tenbit(adap)            (TC90400_I2C_DATA(adap)->tenbit_addr = 1)
#define i2c_clr_tenbit(adap)            (TC90400_I2C_DATA(adap)->tenbit_addr = 0)
/* TC90400_TENBIT */
#define i2c_start(adap)			adap->start(adap)
#define i2c_stop(adap)			adap->stop(adap)
#define i2c_reset(adap)			adap->reset(adap)
/* TC90400_TENBIT */
#define i2c_send(adap)			adap->send(adap)

#define i2c_txlock(adap)		spin_lock(&(TC90400_I2C_DATA(adap)->tx_lock))
#define i2c_txunlock(adap)		spin_unlock(&(TC90400_I2C_DATA(adap)->tx_lock))
#define i2c_txset(adap, buf, len)	i2c_txlock(adap);	                \
					TC90400_I2C_DATA(adap)->tx_buf = buf;	\
					TC90400_I2C_DATA(adap)->tx_size = len;	\
					TC90400_I2C_DATA(adap)->tx_count = 0;	\
					TC90400_I2C_DATA(adap)->trns_mode = 0;	\
					i2c_txunlock(adap);
#define i2c_txdata(adap, reg, val)      i2c_txlock(adap);	                \
					TC90400_I2C_DATA(adap)->trx_status=SLV_WRITE; \
					i2c_outw(adap, reg, val);               \
					TC90400_I2C_DATA(adap)->tx_count++;	\
					i2c_txunlock(adap);
#define i2c_txclr(adap)	i2c_txlock(adap);	                \
					TC90400_I2C_DATA(adap)->tx_buf = 0;	\
					TC90400_I2C_DATA(adap)->tx_size = 0;	\
					TC90400_I2C_DATA(adap)->tx_count = 0;	\
					i2c_txunlock(adap);
#define i2c_txget(adap, len)		len = TC90400_I2C_DATA(adap)->tx_count;
#define i2c_txwait(adap)		wait_for_completion(&(TC90400_I2C_DATA(adap)->tx_comp))
#define i2c_tx_timeout(adap, time, ret)	ret = interruptible_sleep_on_timeout(&(TC90400_I2C_DATA(adap)->tx_wait), time)
#define i2c_tx_notimeout(adap)		interruptible_sleep_on(&(TC90400_I2C_DATA(adap)->tx_wait))

#define i2c_rxlock(adap)		spin_lock(&(TC90400_I2C_DATA(adap)->rx_lock))
#define i2c_rxunlock(adap)		spin_unlock(&(TC90400_I2C_DATA(adap)->rx_lock))
#define i2c_rxset(adap, buf, len)	i2c_rxlock(adap);			\
					TC90400_I2C_DATA(adap)->rx_buf = buf;	\
					TC90400_I2C_DATA(adap)->rx_size = len;	\
					TC90400_I2C_DATA(adap)->rx_count = 0;	\
					TC90400_I2C_DATA(adap)->trns_mode = 1;	\
					i2c_rxunlock(adap);
#define i2c_rxdata(adap, reg)           i2c_rxlock(adap);	                \
					TC90400_I2C_DATA(adap)->trx_status=SLV_READ; \
					i2c_inw(adap, reg);               \
					i2c_rxunlock(adap);
#define i2c_rxget(adap, len)		len = TC90400_I2C_DATA(adap)->rx_count;
#define i2c_rxwait(adap)		wait_for_completion(&(TC90400_I2C_DATA(adap)->rx_comp))
#define i2c_rx_timeout(adap, time, ret)	ret = interruptible_sleep_on_timeout(&(TC90400_I2C_DATA(adap)->rx_wait), time)
#define i2c_rx_notimeout(adap)		interruptible_sleep_on(&(TC90400_I2C_DATA(adap)->rx_wait))
#define i2c_rxclr(adap)	i2c_rxlock(adap);			\
					TC90400_I2C_DATA(adap)->rx_buf = 0;	\
					TC90400_I2C_DATA(adap)->rx_size = 0;	\
					TC90400_I2C_DATA(adap)->rx_count = 0;	\
					i2c_rxunlock(adap);

#define i2c_stsget(adap, sts)		sts = TC90400_I2C_DATA(adap)->trx_status;
#define i2c_stsclr(adap)		TC90400_I2C_DATA(adap)->trx_status = 0;
#if 0 /* not change ACKbit */
#define i2c_timeout(adap)		i2c_outw(adap, TC90400_I2CCR0(adap->ch),(I2C_CNT0|i2c_getclk(adap))); \
					TC90400_I2C_DATA(adap)->trns_mode = -1;
#else
#define i2c_timeout(adap)		TC90400_I2C_DATA(adap)->trns_mode = -1;
#endif

static struct i2c_log i2c_log_buf;

/* --- other auxiliary functions --------------------------------------	*/

/*
 * Puts this process to sleep for a period equal to timeout 
 */
static inline void i2c_sleep(unsigned long timeout)
{

DEB_START

  schedule_timeout( timeout * HZ);
}

static int i2c_init (struct i2c_algo_tc90400_data *adap)
{

DEB_START

  /* Software reset */
  i2c_reset(adap);

  while((i2c_inw(adap, TC90400_I2CCR0(adap->ch)) & I2CCR0_SWRMON)==0){
    mdelay(1);
  };

DEB_SNAP2("i2c_getclk is 0x%x, \n",i2c_getclk(adap))
  /* Set BC/SCLK */
#if 0 /* Set no Ack for slave device */
  i2c_outw(adap, TC90400_I2CCR0(adap->ch),(I2C_CNT0|I2CCR0_ACK|i2c_getclk(adap)));
#else
  i2c_outw(adap, TC90400_I2CCR0(adap->ch),(I2C_CNT0|i2c_getclk(adap)));
#endif

  /* Set slave address & ARS:0 */
  if( adap->ch == 0 ){
    i2c_outw(adap, TC90400_I2CAR(adap->ch),(DEFAULT_OWN_CH0<<1));
  }else{
    i2c_outw(adap, TC90400_I2CAR(adap->ch),(DEFAULT_OWN_CH1<<1));
  }
  /* Set inital mode */
  i2c_outw(adap, TC90400_I2CCR1(adap->ch),I2CCR1_PIN|I2CCR1_SBIM_I2C);

  /* Set Internal clock enable (Boud rate register 1) */
  i2c_outw(adap, TC90400_I2CBR1(adap->ch),I2CBR1_CKEN);

DEB_SNAP2("i2c_init: Initialized I2C on TC90400 CR1 Reg is 0x%x\n",
	  i2c_inw(adap, TC90400_I2CCR1(adap->ch)))

  return 0;
}


/* ----- Utility functions
 */

/* Verify the device we want to talk to on the I2C bus really exists. */
static inline int try_address(struct i2c_algo_tc90400_data *adap,
		       unsigned int addr, int retries)
{
  int i, ret = -1;

DEB_START
  for (i=0;i<retries;i++) {
    i2c_outw(adap, TC90400_I2CDBR(adap->ch), addr);
    i2c_start(adap);

    if (adap->waitforack(adap, 1, DEF_TIMEOUT) >= 0) {
      ret = 0;
      break;	/* success! */
    }
    i2c_stop(adap);
    udelay(adap->udelay);
  }
DEB_SNAP2("needed %d retries for %x\n", i, addr)
  return ret;
}

static inline int i2c_doAddress(struct i2c_algo_tc90400_data *adap,
                                struct i2c_msg *msg, int retries) 
{
  unsigned short flags = msg->flags;
  unsigned char addr;
  int ret;

DEB_START

  /* set slave address */
  i2c_outw(adap, TC90400_I2CCR0(adap->ch), (I2CCR0_BC | I2CCR0_ACK | i2c_getclk(adap)));
  if ( (flags & I2C_M_TEN)  ) { 
    i2c_set_tenbit(adap);
    /* a ten bit address */
    addr = 0xf0 | (( msg->addr >> 7) & 0x06);
DEB_SNAP2("addr0: %x\n", addr)
    /* try extended address code...*/
    ret = try_address(adap, addr, retries);
    if (ret!=0) {
      printk(KERN_ERR "died at extended address code.\n");
      return -EREMOTEIO;
    }
    /* the remaining 8 bit address */
    i2c_outw(adap, TC90400_I2CDBR(adap->ch), msg->addr & 0xff);
    /* Status check comes here */
    if (adap->waitforack(adap, 1, DEF_TIMEOUT) < 0) {
      printk(KERN_ERR "died at 2nd address code.\n");
      return -EREMOTEIO;
    }
    i2c_clr_tenbit(adap);
    if ( flags & I2C_M_RD ) { /* read */
      /* okay, now switch into reading mode */
      addr |= 0x01;
      i2c_outw(adap, TC90400_I2CDBR(adap->ch), addr);
      i2c_start(adap);
    }
    else { /* write */
      i2c_send(adap);
    }
  } else { /* normal 7bit address	*/
    addr = (msg->addr & 0x7f) << 1;
    if (flags & I2C_M_RD )
      addr |= 1;
    i2c_outw(adap, TC90400_I2CDBR(adap->ch), addr);
    /* set start condition */
    if(adap->waitforbb(adap,DEF_TIMEOUT)){
      printk(KERN_ERR "i2c_doAddress : Timeout!\n");
      return -EIO;
    }
    i2c_start(adap);
  }

  return 0;
}

static int i2c_sendbytes(struct i2c_algo_tc90400_data *adap, struct i2c_msg *pmsg, int retries)
{
  int		wrcount, ret;
  long            timeout;

DEB_START

  wrcount = 0;
  i2c_txset(adap, pmsg->buf, pmsg->len);
  i2c_stsclr(adap);

  /* Load address */
  ret = i2c_doAddress(adap, pmsg, retries);
  if (ret)
    return -EIO;

DEB_SNAP2("wait for tx completed\n")
  /* wait for completion */
  i2c_txwait(adap);

  /* check restart */
  if (TC90400_I2C_DATA(adap)->restart == 1) {
    timeout = adap->waitforbb(adap,DEF_TIMEOUT);
    if (timeout) {
      printk(KERN_ERR "i2c_sendbytes : Timeout!\n");
      return -EIO;
    }
    timeout = adap->waitforlrb(adap,DEF_TIMEOUT);
    if (timeout) {
      printk(KERN_ERR "i2c_sendbytes : Timeout!\n");
      return -EIO;
    }
  }

  i2c_txget(adap, wrcount);

  return wrcount;
}

static int i2c_readbytes(struct i2c_algo_tc90400_data *adap, struct i2c_msg *pmsg, int retries)
{
  int		rdcount, ret;
  long            timeout;

DEB_START
  rdcount = 0;

  /* set internal data */
  i2c_rxset(adap,pmsg->buf, pmsg->len);
  i2c_stsclr(adap);

  /* Load address */
  ret = i2c_doAddress(adap, pmsg, retries);
  if (ret)
    return -EIO;

DEB_SNAP2("wait for rx completed\n")
  /* wait for completion */
  i2c_rxwait(adap);

  /* check restart */
  if (TC90400_I2C_DATA(adap)->restart == 1) {
    timeout = adap->waitforbb(adap,DEF_TIMEOUT);
    if (timeout) {
      printk(KERN_ERR "i2c_readbytes : Timeout!\n");
      return -EIO;
    }
    timeout = adap->waitforlrb(adap,DEF_TIMEOUT);
    if (timeout) {
      printk(KERN_ERR "i2c_readbytes : Timeout!\n");
      return -EIO;
    }
  }

  i2c_rxget(adap, rdcount);

  return rdcount;
}


/* Description: Prepares the controller for a transaction (clearing status
 * registers, data buffers, etc), and then calls either i2c_readbytes or
 * i2c_sendbytes to do the actual transaction.
 *
 * still to be done: Before we issue a transaction, we should
 * verify that the bus is not busy or in some unknown state.
 */
static int i2c_xfer(struct i2c_adapter *i2c_adap,
		    struct i2c_msg msgs[],
		    int num)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;
  struct i2c_msg *pmsg, *pmsg_n;
  int i = 0;
  int ret = 0;
  int timeout;

DEB_START
DEB_SNAP2("num is 0x%x\n",num)

  for(i=0; i<num; i++){
    pmsg = &msgs[i];

    i2c_clr_restart(adap);
    if( (num - i) > 1 ){
      pmsg_n = &msgs[i+1];
      /* change direction */
      if( (pmsg->flags==I2C_M_RD) == (pmsg_n->flags==I2C_M_WR) ){
DEB_SNAP2("set restart!\n")
        i2c_set_restart(adap);
      }else if( (pmsg->flags==I2C_M_WR) == (pmsg_n->flags==I2C_M_RD) ){
DEB_SNAP2("set restart!\n")
        i2c_set_restart(adap);
      }
    }

    if(!pmsg->len) {
      printk(KERN_ERR "i2c_xfer: read/write length is 0\n");
      return -EINVAL;
    }

    /* Wait for any pending transfers to complete */
    timeout = adap->waitforbb(adap,DEF_TIMEOUT);
    if (timeout) {
      printk(KERN_ERR "i2c_xfer : Timeout!\n");
      return -EIO;
    }

DEB_SNAP2("i2c_xfer: Msg %d, addr=0x%x, flags=0x%x, len=%d\n",
	  i, msgs[i].addr, msgs[i].flags, msgs[i].len)

    if(pmsg->flags & I2C_M_RD){ /* Read */
      ret = i2c_readbytes(adap, pmsg, i2c_adap->retries);
    }else {	/* Write */ 
      udelay(1000);
      ret = i2c_sendbytes(adap, pmsg, i2c_adap->retries);
    }

    if (ret != pmsg->len){
      if (ret >= 0) pmsg->len = ret;
DEB_SNAP2("i2c_xfer: error or fail on read/write %d bytes.\n",ret)
    }else{
DEB_SNAP2("i2c_xfer: read/write %d bytes.\n",ret)
    }
  }

  if (ret <= 0)
    return ret;
  return i;
}

static int i2c_xfer_clk(struct i2c_adapter *i2c_adap,
			struct i2c_msg_clk msgs[],
			int num)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;
  struct i2c_msg_clk *pmsg, *pmsg_n;
  int i = 0;
  int ret = 0;
  int timeout;

DEB_START
DEB_SNAP2("num is 0x%x\n",num)

  for(i=0; i<num; i++){
    pmsg = &msgs[i];

    i2c_setclk(adap, pmsg->clock);
    i2c_clr_restart(adap);
    if( (num - i) > 1 ){
      pmsg_n = &msgs[i+1];
      /* change direction */
      if( (pmsg->flags==I2C_M_RD) == (pmsg_n->flags==I2C_M_WR) ){
DEB_SNAP2("set restart!\n")
        i2c_set_restart(adap);
      }else if( (pmsg->flags==I2C_M_WR) == (pmsg_n->flags==I2C_M_RD) ){
DEB_SNAP2("set restart!\n")
        i2c_set_restart(adap);
      }
    }

    if(!pmsg->len) {
      printk(KERN_ERR "i2c_xfer_clk: read/write length is 0\n");
      return -EINVAL;
    }

    /* Wait for any pending transfers to complete */
    timeout = adap->waitforbb(adap,DEF_TIMEOUT);
    if (timeout) {
      printk(KERN_ERR "i2c_xfer_clk : Timeout!\n");
      return -EIO;
    }

DEB_SNAP2("i2c_xfer_clk: Msg %d, addr=0x%x, flags=0x%x, len=%d clk=%d\n",
	  i, msgs[i].addr, msgs[i].flags, msgs[i].len, msgs[i].clock)

    if(pmsg->flags & I2C_M_RD){ /* Read */
      ret = i2c_readbytes(adap, (struct i2c_msg *)pmsg, i2c_adap->retries);
    }else {	/* Write */ 
      udelay(1000);
      ret = i2c_sendbytes(adap, (struct i2c_msg *)pmsg, i2c_adap->retries);
    }

    if (ret != pmsg->len){
      if (ret >= 0) pmsg->len = ret;
DEB_SNAP2("i2c_xfer_clk: error or fail on read/write %d bytes.\n",ret)
    }else{
DEB_SNAP2("i2c_xfer_clk: read/write %d bytes.\n",ret)
    }
  }

  if (ret <= 0)
    return ret;
  return i;
}

static void i2cslv_log(int status, int num, unsigned char *buf, unsigned char reg, int trx)
{
  struct i2c_log *log_ptr = &i2c_log_buf;
  int lognum;

  if (num > 16) /* log size check */
    lognum = 16;
  else
    lognum = num;

  if (trx) { /* receive data */
    memcpy(log_ptr->rx_data, buf, lognum);
    log_ptr->rx_num = num;
  }
  else { /* transfer data */
    memcpy(log_ptr->tx_data, buf, lognum);
    log_ptr->tx_num = num;
  }

  log_ptr->status = status;
  log_ptr->cr1 = reg;
}

static int i2c_slave_send(struct i2c_adapter *i2c_adap,
			  struct i2c_msg_clk msgs[],
			  int num)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;
  struct i2c_msg_clk *pmsg;
  int i = 0;
  int ret;
  int wrcount, status;
  int flags;
  long timeout;

DEB_START
DEB_SNAP2("num is 0x%x\n",num)
  pmsg = &msgs[i];
  ret = 0;
  timeout = -1; 
  i2c_setclk(adap, pmsg->clock);

  if(!pmsg->len) {
    printk(KERN_ERR "i2c_slave_send: write length is 0\n");
    return -EINVAL;
  }

  save_flags(flags);
  cli();
  /* get status */
  i2c_stsget(adap, status);
  /* Check Not Bus Free (confirm Bus Busy condition) or error status */
  if ((!(i2c_inw(adap, TC90400_I2CCR1(adap->ch)) & I2CCR1_BUSBUSY)) ||
      (status == SLV_SEQERROR)) {
    i2c_timeout(adap); /* set no Ack, trx mode */
    ret = -EILSEQ;
    printk(KERN_ERR "i2c_slave_send: bus free or error occured\n");
    goto end;
  }

DEB_SNAP2("i2c_slave_send: Msg %d, len=%d clk=%d timeout=%dms\n",
	  i, msgs[i].len, msgs[i].clock,i2c_adap->timeout)
  i2c_txset(adap, pmsg->buf, pmsg->len);
#if 0 /* not change ACKbit */
  /* Set Ack mode */
  i2c_outw(adap,TC90400_I2CCR0(adap->ch),(I2CCR0_BC|I2CCR0_ACK|i2c_getclk(adap)));
#endif
DEB_SNAP2("wait for tx completed or %d0ms\n", (i2c_adap->timeout+5)/10)
  /* write first byte data */
  i2c_txdata(adap,TC90400_I2CDBR(adap->ch),*pmsg->buf);
  restore_flags(flags);

  /* wait for transfer */
  if ((i2c_adap->timeout+5)/10) {
    ret = wait_event_interruptible_timeout((TC90400_I2C_DATA(adap)->tx_wait),(TC90400_I2C_DATA(adap)->trns_mode<0),(i2c_adap->timeout+5)/10);
  }
  else {
    ret = wait_event_interruptible((TC90400_I2C_DATA(adap)->tx_wait),(TC90400_I2C_DATA(adap)->trns_mode<0));
  }

  save_flags(flags);
  cli();
DEB_SNAP2("wakeup tx\n")
  /* get count & status */
  i2c_txget(adap, wrcount);
  i2c_stsget(adap, status);

  if (status == SLV_WRITEEND) {
    TC90400_I2C_DATA(adap)->trns_mode = 1;
    i2c_stsclr(adap);
    /* interrupt clear (for PIN clear) */
    i2c_outw(adap, TC90400_I2CCR1(adap->ch), (I2CCR1_SBIM_I2C | I2CCR1_PIN));
  }
  else if (status == SLV_WRITE) { /* timeout error */
    ret = -ETIME;
  }
  if (ret < 0) { /* signal or timeout */
    i2c_timeout(adap); /* set trx mode */
    goto end;
  }

  if (status == SLV_SEQERROR) { /* illegal error */
    ret = -EILSEQ;
  }
  else if (status == SLV_UNDERRFLOW) { /* buffer underflow */
    ret = -ENODATA;
  }
  else { /* write normal end or next write */
    ret = wrcount;
  }

 end:
  i2cslv_log(status, wrcount, pmsg->buf, i2c_status(adap), 0); /* save to log buffer */
  i2c_txclr(adap);
  restore_flags(flags);

  if (ret < 0) {
DEB_SNAP2("i2c_slave_send: error or fail status:%d.\n",ret)
  }
  else {
    pmsg->len = ret;
DEB_SNAP2("i2c_slave_send: write %d bytes. timeout:%dms\n",ret, timeout<0? timeout:timeout*10)
  }
  return ret;
}

static int i2c_slave_recv(struct i2c_adapter *i2c_adap,
			  struct i2c_msg_clk msgs[],
			  int num)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;
  struct i2c_msg_clk *pmsg;
  int i = 0;
  int ret;
  int rdcount, status;
  int flags;
  long timeout;

DEB_START
DEB_SNAP2("num is 0x%x\n",num)
  pmsg = &msgs[i];
  ret = 0;
  timeout = -1; 
  i2c_setclk(adap, pmsg->clock);

  if(!pmsg->len) {
    printk(KERN_ERR "i2c_slave_recv: read length is 0\n");
    return -EINVAL;
  }

DEB_SNAP2("i2c_slave_recv: Msg %d, len=%d clk=%d timeout=%dms\n",
	  i, msgs[i].len, msgs[i].clock, i2c_adap->timeout)

  save_flags(flags);
  cli();
  /* set internal data */
  i2c_rxset(adap,pmsg->buf, pmsg->len);
DEB_SNAP2("wait for slave address completed\n")
#if 0 /* not change ACKbit */
  /* Set Ack mode */
  i2c_outw(adap,TC90400_I2CCR0(adap->ch),(I2CCR0_BC|I2CCR0_ACK|i2c_getclk(adap)));
#endif
  restore_flags(flags);

  /* wait for slave address */
  if ((i2c_adap->timeout+5)/10) {
    ret = wait_event_interruptible_timeout((TC90400_I2C_DATA(adap)->rx_wait),(TC90400_I2C_DATA(adap)->trx_status),(i2c_adap->timeout+5)/10);
  }
  else {
    ret = wait_event_interruptible((TC90400_I2C_DATA(adap)->rx_wait),(TC90400_I2C_DATA(adap)->trx_status));
  }

  save_flags(flags);
  cli();
  /* get count & status */
  i2c_rxget(adap, rdcount);
  i2c_stsget(adap, status);
  if (!status) { /* timeout error */
    ret = -ETIME;
  }

  if (ret < 0) { /* signal */
    i2c_timeout(adap); /* set trx mode */
    goto end;
  }

DEB_SNAP2("wakeup slv\n")

  /* slave read */
  if (status == SLV_NEXTREAD) {
DEB_SNAP2("wait for rx completed or %d0ms\n", (i2c_adap->timeout+5)/10)
    /* read dummy data (for PIN clear) */
    i2c_rxdata(adap,TC90400_I2CDBR(adap->ch));
    restore_flags(flags);

    /* wait for receive */
    if ((i2c_adap->timeout+5)/10) {
      ret = wait_event_interruptible_timeout((TC90400_I2C_DATA(adap)->rx_wait),(TC90400_I2C_DATA(adap)->trns_mode<0),(i2c_adap->timeout+5)/10);
    }
    else {
      ret = wait_event_interruptible((TC90400_I2C_DATA(adap)->rx_wait),(TC90400_I2C_DATA(adap)->trns_mode<0));
    }

    save_flags(flags);
    cli();
DEB_SNAP2("wakeup rx\n")
    /* get count & status */
    i2c_rxget(adap, rdcount);
    i2c_stsget(adap, status);

    if ((status & SLV_READ) == SLV_READ) { /* timeout error */
      ret = -ETIME;
    }
    if (ret < 0) { /* signal or timeout */
      i2c_timeout(adap); /* set trx mode */
      goto end;
    }
    restore_flags(flags);

    if (status == SLV_CHECKSUM) { /* checksum error */
      mdelay(3);
      ret = -EIO;
    }

    save_flags(flags);
    cli();
    if ((status == SLV_READEND)||(status == SLV_CHECKSUM)) {
      TC90400_I2C_DATA(adap)->trns_mode = 1;
      i2c_stsclr(adap);
      pmsg->buf[rdcount] = i2c_inw(adap,TC90400_I2CDBR(adap->ch));
      rdcount++;
    }
  }

  if (status == SLV_SEQERROR) { /* illegal error */
    ret = -EILSEQ;
  }
  if (ret >= 0) { /* read normal end or next write */
    ret = rdcount;
  }

 end:
  i2cslv_log(status, rdcount, pmsg->buf, i2c_status(adap), 1); /* save to log buffer */
  i2c_rxclr(adap);
  /* GPIO3_1 -> L */
  TC90400_WR(TC90400_XPIODIR,(TC90400_RD(TC90400_XPIODIR)|0x02000000)); /* direction output */
  TC90400_WR(TC90400_XPIODO,TC90400_RD(TC90400_XPIODO)&~0x02000000); /* output H->L */
  restore_flags(flags);

  if (ret < 0) {
DEB_SNAP2("i2c_slave_recv: error or fail status:%d.\n",ret)
  }
  else {
    pmsg->len = ret;
DEB_SNAP2("i2c_slave_recv: read %d bytes. timeout:%dms\n",ret, timeout<0? timeout:timeout*10)
  }
  return ret;
}

int i2c_algo_open(struct i2c_adapter *i2c_adap)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;
  long timeout;
  short status;

#if 0
  timeout = adap->waitforbb(adap,DEF_TIMEOUT);
  if (timeout) {
    printk(KERN_ERR "I2C open Bus busy!\n");
  }
#else
  timeout = 0x28000; /* about 48ms */
  do {
    status = i2c_inw(adap, TC90400_I2CCR1(adap->ch));
    timeout--;
  } while ((timeout > 0) && (status & I2CCR1_BUSBUSY));
  if (timeout <= 0) {
    printk(KERN_ERR "I2C open Bus busy!\n");
  }
#endif
  /* Set Ack mode */
  i2c_outw(adap,TC90400_I2CCR0(adap->ch),(I2CCR0_BC|I2CCR0_ACK|i2c_getclk(adap)));
  i2c_stsclr(adap);
  TC90400_I2C_DATA(adap)->trns_mode = 1;

  return 0;
}

int i2c_algo_close(struct i2c_adapter *i2c_adap)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;
  long timeout;
  short status;

  TC90400_I2C_DATA(adap)->trns_mode = -1;
  i2c_stsclr(adap);
  /* interrupt clear (for PIN clear) */
  i2c_outw(adap, TC90400_I2CCR1(adap->ch), (I2CCR1_SBIM_I2C | I2CCR1_PIN));

#if 0
  timeout = adap->waitforbb(adap,DEF_TIMEOUT);
  if (timeout) {
    printk(KERN_ERR "I2C close Bus busy!\n");
  }
#else
  timeout = 0x28000; /* about 48ms */
  do {
    status = i2c_inw(adap, TC90400_I2CCR1(adap->ch));
    timeout--;
  } while ((timeout > 0) && (status & I2CCR1_BUSBUSY));
  if (timeout <= 0) {
    printk(KERN_ERR "I2C close Bus busy!\n");
  }
#endif
  /* Set no Ack mode */
  i2c_outw(adap,TC90400_I2CCR0(adap->ch),(I2CCR0_BC|i2c_getclk(adap)));
  return 0;
}


/* Implements device specific ioctls.  Higher level ioctls can
 * be found in i2c-core.c and are typical of any i2c controller (specifying
 * slave address, timeouts, etc).  These ioctls take advantage of any hardware
 * features built into the controller for which this algorithm-adapter set
 * was written.  These ioctls allow you to take control of the data and clock
 * lines and set the either high or low,
 * similar to a GPIO pin.
 */
static int algo_control(struct i2c_adapter *i2c_adap, 
			unsigned int cmd, unsigned long arg)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;
  int val;

DEB_START
  switch(cmd) {
  case I2C_SETBAUD:
    i2c_setclk(adap, arg);
    return 0;
  case I2C_GETBAUD:
    val = i2c_getclk(adap);
    if (copy_to_user((int *)arg, &val, sizeof(int)) < 0) {
      return -EFAULT;
    }
    return 0;
  case I2C_RESET:
    if (i2c_adap->trx_flg) {
      return -EBUSY; /* read or write isn't completion */
    }
    disable_irq(i2c_irq(adap));
    i2c_init(adap);
    i2c_txclr(adap);
    i2c_rxclr(adap);
    i2c_stsclr(adap);
    TC90400_I2C_DATA(adap)->trns_mode = 1;
    /* Set Ack mode */
    i2c_outw(adap,TC90400_I2CCR0(adap->ch),(I2CCR0_BC|I2CCR0_ACK|i2c_getclk(adap)));
    memset(&i2c_log_buf, 0, sizeof(struct i2c_log));
    enable_irq(i2c_irq(adap));
    return 0;
  case I2C_GETLOG:
    if (copy_to_user((int *)arg, &i2c_log_buf, sizeof(struct i2c_log)) < 0) {
      return -EFAULT;
    }
    return 0;
  default:
    return -ENOIOCTLCMD;
  }
}


static u32 i2c_func(struct i2c_adapter *i2c_adap)
{
	/* I2C_FUNC_10BIT_ADDR is supported, but confirmation isn't complete. */
        return I2C_FUNC_I2C /* I2C_FUNC_10BIT_ADDR|I2C_FUNC_PROTOCOL_MANGLING*/; 
}

/* -----exported algorithm data: -------------------------------------	*/

static struct i2c_algorithm i2c_algo = {
	"TC90400 I2C algorithm",
	I2C_ALGO_TC90400,
	i2c_xfer,			/* master_xfer		*/
	NULL,				/* smbus_xfer		*/
	i2c_slave_send,			/* slave_send		*/
	i2c_slave_recv,			/* slave_recv		*/
	algo_control,		        /* ioctl		*/
	i2c_func,			/* functionality	*/
	i2c_xfer_clk,			/* master_xfer_clk	*/
};


/* 
 * registering functions to load algorithms at runtime 
 */
int i2c_tc90400_add_bus(struct i2c_adapter *i2c_adap)
{
  struct i2c_algo_tc90400_data *adap = i2c_adap->algo_data;

DEB_START

DEB_SNAP2("i2c-algo-tc90400: hw routines for %s registered.\n",
	  i2c_adap->name)

  /* register new adapter to i2c module... */

  i2c_adap->id |= i2c_algo.id;
  i2c_adap->algo = &i2c_algo;

  i2c_adap->timeout = 100;	/* default values, should	*/
  i2c_adap->retries = 3;		/* be replaced by defines	*/
  i2c_adap->flags = 0;

#ifdef MODULE
  MOD_INC_USE_COUNT;
#endif

  i2c_add_adapter(i2c_adap);
  i2c_init(adap);
  i2c_stsclr(adap);
  i2c_txclr(adap);
  i2c_rxclr(adap);

  return 0;
}


int i2c_tc90400_del_bus(struct i2c_adapter *i2c_adap)
{
  int res;

DEB_START

  if ((res = i2c_del_adapter(i2c_adap)) < 0)
    return res;
DEB_SNAP2("i2c-algo-tc90400: adapter unregistered: %s\n",i2c_adap->name)

#ifdef MODULE
  MOD_DEC_USE_COUNT;
#endif
  return 0;
}


int __init i2c_algo_i2c_init (void)
{

DEB_START

  printk(KERN_INFO "TC90400 i2c algorithm module\n");

  /* Set IRC Control Reg. to I2C edge detect */
 TC90400_WR(TC90400_INTCONT2, TC90400_RD(TC90400_INTCONT2)|TC90400_INTCONT2_I2C0); /* i2c0 edge */
 TC90400_WR(TC90400_INTCONT1, TC90400_RD(TC90400_INTCONT1)|TC90400_INTCONT1_I2C1); /* i2c1 edge */

  return 0;
}


void i2c_algo_i2c_exit(void)
{

DEB_START

  return;
}


EXPORT_SYMBOL(i2c_tc90400_add_bus);
EXPORT_SYMBOL(i2c_tc90400_del_bus);

/* The MODULE_* macros resolve to nothing if MODULES is not defined
 * when this file is compiled.
 */
MODULE_AUTHOR("MontaVista Software <www.mvista.com>");
MODULE_DESCRIPTION("TC90400 i2c algorithm");
MODULE_LICENSE("GPL");

/* MODULE_PARM(i2c_debug,"i"); */

/* MODULE_PARM_DESC(i2c_debug,
   "debug level - 0 off; 1 normal; 2,3 more verbose; 9 i2c-protocol"); */


/* This function resolves to init_module (the function invoked when a module
 * is loaded via insmod) when this file is compiled with MODULES defined.
 * Otherwise (i.e. if you want this driver statically linked to the kernel),
 * a pointer to this function is stored in a table and called
 * during the intialization of the kernel (in do_basic_setup in /init/main.c) 
 *
 * All this functionality is complements of the macros defined in linux/init.h
 */
module_init(i2c_algo_i2c_init);


/* If MODULES is defined when this file is compiled, then this function will
 * resolved to cleanup_module.
 */
module_exit(i2c_algo_i2c_exit);
