/*
 * linux/drivers/i2c/i2c-tc90400.h : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal I2C driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/*
   --------------------------------------------------------------------
   Hai-Pao Fan, MontaVista Software, Inc.
   hpfan@mvista.com or source@mvista.com

   Copyright 2001 MontaVista Software Inc.

 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.

 */


#ifndef I2C_TC90400_H
#define I2C_TC90400_H 1

#include	<asm/tc90400/tc90400.h>

/* I2C channel number */
#define	NUM_OF_CH		2

/* I2C Registers */
#define	TC90400_I2CCR0(ch)	(TC90400_I2C_BASE + (ch * 0x40))		/* Control Register 0 */
#define	TC90400_I2CDBR(ch)	(TC90400_I2C_BASE + 0x04 + (ch * 0x40))	/* Data Buffer Register */
#define	TC90400_I2CAR(ch)	(TC90400_I2C_BASE + 0x08 + (ch * 0x40))	/* Slave Address Register */
#define	TC90400_I2CCR1(ch)	(TC90400_I2C_BASE + 0x0c + (ch * 0x40))	/* Control Register 1 */
#define	TC90400_I2CBR0(ch)	(TC90400_I2C_BASE + 0x10 + (ch * 0x40))	/* Baud rate Register 0 */
#define	TC90400_I2CBR1(ch)	(TC90400_I2C_BASE + 0x14 + (ch * 0x40))	/* Boud rate Register 1 */

/* Control Register 0 */
#define	I2CCR0_BC		0x0000
#define	I2CCR0_BC_1		0x0020
#define	I2CCR0_ACK		0x0010

/*  System Clock : 162MHz */
#define	I2CCR0_SCK_562K		0x0001		/* 562.5kHz */
#define	I2CCR0_SCK_297K		0x0002		/* 297.8kHz */
#define	I2CCR0_SCK_153K		0x0003		/* 153.4kHz */
#define	I2CCR0_SCK_77K9		0x0004		/* 77.9kHz */
#define	I2CCR0_SCK_39K2		0x0005		/* 39.2kHz */
#define	I2CCR0_SCK_19K7		0x0006		/* 19.7kHz */
/*  System Clock : 81MHz */
#define	I2CCR0_SCK_281K		0x0001		/* 281.25kHz */
#define	I2CCR0_SCK_148K		0x0002		/* 148.9kHz */
#define	I2CCR0_SCK_76K7		0x0003		/* 76.7kHz */
#define	I2CCR0_SCK_38K95	0x0004		/* 38.95kHz */
#define	I2CCR0_SCK_19K6		0x0005		/* 19.6kHz */
#define	I2CCR0_SCK_9K85		0x0006		/* 9.85kHz */

#define	I2CCR0_SWRMON		0x0001

#define	I2C_CNT0	        I2CCR0_BC	/* Control Register 0 default */

/* Data Buffer Register */
#define	I2CDBR_DATA		0x00ff

#define	I2CAR_SA		0x00fe
#define	I2CAR_ARS		0x0001

#define	I2C_AR

/* Control Register 1 */
#define	I2CCR1_MST		0x0080
#define	I2CCR1_TRX		0x0040
#define	I2CCR1_BUSBUSY		0x0020
#define	I2CCR1_START_CONDITON	0x0020
#define	I2CCR1_PIN		0x0010
#define	I2CCR1_IRQ_CLR		0x0010
#define	I2CCR1_AL		0x0008
#define	I2CCR1_AAS		0x0004
#define	I2CCR1_SBIM_I2C		0x0008
#define	I2CCR1_AD0		0x0002
#define	I2CCR1_LRB		0x0001
#define	I2CCR1_RESET1		0x0002
#define	I2CCR1_RESET2		0x0001

#define	I2C_CMD		I2CCR1_MST | I2CCR1_SBIM_I2C	/* Control Register 1 default */
#define	I2C_START	I2C_CMD | I2CCR1_TRX | I2CCR1_START_CONDITON | I2CCR1_IRQ_CLR
#define	I2C_STOP	I2C_CMD | I2CCR1_TRX | I2CCR1_IRQ_CLR
#define	I2C_WRITE	I2C_CMD | I2CCR1_TRX
#define	I2C_READ	I2C_CMD

/* Boud rate Register 1 */
#define	I2CBR1_CKEN		0x0080			/* internal cleck enable */

#define DEFAULT_BASE_CH0	(TC90400_I2C_BASE)
#define DEFAULT_BASE_CH1	(TC90400_I2C_BASE + 0x40)
#define TC90400_I2C_IO_SIZE	0x40
#define DEFAULT_IRQ_CH0		TC90400_IRQ_I2C0	/* 0:INT not use */
#define DEFAULT_CLOCK_CH0	I2CCR0_SCK_77K9		/* 162MHz/(2^n+4) */

#ifdef CONFIG_I2C_TC90400_CH0_0B
#define DEFAULT_OWN_CH0		0x0B			/* Ch0 my address */
#elif  CONFIG_I2C_TC90400_CH0_0F
#define DEFAULT_OWN_CH0		0x0F			/* Ch0 my address */
#else
#define DEFAULT_OWN_CH0		0x0F			/* Ch0 my address */
#endif

#define DEFAULT_IRQ_CH1		TC90400_IRQ_I2C1	/* 0:INT not use */
#define DEFAULT_CLOCK_CH1	I2CCR0_SCK_77K9		/* 162MHz/(2^n+4) */

#ifdef CONFIG_I2C_TC90400_CH1_0B
#define DEFAULT_OWN_CH1		0x0B			/* Ch1 my address */
#elif  CONFIG_I2C_TC90400_CH1_0F
#define DEFAULT_OWN_CH1		0x0F			/* Ch1 my address */
#else
#define DEFAULT_OWN_CH1		0x0B			/* Ch1 my address */
#endif

#endif  /* I2C_TC90400_H */
