/*
 * linux/drivers/mtd/maps/tc90400-flash.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal Nor-Flash driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2004 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#include <asm/io.h>
#ifdef CONFIG_TOSHIBA_TC90400GDRS
#include <asm/tc90400/tc90400gdrs.h>
#else
#include <asm/tc90400/tc90400.h>
#endif /* CONFIG_TOSHIBA_TC90400GDRS */
#if (defined(CONFIG_MTD_PARTITIONS) || defined(CONFIG_MTD_PARTITIONS_MODULE))
#define USE_PARTITION
#else
#undef USE_PARTITION
#endif

#undef DEBUG
#ifdef DEBUG
#define printd(arg...)	printk(arg)
#else
#define printd(arg...)
#endif

#if 0
#define TC90400_NOR_FLASH_MAP		0x1fc00000 /* change for chip#1 */
#define TC90400_NOR_FLASH_SIZE		0x00200000 /* change for chip#1 */
#endif

static __u8 tc90400_read8 (struct map_info *map, unsigned long ofs)
{
	register __u8 val;
	val = *((volatile __u8*)(map->map_priv_1 + ofs));
	printd ("%08x -> %02x\n", (map->map_priv_1 + ofs), (__u16)val);
	return (val);
}

static void tc90400_write8 (struct map_info *map, __u8 d, unsigned long adr)
{
	printd ("%08x <- %02x\n", (map->map_priv_1 + adr), (__u16)d);
	*((volatile __u8*)(map->map_priv_1 + adr)) = d;
}

static __u16 tc90400_read16 (struct map_info *map, unsigned long ofs)
{
	register __u16 val;
	val = *((volatile __u16*)(map->map_priv_1 + ofs));
	printd ("%08x -> %04x\n", (map->map_priv_1 + ofs), (__u32)val);
	return (val);
}

static void tc90400_write16 (struct map_info *map, __u16 d, unsigned long adr)
{
	printd ("%08x <- %04x\n", (map->map_priv_1 + adr), (__u32)d);
	*((volatile __u16*)(map->map_priv_1 + adr)) = d;
}

static __u32 tc90400_read32 (struct map_info *map, unsigned long ofs)
{
	register __u32 val;
	val = *((volatile __u32*)(map->map_priv_1 + ofs));
	printd ("%08x -> %08x\n", (map->map_priv_1 + ofs), (__u32)val);
	return (val);
}

static void tc90400_write32 (struct map_info *map, __u32 d, unsigned long adr)
{
	printd ("%08x <- %08x\n", (map->map_priv_1 + adr), (__u32)d);
	*((volatile __u32*)(map->map_priv_1 + adr)) = d;
}

static void tc90400_copy_from (struct map_info *map,
				void *to, unsigned long from, ssize_t len)
{
	memcpy_fromio (to, map->map_priv_1 + from, len);
}

static void tc90400_copy_to (struct map_info *map,
			      unsigned long to, const void *from, ssize_t len)
{
	memcpy_toio (map->map_priv_1 + to, from, len);
}


#ifdef CONFIG_TOSHIBA_TC90400GDRS
static struct tc90400gdrs_norflash norflash_def[3] = {
  { /* mode1 */
    0x1f000000,    /* flash0_addr */
    FLASH_16MB,    /* flash0_size */
             2,    /* flash0_partnum */
    0x18100000,    /* flash1_addr */
    FLASH_15MB,    /* flash1_size */
             2,    /* flash1_partnum */
  },
  { /* mode2 */
    0x1fc00000,    /* flash0_addr */
     FLASH_4MB,    /* flash0_size */
             1,    /* flash0_partnum */
    0x18100000,    /* flash1_addr */
     FLASH_3MB,    /* flash1_size */
             1,    /* flash1_partnum */
  },
  { /* mode3 */
             0,    /* flash0_addr */
             0,    /* flash0_size */
             0,    /* flash0_partnum */
    0x18100000,    /* flash1_addr */
     FLASH_3MB,    /* flash1_size */
             1,    /* flash1_partnum */
  },
};


static struct mtd_info *tc90400_mtd0, *tc90400_mtd1;
struct map_info tc90400_map0 = {
	name:		TC90400_FLASH0_NAME,
	size:		0,
	buswidth:	TC90400_BUSWIDTH,
	read8:		tc90400_read8,
	write8:		tc90400_write8,
	read16:		tc90400_read16,
	write16:	tc90400_write16,
	read32:		tc90400_read32,
	write32:	tc90400_write32,
	copy_from:	tc90400_copy_from,
	copy_to:	tc90400_copy_to,
};
struct map_info tc90400_map1 = {
	name:		TC90400_FLASH1_NAME,
	size:		0,
	buswidth:	TC90400_BUSWIDTH,
	read8:		tc90400_read8,
	write8:		tc90400_write8,
	read16:		tc90400_read16,
	write16:	tc90400_write16,
	read32:		tc90400_read32,
	write32:	tc90400_write32,
	copy_from:	tc90400_copy_from,
	copy_to:	tc90400_copy_to,
};

#else /* CONFIG_TOSHIBA_TC90400GDRS */

static struct mtd_info *tc90400_mtd;
struct map_info tc90400_map = {
	name:		"TC90400EVB NOR-FLASH",
	size:		0,
	buswidth:	2,
	read8:		tc90400_read8,
	write8:		tc90400_write8,
	read16:		tc90400_read16,
	write16:	tc90400_write16,
	read32:		tc90400_read32,
	write32:	tc90400_write32,
	copy_from:	tc90400_copy_from,
	copy_to:	tc90400_copy_to,
};
#endif /* CONFIG_TOSHIBA_TC90400GDRS */


#ifdef USE_PARTITION

#ifdef CONFIG_TOSHIBA_TC90400GDRS
static struct mtd_partition tc90400_partitions0_m0[2] = { /* Total:16MB */
        {
                name:		"Free Area0",
                offset:		0,
                size:		FLASH_12MB,
                mask_flags: 	0,
        },
	{
                name:		"BOOT Area",
                offset: 	MTDPART_OFS_APPEND,
                size:		FLASH_4MB,
                mask_flags: 	MTD_WRITEABLE | MTD_ERASEABLE,
        },
};
static struct mtd_partition tc90400_partitions0_m12[1] = { /* Total:4MB */
	{
                name:		"BOOT Area",
                offset: 	0,
                size:		FLASH_4MB,
                mask_flags: 	MTD_WRITEABLE | MTD_ERASEABLE,
        },
};
static struct mtd_partition tc90400_partitions1_m0[2] = { /* Total:14MB */
        {
                name:		"Free Area1",
                offset:		0,
                size:		FLASH_11MB,
                mask_flags: 	0,
        },
	{
                name:		"Free Area2",
                offset: 	MTDPART_OFS_APPEND,
                size:		FLASH_3MB,
                mask_flags: 	0,
        },
	/* The rest 1Mbyte is an access impossibility area */
};
static struct mtd_partition tc90400_partitions1_m12[1] = { /* Total:3MB */
	{
                name:		"Free Area",
                offset: 	0,
                size:		FLASH_3MB,
                mask_flags: 	0,
        },
};

#else /* CONFIG_TOSHIBA_TC90400GDRS */

#define TC90400_FULLNOR_FLASH_MAP    0x1f000000 /* base addr. for 16Mbyte */
#define FLASH_FULL_NPARTS            4	/* number of partition for 16Mbyte */
#define FLASH_PMON_AREA_SIZE	     0x00040000	/* 256KB */
#define FLASH_PMON_NVRAM_SIZE	     0x00020000	/* 128KB */
#define FLASH_4MB                    0x00400000	/* 4MByte  */
#define FLASH_16MB                   0x01000000	/* 16MByte */
static struct mtd_partition tc90400_full_partitions[FLASH_FULL_NPARTS] = {
        {
                name:		"JFFS Free Area1",
                offset:		0,
                size:		0x00c00000,
                mask_flags: 	0,
        },
	{
                name:		"PMON Area",
                offset: 	MTDPART_OFS_APPEND,
                size:		FLASH_PMON_AREA_SIZE,
                mask_flags: 	MTD_WRITEABLE | MTD_ERASEABLE,
        },
        {
                name:		"JFFS Free Area2",
                offset:		MTDPART_OFS_APPEND,
                size:		0,
                mask_flags: 	0,
        },
	{
                name:		"PMON NVRAM Area",
                offset: 	MTDPART_OFS_APPEND,
                size:		FLASH_PMON_NVRAM_SIZE,
                mask_flags: 	MTD_WRITEABLE | MTD_ERASEABLE,
        }
};
#endif /* CONFIG_TOSHIBA_TC90400GDRS */

#endif /* USE_PARTITION */


#ifdef CONFIG_TOSHIBA_TC90400GDRS
static int __init init_tc90400_maps (void)
{
	unsigned int part_num, base_addr;
	struct mtd_partition *part;
	unsigned char dpsw, mode;

	dpsw = (unsigned char)dpsw_check();
        switch (dpsw) {
	case FPGA0_DIPSW_MODE3:
	    mode = 2;
	    break;
	case FPGA0_DIPSW_MODE2:
	    mode = 1;
	    break;
	case FPGA0_DIPSW_MODE1:
	    mode = 0;
	    break;
	default:
	    printk(KERN_NOTICE "TC90400GDRS illegal mode\n");
	    return -1;
	}

	/* Nor Flash0 */
	if (mode != 2) { /* not NandBoot */
	    base_addr = norflash_def[mode].flash0_addr;
	    tc90400_map0.size = norflash_def[mode].flash0_size;
            part_num = norflash_def[mode].flash0_partnum;
	    if (!mode) {
	        part = tc90400_partitions0_m0;
	    }
	    else {
	        part = tc90400_partitions0_m12;
	    }

	    tc90400_map0.map_priv_1 = (unsigned long)ioremap_nocache(base_addr, tc90400_map0.size);
	    printk (KERN_NOTICE "Probing for Nor-Flash chip0 at %lx:\n",
				tc90400_map0.map_priv_1);

	    tc90400_mtd0 = do_map_probe ("cfi_probe", &tc90400_map0);
	    if (!tc90400_mtd0) {
		printk (KERN_NOTICE "Nor-Flash chip0 not detected "
			"at either possible location.\n");
		if( tc90400_map0.map_priv_1 ) { /* add.iounmap */
			iounmap((void *)tc90400_map0.map_priv_1);
			tc90400_map0.map_priv_1 = 0;
		}
		return (-ENXIO);
	    }

	    printk (KERN_NOTICE "TC90400: Nor-Flash0 at %lx\n",
				tc90400_map0.map_priv_1);
	    tc90400_mtd0->owner = THIS_MODULE;
#ifdef USE_PARTITION
	    add_mtd_partitions (tc90400_mtd0, part, part_num);
#endif
	}

	/* Nor Flash1 */
	base_addr = norflash_def[mode].flash1_addr;
	tc90400_map1.size = norflash_def[mode].flash1_size;
        part_num = norflash_def[mode].flash1_partnum;
	if (!mode) {
	    part = tc90400_partitions1_m0;
	}
	else {
	    part = tc90400_partitions1_m12;
	}

	tc90400_map1.map_priv_1 = (unsigned long)ioremap_nocache(base_addr, tc90400_map1.size);
	printk (KERN_NOTICE "Probing for Nor-Flash chip1 at %lx:\n",
				tc90400_map1.map_priv_1);

	tc90400_mtd1 = do_map_probe ("cfi_probe", &tc90400_map1);
	if (!tc90400_mtd1) {
		printk (KERN_NOTICE "Nor-Flash chip1 not detected "
			"at either possible location.\n");
		if( tc90400_map1.map_priv_1 ) { /* add.iounmap */
			iounmap((void *)tc90400_map1.map_priv_1);
			tc90400_map1.map_priv_1 = 0;
		}
		return (-ENXIO);
	}

	printk (KERN_NOTICE "TC90400: Nor-Flash1 at %lx\n",
				tc90400_map1.map_priv_1);
	tc90400_mtd1->owner = THIS_MODULE;
#ifdef USE_PARTITION
	add_mtd_partitions (tc90400_mtd1, part, part_num);
#endif

	return (0);
}

static void __exit cleanup_tc90400_maps(void)
{
        if (tc90400_mtd0) { /* NorFlash0 */
#ifdef USE_PARTITION
		del_mtd_partitions (tc90400_mtd0);
#endif
		map_destroy (tc90400_mtd0);
		if( tc90400_map0.map_priv_1 ) {
			iounmap((void *)tc90400_map0.map_priv_1);
			tc90400_map0.map_priv_1 = 0;
		}
	}
	if (tc90400_mtd1) { /* NorFlash1 */
#ifdef USE_PARTITION
		del_mtd_partitions (tc90400_mtd1);
#endif
		map_destroy (tc90400_mtd1);
		if( tc90400_map1.map_priv_1 ) {
			iounmap((void *)tc90400_map1.map_priv_1);
			tc90400_map1.map_priv_1 = 0;
		}
	}
}

#else /* CONFIG_TOSHIBA_TC90400GDRS */

static int __init init_tc90400_maps (void)
{
	unsigned int part_num, base_addr;
	struct mtd_partition *part;

	base_addr = TC90400_FULLNOR_FLASH_MAP;
	tc90400_map.size = FLASH_16MB; /* 16Mbyte */

        part_num = 4;
	tc90400_full_partitions[2].size = FLASH_4MB - (FLASH_PMON_AREA_SIZE+FLASH_PMON_NVRAM_SIZE); /* free area1 */
	part = tc90400_full_partitions;

	tc90400_map.map_priv_1 = (unsigned long)ioremap_nocache(base_addr, tc90400_map.size);
	printk (KERN_NOTICE "Probing for Nor-Flash chips at %lx:\n",
				tc90400_map.map_priv_1);

	tc90400_mtd = do_map_probe ("cfi_probe", &tc90400_map);
	if (!tc90400_mtd) {
		printk (KERN_NOTICE "Nor-Flash chips not detected "
			"at either possible location.\n");
		if( tc90400_map.map_priv_1 ) { /* add.iounmap */
			iounmap((void *)tc90400_map.map_priv_1);
			tc90400_map.map_priv_1 = 0;
		}
		return (-ENXIO);
	}

	printk (KERN_NOTICE "TC90400EVB: Nor-Flash at %lx\n",
				tc90400_map.map_priv_1);

	tc90400_mtd->owner = THIS_MODULE;

#ifdef USE_PARTITION
	add_mtd_partitions (tc90400_mtd, part, part_num);
#endif

	return (0);
}

static void __exit cleanup_tc90400_maps(void)
{
        if (tc90400_mtd) {
#ifdef USE_PARTITION
		del_mtd_partitions (tc90400_mtd);
#endif
		map_destroy (tc90400_mtd);
		if( tc90400_map.map_priv_1 ) {
			iounmap((void *)tc90400_map.map_priv_1);
			tc90400_map.map_priv_1 = 0;
		}
	}
}
#endif /* CONFIG_TOSHIBA_TC90400GDRS */

module_init(init_tc90400_maps);
module_exit(cleanup_tc90400_maps);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("MTD map driver for TC90400 GDRS boards");
