/*
 * linux/drivers/mtd/maps/tc90400_sharp_board-flash.c
 * : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal Nor-Flash driver for TC90400 SHARP_BOARD Board
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2004 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 * Modified from TC90400QFP_BOARD01
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#include <asm/io.h>
#include <asm/tc90400/tc90400_sharp_board.h>

#if (defined(CONFIG_MTD_PARTITIONS) || defined(CONFIG_MTD_PARTITIONS_MODULE))
#define USE_PARTITION
#else
#undef USE_PARTITION
#endif

#undef DEBUG
#ifdef DEBUG
#define printd(arg...)	printk(arg)
#else
#define printd(arg...)
#endif

static __u8 tc90400_read8 (struct map_info *map, unsigned long ofs)
{
	register __u8 val;
	val = *((volatile __u8*)(map->map_priv_1 + ofs));
	printd ("%08x -> %02x\n", (map->map_priv_1 + ofs), (__u16)val);
	return (val);
}

static void tc90400_write8 (struct map_info *map, __u8 d, unsigned long adr)
{
	printd ("%08x <- %02x\n", (map->map_priv_1 + adr), (__u16)d);
	*((volatile __u8*)(map->map_priv_1 + adr)) = d;
}

static __u16 tc90400_read16 (struct map_info *map, unsigned long ofs)
{
	register __u16 val;
	val = *((volatile __u16*)(map->map_priv_1 + ofs));
	printd ("%08x -> %04x\n", (map->map_priv_1 + ofs), (__u32)val);
	return (val);
}

static void tc90400_write16 (struct map_info *map, __u16 d, unsigned long adr)
{
	printd ("%08x <- %04x\n", (map->map_priv_1 + adr), (__u32)d);
	*((volatile __u16*)(map->map_priv_1 + adr)) = d;
}

static __u32 tc90400_read32 (struct map_info *map, unsigned long ofs)
{
	register __u32 val;
	val = *((volatile __u32*)(map->map_priv_1 + ofs));
	printd ("%08x -> %08x\n", (map->map_priv_1 + ofs), (__u32)val);
	return (val);
}

static void tc90400_write32 (struct map_info *map, __u32 d, unsigned long adr)
{
	printd ("%08x <- %08x\n", (map->map_priv_1 + adr), (__u32)d);
	*((volatile __u32*)(map->map_priv_1 + adr)) = d;
}

static void tc90400_copy_from (struct map_info *map,
			       void *to, unsigned long from, ssize_t len)
{
	memcpy_fromio (to, map->map_priv_1 + from, len);
}

static void tc90400_copy_to (struct map_info *map,
			     unsigned long to, const void *from, ssize_t len)
{
	memcpy_toio (map->map_priv_1 + to, from, len);
}

static TC90400_SHARP_BOARD_NORFLASH norflash_def[NUM_FLASH_CHIP] = {
{
	0x1f000000, /* flash_addr */
	FLASH_16MB,  /* flash_size */
#ifdef CONFIG_TOSHIBA_TC90400_NORMAL
	3,          /* flash_partnum */
#endif  /* CONFIG_TOSHIBA_TC90400_NORMAL */
#ifdef CONFIG_TOSHIBA_TC90400_UPDATE
	5,          /* flash_partnum (DualBoot Support) */
#endif  /* CONFIG_TOSHIBA_TC90400_UPDATE */
#ifdef CONFIG_TOSHIBA_TC90400_PRODUCT
	3,          /* flash_partnum (DualBoot Support) */
#endif  /* CONFIG_TOSHIBA_TC90400_PRODUCT */
},
{
	0x19000000,	/* flash_addr */
	FLASH_8MB,	/* flash_size */
//	1,		/* flash_partnum */
	2,		/* flash_partnum */	//	divide into 2 --> for ES
}
#if 0	//Flash P͂
,
{
	0x19800000,	/* flash_addr */
	FLASH_8MB,	/* flash_size */
	1,		/* flash_partnum */
}
#endif
};

static struct mtd_info *tc90400_mtd[NUM_FLASH_CHIP];
struct map_info tc90400_map[NUM_FLASH_CHIP] = {
{
	name:		TC90400_SHARP_BOARD_FLASH_NAME"0",
	size:		0,
	buswidth:	TC90400_BUSWIDTH,
	read8:		tc90400_read8,
	write8:		tc90400_write8,
	read16:		tc90400_read16,
	write16:	tc90400_write16,
	read32:		tc90400_read32,
	write32:	tc90400_write32,
	copy_from:	tc90400_copy_from,
	copy_to:	tc90400_copy_to,
},
{
	name:		TC90400_SHARP_BOARD_FLASH_NAME"1",
	size:		0,
	buswidth:	TC90400_BUSWIDTH,
	read8:		tc90400_read8,
	write8:		tc90400_write8,
	read16:		tc90400_read16,
	write16:	tc90400_write16,
	read32:		tc90400_read32,
	write32:	tc90400_write32,
	copy_from:	tc90400_copy_from,
	copy_to:	tc90400_copy_to,
}
#if 0	//	Flash P͂
,
{
	name:		TC90400_SHARP_BOARD_FLASH_NAME"2",
	size:		0,
	buswidth:	TC90400_BUSWIDTH,
	read8:		tc90400_read8,
	write8:		tc90400_write8,
	read16:		tc90400_read16,
	write16:	tc90400_write16,
	read32:		tc90400_read32,
	write32:	tc90400_write32,
	copy_from:	tc90400_copy_from,
	copy_to:	tc90400_copy_to,
},
#endif
};


#ifdef USE_PARTITION

#ifdef CONFIG_TOSHIBA_TC90400_NORMAL
static struct mtd_partition _sharp_partitions[NUM_FLASH_CHIP][3]={ /* Total:16MB */
{
	{
		name:               "RootFS(JFFS2)",
		offset:             0,
		size:               FLASH_12MB,
		mask_flags:         0,
	},
	{
		name:               "BOOT Area",
		offset:             MTDPART_OFS_APPEND,
		size:               FLASH_1MB,
		mask_flags:         MTD_WRITEABLE | MTD_ERASEABLE,
	},
	{
		name:               "Free Area",
		offset:             MTDPART_OFS_APPEND,
		size:               FLASH_3MB,
		mask_flags:         0,
	},
},
{
	{
		name:               "Free Area2",
		offset:             0,
		size:               FLASH_8MB,
		mask_flags:         0,
	},
}
#if 0	//	Flash P͂
,
{
	{
		name:               "Free Area3",
		offset:             0,
		size:               FLASH_8MB,
		mask_flags:         0,
	},
}
#endif
};
#endif	/* CONFIG_TOSHIBA_TC90400_NORMAL */

#ifdef CONFIG_TOSHIBA_TC90400_UPDATE
static struct mtd_partition _sharp_partitions[NUM_FLASH_CHIP][5]={ /* Total:16MB (DualBoot SUpport) */
{
	{
		name:              "RootFS (JFFS2)",
		offset:            0,
		size:              FLASH_8MB,			//	start address = 0x1f00 0000
		mask_flags:        0,
	},
	{
		name:              "FS1 (JFFS2)",
		offset:            MTDPART_OFS_APPEND,
		size:              FLASH_4MB,			//	start address = 0x1f80 0000
		mask_flags:        0,
	},
#if 0	//	original = 1.5MB x 2 
	{
		name:              "Maintenance Boot Area",
		offset:            MTDPART_OFS_APPEND,
		size:              0x00180000,     /* 1.5 MB */		//	start address = 0x1fc0 0000
		mask_flags:        MTD_WRITEABLE | MTD_ERASEABLE,
	},
	{
		name:              "Product Boot Area",
		offset:            MTDPART_OFS_APPEND,
		size:              0x00180000,     /* 1.5 MB */		//	start address = 0x1fd8 0000
		mask_flags:        0,
	},
#else	//	changed for 1MB (PMON) + 2MB (Kernel)
	{
		name:              "Maintenance Boot Area",
		offset:            MTDPART_OFS_APPEND,
		size:              0x00100000,     /* 1 MB */		//	start address = 0x1fc0 0000
		mask_flags:        MTD_WRITEABLE | MTD_ERASEABLE,
	},
	{
		name:              "Product Boot Area",
		offset:            MTDPART_OFS_APPEND,
		size:              0x00200000,     /* 2 MB */		//	start address = 0x1fd0 0000
		mask_flags:        0,
	},
#endif
	{
		name:              "Free Area (jffs2)",
		offset:            MTDPART_OFS_APPEND,
		size:              0x00100000,     /* 1.0 MB */		//	start address = 0x1fe0 0000
		mask_flags:        0,
	},
},
{
#if 0	//	original = 8MB // one partition
	{
		name:               "Free Area2",
		offset:             0,
		size:               FLASH_8MB,
		mask_flags:         0,
	},
#else	//	divide 4MB + 4MB
	{
		name:               "Free Area2-1",
		offset:             0,
		size:               FLASH_4MB,
		mask_flags:         0,
	},
	{
		name:               "Free Area2-2",
		offset:             MTDPART_OFS_APPEND,
		size:               FLASH_4MB,
		mask_flags:         0,
	},
#endif
}
#if 0	//	Flash P͂
,
{
	{
		name:               "Free Area3",
		offset:             0,
		size:               FLASH_8MB,
		mask_flags:         0,
	},
}
#endif
};
#endif  /* CONFIG_TOSHIBA_TC90400_UPDATE */

#ifdef CONFIG_TOSHIBA_TC90400_PRODUCT
static struct mtd_partition _sharp_partitions[NUM_FLASH_CHIP][3]={ /* Total:16MB (DualBoot Support) */
{
	{
		name:               "RootFS (cramfs)",
		offset:             0,
		size:               FLASH_12MB,
		mask_flags:         0,
	},
	{
		name:              "Boot Area",
		offset:            MTDPART_OFS_APPEND,
		size:              0x00300000,     /* 3.0 MB */
		mask_flags:         MTD_WRITEABLE | MTD_ERASEABLE,
	},
	{
		name:              "Free Area (jffs2)",
		offset:            MTDPART_OFS_APPEND,
		size:              0x00100000,     /* 1.0 MB */
		mask_flags:        0,
	},
},
{
	{
		name:               "Free Area2",
		offset:             0,
		size:               FLASH_8MB,
		mask_flags:         0,
	},
}
#if 0	//	Flash P͂
,{
	{
		name:               "Free Area3",
		offset:             0,
		size:               FLASH_8MB,
		mask_flags:         0,
	},
}
#endif
};
#endif  /* CONFIG_TOSHIBA_TC90400_PRODUCT */

#endif /* USE_PARTITION */

static int init_each_maps(int nchip)
{
	unsigned int part_num, base_addr;
	struct mtd_partition *part;

	if (nchip >= NUM_FLASH_CHIP) {
		return (-EINVAL);
	}

	base_addr = norflash_def[nchip].flash_addr;
	tc90400_map[nchip].size = norflash_def[nchip].flash_size;
	part_num = norflash_def[nchip].flash_partnum;
	part = _sharp_partitions[nchip];

	tc90400_map[nchip].map_priv_1 = (unsigned long)ioremap_nocache(base_addr, tc90400_map[nchip].size);
	printk (KERN_NOTICE "Probing for Nor-Flash chip%d at %lx:\n",
			nchip, tc90400_map[nchip].map_priv_1);

	tc90400_mtd[nchip] = do_map_probe("cfi_probe", &tc90400_map[nchip]);
	if (!tc90400_mtd[nchip]) {
		printk (KERN_NOTICE "Nor-Flash chip%d not detected "
			"at either possible location.\n", nchip);
		if( tc90400_map[nchip].map_priv_1 ) { /* add.iounmap */
			iounmap((void *)tc90400_map[nchip].map_priv_1);
			tc90400_map[nchip].map_priv_1 = 0;
		}
		return (-ENXIO);
	}

	printk (KERN_NOTICE "TC90400: Nor-Flash%d at %lx\n",
			nchip, tc90400_map[nchip].map_priv_1);
	tc90400_mtd[nchip]->owner = THIS_MODULE;
#ifdef USE_PARTITION
	add_mtd_partitions (tc90400_mtd[nchip], part, part_num);
#endif

	return (0);
}

static int __init init_tc90400_maps (void)
{
	int i, ret;

	for (i = 0; i < NUM_FLASH_CHIP; i++) {
		ret = init_each_maps(i);
		if (ret != 0) {
			return ret;
		}
	}
	return 0;
}

static void cleanup_each_maps(int nchip)
{
	if (tc90400_mtd[nchip]) {
#ifdef USE_PARTITION
		del_mtd_partitions (tc90400_mtd[nchip]);
#endif
		map_destroy (tc90400_mtd[nchip]);
		if( tc90400_map[nchip].map_priv_1 ) {
			iounmap((void *)tc90400_map[nchip].map_priv_1);
			tc90400_map[nchip].map_priv_1 = 0;
		}
	}
}

static void __exit cleanup_tc90400_maps(void)
{
	int i;

	for (i = 0; i < NUM_FLASH_CHIP; i++) {
		cleanup_each_maps(i);
	}
}


module_init(init_tc90400_maps);
module_exit(cleanup_tc90400_maps);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("MTD map driver for TC90400 SHARP Board");

