/*
 * linux/drivers/mtd/maps/tc90400qfp_board01-flash.c : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal Nor-Flash driver for TC90400 QFP_BOARD01 Board
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2004 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#include <asm/io.h>
#include <asm/tc90400/tc90400qfp_board01.h>

#if (defined(CONFIG_MTD_PARTITIONS) || defined(CONFIG_MTD_PARTITIONS_MODULE))
#define USE_PARTITION
#else
#undef USE_PARTITION
#endif

#undef DEBUG
#ifdef DEBUG
#define printd(arg...)	printk(arg)
#else
#define printd(arg...)
#endif

static __u8 tc90400_read8 (struct map_info *map, unsigned long ofs)
{
  register __u8 val;
  val = *((volatile __u8*)(map->map_priv_1 + ofs));
  printd ("%08x -> %02x\n", (map->map_priv_1 + ofs), (__u16)val);
  return (val);
}

static void tc90400_write8 (struct map_info *map, __u8 d, unsigned long adr)
{
  printd ("%08x <- %02x\n", (map->map_priv_1 + adr), (__u16)d);
  *((volatile __u8*)(map->map_priv_1 + adr)) = d;
}

static __u16 tc90400_read16 (struct map_info *map, unsigned long ofs)
{
  register __u16 val;
  val = *((volatile __u16*)(map->map_priv_1 + ofs));
  printd ("%08x -> %04x\n", (map->map_priv_1 + ofs), (__u32)val);
  return (val);
}

static void tc90400_write16 (struct map_info *map, __u16 d, unsigned long adr)
{
  printd ("%08x <- %04x\n", (map->map_priv_1 + adr), (__u32)d);
  *((volatile __u16*)(map->map_priv_1 + adr)) = d;
}

static __u32 tc90400_read32 (struct map_info *map, unsigned long ofs)
{
  register __u32 val;
  val = *((volatile __u32*)(map->map_priv_1 + ofs));
  printd ("%08x -> %08x\n", (map->map_priv_1 + ofs), (__u32)val);
  return (val);
}

static void tc90400_write32 (struct map_info *map, __u32 d, unsigned long adr)
{
  printd ("%08x <- %08x\n", (map->map_priv_1 + adr), (__u32)d);
  *((volatile __u32*)(map->map_priv_1 + adr)) = d;
}

static void tc90400_copy_from (struct map_info *map,
			       void *to, unsigned long from, ssize_t len)
{
  memcpy_fromio (to, map->map_priv_1 + from, len);
}

static void tc90400_copy_to (struct map_info *map,
			     unsigned long to, const void *from, ssize_t len)
{
  memcpy_toio (map->map_priv_1 + to, from, len);
}

static struct tc90400qfp_board01_norflash norflash_def = {
  0x1f800000, /* flash_addr */
  FLASH_8MB,  /* flash_size */
#ifdef CONFIG_TOSHIBA_TC90400_NORMAL
  3,          /* flash_partnum */
#endif  /* CONFIG_TOSHIBA_TC90400_NORMAL */
#ifdef CONFIG_TOSHIBA_TC90400_UPDATE
  4,          /* flash_partnum (DualBoot Support) */
#endif  /* CONFIG_TOSHIBA_TC90400_UPDATE */
#ifdef CONFIG_TOSHIBA_TC90400_PRODUCT
  3,          /* flash_partnum (DualBoot Support) */
#endif  /* CONFIG_TOSHIBA_TC90400_PRODUCT */
};

static struct mtd_info *tc90400_mtd;
struct map_info tc90400_map = {
  name:		TC90400QFP_BOARD01_FLASH_NAME,
  size:		0,
  buswidth:	TC90400_BUSWIDTH,
  read8:	tc90400_read8,
  write8:	tc90400_write8,
  read16:	tc90400_read16,
  write16:	tc90400_write16,
  read32:	tc90400_read32,
  write32:	tc90400_write32,
  copy_from:	tc90400_copy_from,
  copy_to:	tc90400_copy_to,
};


#ifdef USE_PARTITION

#ifdef CONFIG_TOSHIBA_TC90400_NORMAL
static struct mtd_partition tc90400qfp_board01_partitions[3]={ /* Total:8MB */
  {
    name:               "RootFS(JFFS2)",
    offset:             0,
    size:               FLASH_4MB,
    mask_flags:         0,
  },
  {
    name:               "BOOT Area",
    offset:             MTDPART_OFS_APPEND,
    size:               FLASH_1MB,
    mask_flags:         MTD_WRITEABLE | MTD_ERASEABLE,
  },
  {
    name:               "Free Area",
    offset:             MTDPART_OFS_APPEND,
    size:               FLASH_3MB,
    mask_flags:         0,
  },
};
#endif	/* CONFIG_TOSHIBA_TC90400_NORMAL */

#ifdef CONFIG_TOSHIBA_TC90400_UPDATE
static struct mtd_partition tc90400qfp_board01_partitions[4]={ /* Total:8MB (DualBoot SUpport) */
  {
    name:               "RootFS (cramfs)",
    offset:             0,
    size:               FLASH_4MB,
    mask_flags:         0,
  },
  {
    name:              "Maintenance Boot Area",
    offset:            MTDPART_OFS_APPEND,
    size:              0x00180000,     /* 1.5 MB */
    mask_flags:        MTD_WRITEABLE | MTD_ERASEABLE,
  },
  {
    name:              "Product Boot Area",
    offset:            MTDPART_OFS_APPEND,
    size:              0x00180000,     /* 1.5 MB */
    mask_flags:        0,
  },
  {
    name:              "Free Area (jffs2)",
    offset:            MTDPART_OFS_APPEND,
    size:              0x00100000,     /* 1.0 MB */
    mask_flags:        0,
  },
};
#endif  /* CONFIG_TOSHIBA_TC90400_UPDATE */

#ifdef CONFIG_TOSHIBA_TC90400_PRODUCT
static struct mtd_partition tc90400qfp_board01_partitions[3]={ /* Total:8MB (DualBoot Support) */
  {
    name:               "RootFS (cramfs)",
    offset:             0,
    size:               FLASH_4MB,
    mask_flags:         0,
  },
  {
    name:              "Boot Area",
    offset:            MTDPART_OFS_APPEND,
    size:              0x00300000,     /* 3.0 MB */
    mask_flags:         MTD_WRITEABLE | MTD_ERASEABLE,
  },
  {
    name:              "Free Area (jffs2)",
    offset:            MTDPART_OFS_APPEND,
    size:              0x00100000,     /* 1.0 MB */
    mask_flags:        0,
  },
};
#endif  /* CONFIG_TOSHIBA_TC90400_PRODUCT */

#endif /* USE_PARTITION */


static int __init init_tc90400_maps (void)
{
  unsigned int part_num, base_addr;
  struct mtd_partition *part;

  base_addr = norflash_def.flash_addr;
  tc90400_map.size = norflash_def.flash_size;
  part_num = norflash_def.flash_partnum;
  part = tc90400qfp_board01_partitions;

  tc90400_map.map_priv_1 = (unsigned long)ioremap_nocache(base_addr, tc90400_map.size);
  printk (KERN_NOTICE "Probing for Nor-Flash chip at %lx:\n",
	  tc90400_map.map_priv_1);

  tc90400_mtd = do_map_probe ("cfi_probe", &tc90400_map);
  if (!tc90400_mtd) {
    printk (KERN_NOTICE "Nor-Flash chip0 not detected "
	    "at either possible location.\n");
    if( tc90400_map.map_priv_1 ) { /* add.iounmap */
      iounmap((void *)tc90400_map.map_priv_1);
      tc90400_map.map_priv_1 = 0;
    }
    return (-ENXIO);
  }

  printk (KERN_NOTICE "TC90400: Nor-Flash at %lx\n",
	  tc90400_map.map_priv_1);
  tc90400_mtd->owner = THIS_MODULE;
#ifdef USE_PARTITION
  add_mtd_partitions (tc90400_mtd, part, part_num);
#endif

  return (0);
}

static void __exit cleanup_tc90400_maps(void)
{
  if (tc90400_mtd) {
#ifdef USE_PARTITION
    del_mtd_partitions (tc90400_mtd);
#endif
    map_destroy (tc90400_mtd);
    if( tc90400_map.map_priv_1 ) {
      iounmap((void *)tc90400_map.map_priv_1);
      tc90400_map.map_priv_1 = 0;
    }
  }
}


module_init(init_tc90400_maps);
module_exit(cleanup_tc90400_maps);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("MTD map driver for TC90400 QFP Board01");
