/*
 * linux/include/asm-mips/tc90400/tc90400.h : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */

#ifndef TC90400_TC90400_H
#define TC90400_TC90400_H

#include <asm/tc90400/tc90400_mips.h>

/*
 * System specification
*/
/* external input clock */
#define TC90400_XT_CLCK_27		27000000	/* 27MHz */
#define TC90400_XT_CLCK_30		30000000	/* 30MHz */

#define TC90400_USE_KERNEL_MEMORY_SIZE	16		/* 16MB added */

/*
 This register naming came from the intergrate cpu/controoler name TC90400
 followed by the device name from table 4.2.2 on page 4-3 and then followed
 by the register name from table 4.2.3 on pages 4-4 to 4-8.  The manaul
 used is "TMPR4927BT Preliminary Rev 0.1 20.Jul.2001".
 */

#define EGBUS_REG_BASE			(0xa0000000)
#define EGBUS_REG(ofs)			(EGBUS_REG_BASE + (ofs))

/*
 * TX49 address space register base
*/

#define TC90400_GMBR_BASE		EGBUS_REG(0x17400000)
/* controller on MBus */
#define TC90400_VMMS_BASE	  	EGBUS_REG(0x17410000)
#define TC90400_MBUG_BASE	  	EGBUS_REG(0x17420000)
#define TC90400_TSP_RAM_BASE	  	EGBUS_REG(0x17430000)
#define TC90400_ASP_RAM_BASE	  	EGBUS_REG(0x17440000)
#define TC90400_AMP_RAM_BASE	  	EGBUS_REG(0x17450000)
#define TC90400_CG_BASE			EGBUS_REG(0x17500000)
#define TC90400_VDEC_BASE	  	EGBUS_REG(0x17510000)
#define TC90400_GFX_BASE		EGBUS_REG(0x17520000)
#define TC90400_VIOU_BASE	  	EGBUS_REG(0x17530000)
#define TC90400_GEBR_BASE		EGBUS_REG(0x18000000)

/* controller on GBus */
#define TC90400_DMAC_BASE		EGBUS_REG(0x1c000000)
#define TC90400_ATAC_BASE		EGBUS_REG(0x1c004000)
#define TC90400_MPIF_BASE		EGBUS_REG(0x1c004100)
#define TC90400_USB_BASE		EGBUS_REG(0x1c008000)
#define TC90400_GBUS_BASE		EGBUS_REG(0x1c00c000)

/* controller on Ebus */
#define TC90400_IRC_BASE		EGBUS_REG(0x1e000000)
#define TC90400_I2C_BASE		EGBUS_REG(0x1e004000)
#define TC90400_PIO_BASE		EGBUS_REG(0x1e008000)
#define TC90400_ACLC_BASE		EGBUS_REG(0x1e00c000)
#define TC90400_UART_BASE		EGBUS_REG(0x1e010000)
#define TC90400_SCC_BASE		EGBUS_REG(0x1e014000)
#define TC90400_TMR_BASE		EGBUS_REG(0x1e018000)
#define TC90400_ZTEST_BASE		EGBUS_REG(0x1e01c000)
#define TC90400_EBIF_BASE		EGBUS_REG(0x1e020000)
#define TC90400_NANDF_BASE		EGBUS_REG(0x1e020100)
#define TC90400_CI_BASE			EGBUS_REG(0x1e020200)


/*
 * ZTEST
*/
/* ZTEST register map */
#define TC90400_CFINI			(TC90400_ZTEST_BASE)		/* initial hardware configuration */
#define TC90400_CFSH0			(TC90400_ZTEST_BASE + 0x0004)	/* Pin Share Mode 0 */
#define TC90400_CFSH1			(TC90400_ZTEST_BASE + 0x0008)	/* Pin Share Mode 1 */
#define TC90400_CFSH2			(TC90400_ZTEST_BASE + 0x000c)	/* Pin Share Mode 2 */
#define TC90400_CFSHDMC			(TC90400_ZTEST_BASE + 0x0010)	/* DMAC mode */
#define TC90400_CFDACPDN		(TC90400_ZTEST_BASE + 0x0014)	/* DAC Power Down mode */
#define TC90400_CFDACMD			(TC90400_ZTEST_BASE + 0x0018)	/* DAC select mode */
#define TC90400_CFREV			(TC90400_ZTEST_BASE + 0x001c)	/* Chip ID */
#define TC90400_WDMSK			(TC90400_ZTEST_BASE + 0x0020)	/* Watch dog timer Interrupt Mask */
#ifdef CONFIG_TOSHIBA_TC90400_REV1
#define TC90400_TINTDIS			(TC90400_ZTEST_BASE + 0x0024)	/* Watch dog timer Interrupt disable */
#endif /* #ifdef CONFIG_TOSHIBA_TC90400_REV1 */
#define TC90400_DLLCLKDIV		(TC90400_ZTEST_BASE + 0x0028)	/* DLL clock division enable */
#define TC90400_CFSH3			(TC90400_ZTEST_BASE + 0x002c)	/* Pin Share Mode 3 */

/* initial configuration bits */
#define TC90400_CFINI_208PIN			(0x01)
#define TC90400_CFINI_BIG_ENDIAN		(0x01 << 1)
#define TC90400_CFINI_NAND_FLASH		(0x01 << 2)
#define TC90400_CFINI_EBIF_8BITS		(0x01 << 3)
#define TC90400_CFINI_ACE			(0x01 << 4)
#define TC90400_CFINI_ACE_HOLD			(0x01 << 5)
#define TC90400_CFINI_EXT_VALID			(0x01 << 6)
#define TC90400_CFINI_TSP_PORT0			(0x01 << 7)
#define TC90400_CFINI_TSP_PORT1			(0x01 << 8)
#define TC90400_CFINI_DBG_PORT0			(0x01 << 9)
#define TC90400_CFINI_DBG_PORT1			(0x01 << 10)
#ifndef CONFIG_TOSHIBA_TC90400_REV1
#define TC90400_CFINI_TX49_TMRINT               (0x01 << 11)
#endif /* #ifndef CONFIG_TOSHIBA_TC90400_REV1 */
#define TC90400_CFINI_FREQ_SEL0			(0x01 << 18)
#define TC90400_CFINI_FREQ_SEL1			(0x01 << 19)
#define TC90400_CFINI_NF_MULTI			(0x01 << 20)

/* configuration pin share mode0 bits */
#define TC90400_CFSH0_NOIDE                     (0x01 << 15)
#define TC90400_CFSH0_IDE208PIN                 (0x01 << 16)
#define TC90400_CFSH0_I2C0			(0x01 << 19)
#define TC90400_CFSH0_I2C1			(0x01 << 20)
#define TC90400_CFSH0_USB_P1			(0x01 << 22)
#define TC90400_CFSH0_TMR0			(0x01 << 30)
#define TC90400_CFSH0_TMR1			(0x01 << 31)
/* configuration pin share mode1 bits */
#define TC90400_CFSH1_SCC0IN			(0x01 << 17)
#define TC90400_CFSH1_SCC1INP1                  (0x01 << 18)
#define TC90400_CFSH1_SCC1INP0                  (0x01 << 19)
#define TC90400_CFSH1_SCCEXTCK                  (0x01 << 20)
#define TC90400_CFSH1_INT0			(0x01 << 21)
#define TC90400_CFSH1_INT1			(0x01 << 22)
#define TC90400_CFSH1_INT2			(0x01 << 23)
#define TC90400_CFSH1_SIO0			(0x01 << 24)
#define TC90400_CFSH1_SIO1			(0x01 << 25)
#define TC90400_CFSH1_SIO1SEND			(0x01 << 26)
/* configuration pin share mode3 bits */
#ifndef CONFIG_TOSHIBA_TC90400_REV1
#define TC90400_CFSH3_TMRIN                     (0x01 << 16)
#define TC90400_CFSH3_INT0INP1                  (0x01 << 24)
#define TC90400_CFSH3_SCCIO1                    (0x01 << 25)
#define TC90400_CFSH3_SCCCLK1                   (0x01 << 26)
#endif /* #ifndef CONFIG_TOSHIBA_TC90400_REV1 */
#define TC90400_CFSH3_FABPID			(0x01 << 31)

/*
 * GBUS
*/
/* GBUS register map */
#define TC90400_GBREQ_ARB		(TC90400_GBUS_BASE)		/* Request/Arbitration conntrol */
#define TC90400_GBTOUT			(TC90400_GBUS_BASE + 0x004)	/* Time-out function */
#define TC90400_GBTOUT_ADD_U		(TC90400_GBUS_BASE + 0x008)	/* Time-out error access upper address */
#define TC90400_GBTOUT_ADD_L		(TC90400_GBUS_BASE + 0x00c)	/* Time-out error access lower address */
#define TC90400_GBCLR_IRQ		(TC90400_GBUS_BASE + 0x010)	/* Time-out interrupt clear */
#define TC90400_GBTOUT_TIME		(TC90400_GBUS_BASE + 0x014)	/* Time-out time set */
#define TC90400_GBSDRAM_SET		(TC90400_GBUS_BASE + 0x018)	/* SDRAM Address set */

/* Time-out function bits */
#define TC90400_GBTOUT_ON			(0x01)			/* time-out ON */

/*
 * VMMS on MBUS
*/
/* VMMS register map */
/*      DDR SDRAM setup register map */
#define TC90400_VMMS_MEMCONFIG		(TC90400_VMMS_BASE + 0x300)	/* SDRAM size */
#define TC90400_VMMS_MEMBANK		(TC90400_VMMS_BASE + 0x304)	/* bank status in memory */
#define TC90400_VMMS_REFMODE		(TC90400_VMMS_BASE + 0x308)	/* Refresh mode */

/* DDR SDRAM setup bits */
#define TC90400_VMMS_MEMCONFIG_MASK		(0x07)		/* SDRAM size capacity mask */

/*
 * TMR
*/
/* Timer register map */
#define TC90400_TMTCR(ch)        	(TC90400_TMR_BASE + ((ch) * 0x100))
#define TC90400_TMTISR(ch)       	(TC90400_TMR_BASE + 0x04 + ((ch) * 0x100))
#define TC90400_TMCPRA(ch)       	(TC90400_TMR_BASE + 0x08 + ((ch) * 0x100))
#define TC90400_TMCPRB(ch)       	(TC90400_TMR_BASE + 0x0c + ((ch) * 0x100))
#define TC90400_TMITMR(ch)       	(TC90400_TMR_BASE + 0x10 + ((ch) * 0x100))
#define TC90400_TMCCDR(ch)       	(TC90400_TMR_BASE + 0x20 + ((ch) * 0x100))
#define TC90400_TMPGMR(ch)       	(TC90400_TMR_BASE + 0x30 + ((ch) * 0x100))
#define TC90400_TMTRR(ch)        	(TC90400_TMR_BASE + 0xf0 + ((ch) * 0x100))

#define TC90400_TMWTMR              (TC90400_TMR_BASE + 0x240)

#define TC90400_TMCMPCONF        	(TC90400_TMR_BASE + 0x300)
#define TC90400_TMCMPR(no)       	(TC90400_TMR_BASE + 0x304 + ((no) * 0x4))
#define TC90400_TMCAPCONF        	(TC90400_TMR_BASE + 0x30c)
#define TC90400_TMCAPR(no)       	(TC90400_TMR_BASE + 0x310 + ((no) * 0x4))
#define TC90400_TMRR         	    (TC90400_TMR_BASE + 0x318)

/* timer control bits */
#define TC90400_TMTCR_TCE_RUNS			(0x1 << 7)		/* Timer counter enable */
#define TC90400_TMTCR_CCDE_ENABLE   		(0x1 << 6)		/* Counter Clock Divide Enable */
#define TC90400_TMTCR_CRE_ENABLE    		(0x1 << 5)		/* Counter Reset Enable */
#define TC90400_TMTCR_CCS_INTERNAL  		(0x0 << 2)		/* Counter Clock Select */
#define TC90400_TMTCR_TMODE_IT      		(0x0)   		/* Timer Mode :Interval Timer Mode */

/* Interval Timer mode bits */
#define TC90400_TMITMR_TIIE_ENABLE  		(0x1 << 15)
#define TC90400_TMITMR_TZCE_ENABLE  		(0x1)

/* Capture Configuration bit */
#define TC90400_TMCAPCONF_STA			(0x1 << 31)	  /* Timer start/stop */
#define TC90400_TMCAPCONF_RST			(0x1 << 30)	  /* Timer reset */
#define TC90400_TMCAPCONF_TPRESCT1		(0x0 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_TPRESCT2		(0x1 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_TPRESCT3		(0x2 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_TPRESCT4		(0x3 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_TPRESCT5		(0x4 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_TPRESCT6		(0x5 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_TPRESCT7		(0x6 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_TPRESCT8		(0x7 << 26)	  /* Timer prescaler */
#define TC90400_TMCAPCONF_EN1			(0x1 << 7)	  /* Interrupt enable/disable  */
#define TC90400_TMCAPCONF_EN0			(0x1 << 6)	  /* Interrupt enable/disable  */
#define TC90400_TMCAPCONF_CAP1R			(0x1 << 4)	  /* Capture rising */
#define TC90400_TMCAPCONF_CAP1F			(0x2 << 4)	  /* Capture falling */
#define TC90400_TMCAPCONF_CAP1RF		(0x3 << 4)	  /* Capture rising/falling */
#define TC90400_TMCAPCONF_CAP0R			(0x1 << 2)	  /* Capture rising */
#define TC90400_TMCAPCONF_CAP0F			(0x2 << 2)	  /* Capture falling */
#define TC90400_TMCAPCONF_CAP0RF		(0x3 << 2)	  /* Capture rising/falling */
#define TC90400_TMCAPCONF_INT1			(0x1 << 1)	  /* Interrupt status */
#define TC90400_TMCAPCONF_INT0			(0x1 << 0)	  /* Interrupt status */

/* Compare Configuration bit */
#define TC90400_TMCMPCONF_TZCE1			(0x1 << 7)	  /* Counter reset */
#define TC90400_TMCMPCONF_TZCE0			(0x1 << 6)	  /* Counter reset */
#define TC90400_TMCMPCONF_EN1			(0x1 << 5)	  /* Interrupt enable/disable */
#define TC90400_TMCMPCONF_EN0			(0x1 << 4)	  /* Interrupt enable/disable */
#define TC90400_TMCMPCONF_CMPEN1		(0x1 << 3)	  /* Compare enable/disable */
#define TC90400_TMCMPCONF_CMPEN0		(0x1 << 2)	  /* Compare enable/disable */
#define TC90400_TMCMPCONF_INT1			(0x1 << 1)	  /* Interrupt status */
#define TC90400_TMCMPCONF_INT0			(0x1 << 0)	  /* Interrupt status */


/*
 * PIO
*/
/* Parallel I/O Port register map */
#define TC90400_XPIODO		(TC90400_PIO_BASE)
#define TC90400_XPIODI		(TC90400_PIO_BASE + 0x0004)
#define TC90400_XPIODIR		(TC90400_PIO_BASE + 0x0008)
#define TC90400_XPIOFLAG0	(TC90400_PIO_BASE + 0x0010)
#define TC90400_XPIOFLAG1	(TC90400_PIO_BASE + 0x0014)
#define TC90400_XPIOPOL	        (TC90400_PIO_BASE + 0x0018)
#define TC90400_XPIOINT 	(TC90400_PIO_BASE + 0x001c)
#define TC90400_XPIOMASKCPU	(TC90400_PIO_BASE + 0x0020)
#define TC90400_XPIOMASKEXT	(TC90400_PIO_BASE + 0x0024)


/*
 * CG
*/
/* Clock Generator register map */
#define TC90400_CG_COCR		(TC90400_CG_BASE + 0x04)
#define TC90400_CG_OMCR		(TC90400_CG_BASE + 0x08)
#define TC90400_CG_RTCDCR	(TC90400_CG_BASE + 0x34)
#define TC90400_CG_RTCCR	(TC90400_CG_BASE + 0x38)
#define TC90400_CG_RTCRCR	(TC90400_CG_BASE + 0x3c)
#define TC90400_CG_RTCSR	(TC90400_CG_BASE + 0x40)
#define TC90400_CG_RSTCR	(TC90400_CG_BASE + 0x44)
#define TC90400_CG_INTMCR	(TC90400_CG_BASE + 0x48)
#define TC90400_CG_INTCCR	(TC90400_CG_BASE + 0x4c)
#define TC90400_CG_INTSR	(TC90400_CG_BASE + 0x50)


/*
 * IRC
*/
/* Interrupt Controler register map */
#define TC90400_INTCONT0		(TC90400_IRC_BASE)
#define TC90400_INTCONT1		(TC90400_IRC_BASE + 0x0004)
#define TC90400_INTCONT2		(TC90400_IRC_BASE + 0x0008)
#define TC90400_INTLEVEL(no)		(TC90400_IRC_BASE + 0x0010 + ((no) * 0x4))
#define TC90400_INTMASK			(TC90400_IRC_BASE + 0x0040)
#define TC90400_INTSTSC		        (TC90400_IRC_BASE + 0x0060)
#define TC90400_INTVEC			(TC90400_IRC_BASE + 0x00a0)

/* interrupt control bits */
#define TC90400_INTCONT0_ICF		0x01
#define TC90400_INTCONT1_I2C1		0x00c00000	/* I2C ch.1 to Rising edge sense, other Low level sense */
#define TC90400_INTCONT1_RTCC		0xc0000000	/* RTCclock to Rising edge sense */
#define TC90400_INTCONT1_EXT0_F		0x00000008	/* EXTINT0 to Falling edge sense */
#define TC90400_INTCONT1_EXT0		0x0000000c	/* EXTINT0 to Rising edge sense */
#define TC90400_INTCONT1_EXT1		0x00000030	/* EXTINT1 to Rising edge sense */
#define TC90400_INTCONT2_I2C0		0x00300000	/* I2C ch.0 to Rising edge sense, other Low level sense */
/* interrupt mask bits */
#define TC90400_INTMASK_IML(n)		((n) & 0x7)
/* interrupt vector bits */
#define TC90400_INTVEC_FLAG		(0x1 << 16)
#define TC90400_INTVEC_CAUSE_MASK	0x1f
#define TC90400_INTSTSC_EDGECLEAR	0x100
#define TC90400_INTSTSC_EICLR0	        0x0
#define TC90400_INTSTSC_EICLR1	        0x1
#define TC90400_INTSTSC_EICLR2	        0x2
#define TC90400_INTSTSC_EICLR3	        0x3
#define TC90400_INTSTSC_EICLR4	        0x4
#define TC90400_INTSTSC_EICLR5	        0x5
#define TC90400_INTSTSC_EICLR6	        0x6
#define TC90400_INTSTSC_EICLR7	        0x7
#define TC90400_INTSTSC_EICLR8	        0x8
#define TC90400_INTSTSC_EICLR9	        0x9
#define TC90400_INTSTSC_EICLR10	        0xa
#define TC90400_INTSTSC_EICLR11	        0xb
#define TC90400_INTSTSC_EICLR12	        0xc
#define TC90400_INTSTSC_EICLR13	        0xd
#define TC90400_INTSTSC_EICLR14	        0xe
#define TC90400_INTSTSC_EICLR15	        0xf

/*
 * DMA
*/
/* DMA Controler register map */
#define TC90400_DMACMCR			(TC90400_DMAC_BASE + 0x0154)

/* DMA Master control bits */
#define TC90400_DMACMCR_RESET		0		/* Master reset */
#define TC90400_DMACMCR_MSTEN		0x1		/* Master Enable */
#define TC90400_DMACMCR_RSFIF		(0x1 << 7)	/* Reset FIFO */

/*
 * MPIF define.
 */
#ifndef _LANGUAGE_ASSEMBLY
#ifndef CONFIG_TOSHIBA_TC90400_REV1
struct tc90400_mpif_reg {
  volatile __u64 mcfr;          /* Configuration Register */
  volatile __u64 mctr;          /* CI Timeout Register */
  volatile __u64 matr;          /* ATA Timeout Register */
  volatile __u64 mcmr;          /* CI Masking Register */
  volatile __u64 mamr;          /* ATA Masking Register */
  volatile __u64 mcr;           /* Control Register */
  volatile __u64 mnfcr;         /* NF Control Register */
  volatile __u64 matcr;         /* ATA Control Register */
  volatile __u64 msr;           /* Status Register */
};
/* +4: TC90400 GBUS interface for BigEndian access */
#define tc90400_mpifptr         ((struct tc90400_mpif_reg *)(TC90400_MPIF_BASE + 4))
#endif /* #ifndef CONFIG_TOSHIBA_TC90400_REV1 */
#endif /* #ifndef _LANGUAGE_ASSEMBLY */

/* MPIF controller registers bitd */
#define TC90400_MPIF_MCFR_CS0NF       0x00000001 /* Nand Flash connected to CS0 */
#define TC90400_MPIF_MCFR_CS1NF       0x00000002 /* Nand Flash connected to CS1 */
#define TC90400_MPIF_MCR_DISNF        0x00000008 /* Nand Flash Module disabled */
#define TC90400_MPIF_MATCR_PTRFSDVH_1 0x00000001 /* ATA Control PTRFSDVH=1 */
                                                                         


#define TC90400_RD08( reg      )   (*(vu08*)(reg))
#define TC90400_WR08( reg, val )  ((*(vu08*)(reg))=(val))

#define TC90400_RD16( reg      )   (*(vu16*)(reg))
#define TC90400_WR16( reg, val )  ((*(vu16*)(reg))=(val))

#define TC90400_RD32( reg      )   (*(vu32*)(reg))
#define TC90400_WR32( reg, val )  ((*(vu32*)(reg))=(val))

#define TC90400_RD64( reg      )   (*(vu64*)(reg))
#define TC90400_WR64( reg, val )  ((*(vu64*)(reg))=(val))

#define TC90400_RD( reg      ) TC90400_RD32( reg )
#define TC90400_WR( reg, val ) TC90400_WR32( reg, val )


/**** define interrupt sources ****/
#define TC90400_IR_EXT0			1
#define TC90400_IR_EXT1			2
#define TC90400_IR_EXT2			3
#define TC90400_IR_CI			4	/* MPIF */
#define TC90400_IR_ATA			5	/* MPIF */
#define TC90400_IR_NF			6	/* MPIF */
#define TC90400_IR_GMBR			7
#define TC90400_IR_GFX			8
#define TC90400_IR_VIOU			9	/* VIOU TX IRQ */
#define TC90400_IR_SCC0			10
#define TC90400_IR_SCC1			11
#define TC90400_IR_USB			12
#define	TC90400_IR_ALOC0		13
#define	TC90400_IR_ALOC1		14
#define	TC90400_IR_DMAC0		15
#define	TC90400_IR_DMAC1		16
#define	TC90400_IR_DMAC2		17
#define	TC90400_IR_DMAC3		18
#define	TC90400_IR_I2C1			19
#define	TC90400_IR_SIO0			20
#define	TC90400_IR_SIO1			21
#define	TC90400_IR_SIO2			22
#define	TC90400_IR_RTCC			23
#define	TC90400_IR_RTCWU		24
#define	TC90400_IR_PIO			25
#define	TC90400_IR_I2C0			26
#define	TC90400_IR_TMR3			27
#define	TC90400_IR_TMR0			28
#define	TC90400_IR_TMR1			29
#define	TC90400_IR_TMR2			30
#define	TC90400_IR_GBUS			31

#define TC90400_IRQ_CP0_RAW_BEG   0    /* tc90400 cpu built-in cp0 */
#define TC90400_IRQ_CP0_RAW_END   7
#define TC90400_IRQ_PIC_RAW_BEG   0    /* tc90400 cpu build-in pic */
#define TC90400_IRQ_PIC_RAW_END  31

#define TC90400_IRQ_CP0_BEG  TC90400_IRQ_CP0_RAW_BEG  /* 00 */
#define TC90400_IRQ_CP0_END  TC90400_IRQ_CP0_RAW_END  /* 07 */

#define TC90400_IRQ_PIC_BEG  ((TC90400_IRQ_CP0_END+1)+TC90400_IRQ_PIC_RAW_BEG)  /* 08 */
#define TC90400_IRQ_PIC_END  ((TC90400_IRQ_CP0_END+1)+TC90400_IRQ_PIC_RAW_END)  /* 39 */


#define TC90400_IRQ_USER0            (TC90400_IRQ_CP0_BEG+0)
#define TC90400_IRQ_USER1            (TC90400_IRQ_CP0_BEG+1)
#define TC90400_IRQ_NEST_PIC_ON_CP0  (TC90400_IRQ_CP0_BEG+7)


#define	TC90400_IRQ_EXT0	(TC90400_IRQ_PIC_BEG + TC90400_IR_EXT0)		/* IRQ 9  :             */
#define	TC90400_IRQ_EXT1	(TC90400_IRQ_PIC_BEG + TC90400_IR_EXT1)		/* IRQ 10 :             */
#define	TC90400_IRQ_EXT2	(TC90400_IRQ_PIC_BEG + TC90400_IR_EXT2)		/* IRQ 11 :             */
#define	TC90400_IRQ_CI		(TC90400_IRQ_PIC_BEG + TC90400_IR_CI)		/* IRQ 12 : MPIF-CI     */
#define	TC90400_IRQ_ATA		(TC90400_IRQ_PIC_BEG + TC90400_IR_ATA)		/* IRQ 13 : MPIF-ATA    */
#define	TC90400_IRQ_NF		(TC90400_IRQ_PIC_BEG + TC90400_IR_NF)		/* IRQ 14 : MPIF-NF     */
#define	TC90400_IRQ_GMBR	(TC90400_IRQ_PIC_BEG + TC90400_IR_GMBR)		/* IRQ 15 : GMBR        */
#define	TC90400_IRQ_GFX		(TC90400_IRQ_PIC_BEG + TC90400_IR_GFX)		/* IRQ 16 : GFX         */
#define	TC90400_IRQ_VIOU	(TC90400_IRQ_PIC_BEG + TC90400_IR_VIOU)		/* IRQ 17 : VIOU TX IRQ */
#define	TC90400_IRQ_SCC0	(TC90400_IRQ_PIC_BEG + TC90400_IR_SCC0)		/* IRQ 18 : SCC ch.0    */
#define	TC90400_IRQ_SCC1	(TC90400_IRQ_PIC_BEG + TC90400_IR_SCC1)		/* IRQ 19 : SCC ch.1    */
#define	TC90400_IRQ_USB		(TC90400_IRQ_PIC_BEG + TC90400_IR_USB)		/* IRQ 20 : USB         */
#define	TC90400_IRQ_ALOC0	(TC90400_IRQ_PIC_BEG + TC90400_IR_ALOC0)	/* IRQ 21 : ALOC ch.0   */
#define	TC90400_IRQ_ALOC1	(TC90400_IRQ_PIC_BEG + TC90400_IR_ALOC1)	/* IRQ 22 : ALOC ch.1   */
#define	TC90400_IRQ_DMAC0	(TC90400_IRQ_PIC_BEG + TC90400_IR_DMAC0)	/* IRQ 23 : DMAC ch.0   */
#define	TC90400_IRQ_DMAC1	(TC90400_IRQ_PIC_BEG + TC90400_IR_DMAC1)	/* IRQ 24 : DMAC ch.1   */
#define	TC90400_IRQ_DMAC2	(TC90400_IRQ_PIC_BEG + TC90400_IR_DMAC2)	/* IRQ 25 : DMAC ch.2   */
#define	TC90400_IRQ_DMAC3	(TC90400_IRQ_PIC_BEG + TC90400_IR_DMAC3)	/* IRQ 26 : DMAC ch.3   */
#define	TC90400_IRQ_I2C1	(TC90400_IRQ_PIC_BEG + TC90400_IR_I2C1)		/* IRQ 27 : I2C ch.1    */
#define	TC90400_IRQ_SIO0	(TC90400_IRQ_PIC_BEG + TC90400_IR_SIO0)		/* IRQ 28 : SIO ch.0    */
#define	TC90400_IRQ_SIO1	(TC90400_IRQ_PIC_BEG + TC90400_IR_SIO1)		/* IRQ 29 : SIO ch.1    */
#define	TC90400_IRQ_SIO2	(TC90400_IRQ_PIC_BEG + TC90400_IR_SIO2)		/* IRQ 30 : SIO ch.2    */
#define	TC90400_IRQ_RTCC	(TC90400_IRQ_PIC_BEG + TC90400_IR_RTCC)		/* IRQ 31 : RTC C       */
#define	TC90400_IRQ_RTCWU	(TC90400_IRQ_PIC_BEG + TC90400_IR_RTCWU)	/* IRQ 32 : RTC WU      */
#define	TC90400_IRQ_PIO		(TC90400_IRQ_PIC_BEG + TC90400_IR_PIO)		/* IRQ 33 : PIO         */
#define	TC90400_IRQ_I2C0	(TC90400_IRQ_PIC_BEG + TC90400_IR_I2C0)		/* IRQ 34 : I2C ch.0    */
#define	TC90400_IRQ_TMR3	(TC90400_IRQ_PIC_BEG + TC90400_IR_TMR3)		/* IRQ 35 : Timer ch.3  */
#define	TC90400_IRQ_TMR0	(TC90400_IRQ_PIC_BEG + TC90400_IR_TMR0)		/* IRQ 36 : Timer ch.0  */
#define	TC90400_IRQ_TMR1	(TC90400_IRQ_PIC_BEG + TC90400_IR_TMR1)		/* IRQ 37 : Timer ch.1  */
#define	TC90400_IRQ_TMR2	(TC90400_IRQ_PIC_BEG + TC90400_IR_TMR2)		/* IRQ 38 : Timer ch.2  */
#define	TC90400_IRQ_GBUS	(TC90400_IRQ_PIC_BEG + TC90400_IR_GBUS)		/* IRQ 39 : GBUS        */

#endif	/* TC90400_TC90400_H */
