/*
 * linux/include/asm-mips/tc90400/tc90400gdrs.h : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal driver for TC90400 gdrsboard
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
#ifndef __ASM_TC90400_TC90400GDRS_H
#define __ASM_TC90400_TC90400GDRS_H

#include <asm/tc90400/tc90400.h>
#ifdef CONFIG_PCI
#include <asm/tc90400/tc90400_pci.h>
#endif


#define TC90400GDRS_WR16(a,b) do { TC90400_WR16(a,b); wbflush(); } while ( 0 )
#define TC90400GDRS_WR08(a,b) do { TC90400_WR08(a,b); wbflush(); } while ( 0 )

#ifdef CONFIG_PCI
#define TC90400GDRS_ISA_IO_OFFSET TC90400_PCIIO
#else
#define TC90400GDRS_ISA_IO_OFFSET 0
#endif

/* Used memory size by tc90400 gdrs */
#define TC90400GDRS_USE_KERNEL_MEMORY_SIZE	16 /* 16MB */

/* Interrupt Number of FPGA0/1 */ 
#define TC90400GDRS_IRQ_FPGA_RAW_BEG   0
#define TC90400GDRS_IRQ_FPGA_RAW_END   15

#define TC90400GDRS_IRQ_FPGA0_BEG  ((TC90400_IRQ_PIC_END+1)+TC90400GDRS_IRQ_FPGA_RAW_BEG)	/* 40 */
#define TC90400GDRS_IRQ_FPGA0_END  ((TC90400_IRQ_PIC_END+1)+TC90400GDRS_IRQ_FPGA_RAW_END)	/* 55 */
#define TC90400GDRS_IRQ_FPGA1_BEG  ((TC90400GDRS_IRQ_FPGA0_END+1)+TC90400GDRS_IRQ_FPGA_RAW_BEG)	/* 56 */
#define TC90400GDRS_IRQ_FPGA1_END  ((TC90400GDRS_IRQ_FPGA0_END+1)+TC90400GDRS_IRQ_FPGA_RAW_END)	/* 71 */

/* FPGA0/1 Interrupt Number on PIC */
#define TC90400GDRS_IRQ_NEST_FPGA0_ON_PIC    TC90400_IRQ_EXT0
#define TC90400GDRS_IRQ_NEST_FPGA1_ON_PIC    TC90400_IRQ_EXT1
/* FPGA0 Interrupt Number */
#define	TC90400GDRS_IRQ_RSV0	   (TC90400GDRS_IRQ_FPGA0_BEG + 0) /* IRQ40 */
#define	TC90400GDRS_IRQ_HDMICNT	   (TC90400GDRS_IRQ_FPGA0_BEG + 1) /* IRQ41 */
#define	TC90400GDRS_IRQ_RSV2	   (TC90400GDRS_IRQ_FPGA0_BEG + 2) /* IRQ42 */
#define	TC90400GDRS_IRQ_CARD	   (TC90400GDRS_IRQ_FPGA0_BEG + 3) /* IRQ43 */
#define	TC90400GDRS_IRQ_POD	   (TC90400GDRS_IRQ_FPGA0_BEG + 4) /* IRQ44 */
#define	TC90400GDRS_IRQ_PLAN	   (TC90400GDRS_IRQ_FPGA0_BEG + 5) /* IRQ45 */
#define	TC90400GDRS_IRQ_CI	   (TC90400GDRS_IRQ_FPGA0_BEG + 6) /* IRQ46 */
/* FPGA1 Interrupt Number */
#define	TC90400GDRS_IRQ_SD	   (TC90400GDRS_IRQ_FPGA1_BEG + 0) /* IRQ56 */
#define	TC90400GDRS_IRQ_AVOUT	   (TC90400GDRS_IRQ_FPGA1_BEG + 1) /* IRQ57 */
#define	TC90400GDRS_IRQ_TUNER	   (TC90400GDRS_IRQ_FPGA1_BEG + 2) /* IRQ58 */
#define	TC90400GDRS_IRQ_HDMIRX	   (TC90400GDRS_IRQ_FPGA1_BEG + 3) /* IRQ59 */
#define	TC90400GDRS_IRQ_HDMITX	   (TC90400GDRS_IRQ_FPGA1_BEG + 4) /* IRQ60 */
#define	TC90400GDRS_IRQ_DBLAN	   (TC90400GDRS_IRQ_FPGA1_BEG + 5) /* IRQ61 */
#define	TC90400GDRS_IRQ_MB86619	   (TC90400GDRS_IRQ_FPGA1_BEG + 6) /* IRQ62 */
#define	TC90400GDRS_IRQ_90700AM	   (TC90400GDRS_IRQ_FPGA1_BEG + 7) /* IRQ63 */
#define	TC90400GDRS_IRQ_90700PA	   (TC90400GDRS_IRQ_FPGA1_BEG + 8) /* IRQ64 */
#define	TC90400GDRS_IRQ_90700PB	   (TC90400GDRS_IRQ_FPGA1_BEG + 9) /* IRQ65 */

/* base address */
#define TC90400GDRS_FPGA1_BASE		EGBUS_REG(0x18000000) /* MainBoard */
#define TC90400GDRS_FPGA0_BASE		EGBUS_REG(0x18008000) /* ChipBoard */
#define TC90400GDRS_SRAM_BASE		EGBUS_REG(0x18010000)

/* FPGA0 on ChipBoard Registers */
#define TC90400GDRS_FPGA0_DIPSW         (TC90400GDRS_FPGA0_BASE + 0x0000)
#define TC90400GDRS_FPGA0_PIO07_OUT     (TC90400GDRS_FPGA0_BASE + 0x0020)
#define TC90400GDRS_FPGA0_PIO07_IN      (TC90400GDRS_FPGA0_BASE + 0x0022)
#define TC90400GDRS_FPGA0_PIO07_DIR     (TC90400GDRS_FPGA0_BASE + 0x0024)
#define TC90400GDRS_FPGA0_PIO815_OUT    (TC90400GDRS_FPGA0_BASE + 0x0030)
#define TC90400GDRS_FPGA0_PIO815_IN     (TC90400GDRS_FPGA0_BASE + 0x0032)
#define TC90400GDRS_FPGA0_PIO815_DIR    (TC90400GDRS_FPGA0_BASE + 0x0034)
#define TC90400GDRS_FPGA0_PIO1623_OUT   (TC90400GDRS_FPGA0_BASE + 0x0040)
#define TC90400GDRS_FPGA0_PIO1623_IN    (TC90400GDRS_FPGA0_BASE + 0x0042)
#define TC90400GDRS_FPGA0_PIO1623_DIR   (TC90400GDRS_FPGA0_BASE + 0x0044)
#define TC90400GDRS_FPGA0_PIO2431_OUT   (TC90400GDRS_FPGA0_BASE + 0x0050)
#define TC90400GDRS_FPGA0_PIO2431_IN    (TC90400GDRS_FPGA0_BASE + 0x0052)
#define TC90400GDRS_FPGA0_PIO2431_DIR   (TC90400GDRS_FPGA0_BASE + 0x0054)
#define TC90400GDRS_FPGA0_PIO32_OUT     (TC90400GDRS_FPGA0_BASE + 0x0060)
#define TC90400GDRS_FPGA0_PIO32_IN      (TC90400GDRS_FPGA0_BASE + 0x0062)
#define TC90400GDRS_FPGA0_PIO32_DIR     (TC90400GDRS_FPGA0_BASE + 0x0064)
#define TC90400GDRS_FPGA0_INT_MASK      (TC90400GDRS_FPGA0_BASE + 0x0070)
#define TC90400GDRS_FPGA0_INT_LEVEL     (TC90400GDRS_FPGA0_BASE + 0x0072)
#define TC90400GDRS_FPGA0_INT_STAT      (TC90400GDRS_FPGA0_BASE + 0x0080)
#define TC90400GDRS_FPGA0_INT_CLEAR     (TC90400GDRS_FPGA0_BASE + 0x0084)
#define TC90400GDRS_FPGA0_REVISION      (TC90400GDRS_FPGA0_BASE + 0x0090)

/* FPGA1 on MainBoard Registers */
#define TC90400GDRS_FPGA1_REVISION      (TC90400GDRS_FPGA1_BASE)
#define TC90400GDRS_FPGA1_LED69         (TC90400GDRS_FPGA1_BASE + 0x0006)
#define TC90400GDRS_FPGA1_LED_DSC       (TC90400GDRS_FPGA1_BASE + 0x0008)
#ifdef CONFIG_TOSHIBA_TC90400_QFP
#define TC90400GDRS_FPGA1_INT_MASK1     (TC90400GDRS_FPGA1_BASE + 0x000a)
#define TC90400GDRS_FPGA1_INT_MASK2     (TC90400GDRS_FPGA1_BASE + 0x000b)
#define TC90400GDRS_FPGA1_INT_STAT1     (TC90400GDRS_FPGA1_BASE + 0x000c)
#define TC90400GDRS_FPGA1_INT_STAT2     (TC90400GDRS_FPGA1_BASE + 0x000d)
#define TC90400GDRS_FPGA1_INT_CLEAR1    (TC90400GDRS_FPGA1_BASE + 0x000e)
#define TC90400GDRS_FPGA1_INT_CLEAR2    (TC90400GDRS_FPGA1_BASE + 0x000f)
#else
#define TC90400GDRS_FPGA1_INT_MASK      (TC90400GDRS_FPGA1_BASE + 0x000a)
#define TC90400GDRS_FPGA1_INT_STAT      (TC90400GDRS_FPGA1_BASE + 0x000c)
#define TC90400GDRS_FPGA1_INT_CLEAR     (TC90400GDRS_FPGA1_BASE + 0x000e)
#endif /* #ifdef CONFIG_TOSHIBA_TC90400_QFP */

/* FPGA0 registers bit define */
#define FPGA0_INTMASK_ALLMASK           0x007f
#define FPGA0_DIPSW_MODE1               0x0002
#define FPGA0_DIPSW_MODE2               0x0001
#define FPGA0_DIPSW_MODE3               0x0000
/* FPGA1 registers bit define */
#define FPGA1_INTMASK_ALLMASK           0x03ff

/*
 * ethernet controller register physical address
*/
#define SMC91111CNT_BASE                0x300
/* debug ehternet controller on mainboard */
#define TC90400GDRS_ETH0_PHYS_BASE	(0x180c0000 + SMC91111CNT_BASE)
/* product ehternet controller on chipboard */
#define TC90400GDRS_ETH1_PHYS_BASE	(0x18090000 + SMC91111CNT_BASE)

/*
 * NorFlash on GDRSboard define
*/
#define TC90400_FLASH0_NAME             "TC90400GDRS NOR-FLASH0"
#define TC90400_FLASH1_NAME             "TC90400GDRS NOR-FLASH1"

#ifdef CONFIG_TOSHIBA_TC90400_QFP
#define TC90400_BUSWIDTH                1 /* 8bit  */
#else
#define TC90400_BUSWIDTH                2 /* 16bit */
#endif /* #ifdef CONFIG_TOSHIBA_TC90400_QFP */

#define FLASH_3MB         0x00300000	/* 3MByte */
#define FLASH_4MB         0x00400000	/* 4MByte */
#define FLASH_11MB        0x00b00000	/* 11MByte */
#define FLASH_12MB        0x00c00000	/* 12MByte */
#define FLASH_15MB        0x00f00000	/* 15MByte */
#define FLASH_16MB        0x01000000	/* 16MByte */

struct tc90400gdrs_norflash {
    unsigned long flash0_addr;
    unsigned long flash0_size;
    unsigned long flash0_partnum;
    unsigned long flash1_addr;
    unsigned long flash1_size;
    unsigned long flash1_partnum;
};


#endif /* __ASM_TC90400_TC90400GDRS_H */
