/*
 * linux/include/linux/i2c-adap-tc90400.h : Reference No : 43PJ292-20051019-QFP_BOARD01-000
 *
 * Internal I2C driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/* ------------------------------------------------------------------------- */
/*   Copyright (C) 1995-97 Simon G. Vogl
                   1998-99 Hans Berglund

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/* ------------------------------------------------------------------------- */

/* With some changes from Kyti Mkki <kmalkki@cc.hut.fi> and even
   Frodo Looijaard <frodol@dds.nl> */

/* Modified by MontaVista Software, 2001
   Added support for the TC90400 I2C peripheral */


#ifndef I2C_ADAP_TC90400_H
#define I2C_ADAP_TC90400_H 1

/*
 * This struct contains the hw-dependent functions of TC90400 I2C adapters 
 * to manipulate the registers, and to init any hw-specific features. 
 */
#define TC90400_SLAVE_BUFSIZE	256
/* define trx_status */
#define SLV_NORMAL	        0
#define SLV_READ	      0x1
#define SLV_NEXTREAD	      0x2
#define SLV_READEND	      0x4
#define SLV_CHECKSUM	      0x8
#define SLV_WRITE	     0x10
#define SLV_NEXTWRITE	     0x20
#define SLV_WRITEEND	     0x40
#define SLV_UNDERRFLOW	    0x100
#define SLV_SEQERROR	    0x200
#define SLV_ALERROR	    0x400

struct i2c_tc90400 {
	int		i2c_base;		/* I2C I/O Base Address */
	int		i2c_irq;		/* I2C Interrupt Number */
	int		i2c_clk;		/* I2C clock frequency */
	int		i2c_own;		/* I2C Slave Address */

	spinlock_t	lock;			/* spinlock for internal data */

	spinlock_t	tx_lock;		/* spinlock for tx complete */
	struct completion tx_comp;
	wait_queue_head_t tx_wait;              /* wait queue(slave driver) */
	char*		tx_buf;			/* buffer for transmit */
	int		tx_size;		/* size of buffer */
	int		tx_count;		/* number of transmit */

	spinlock_t	rx_lock;		/* spinlock for rx complete */
	struct completion rx_comp;
        wait_queue_head_t rx_wait;              /* wait queue(slave driver) */
	char*		rx_buf;			/* buffer for receive */
	int		rx_size;		/* size of buffer */
	int		rx_count;		/* number of receive */

	char		skip_intr;		/* flag for interrupt skip */
	short		i2c_sts;		/* saved for SBICR2 */
	int		restart;
	int		trns_mode;		/* transmission mode */
        /* TC90400_TENBIT */
	int		tenbit_addr;		/* 10bit address sending flag */
	int		trx_status;		/* transfer/receive status */
};

#endif /* I2C_ADAP_TC90400_H */
