/*
 * linux/include/linux/i2c-algo-tc90400.h : Reference No : 43PJ292-20050906-QFP_BOARD01-000
 *
 * Internal I2C driver for TC90400
 *
 *  (c) Copyright TOSHIBA CORPORATION SEMICONDUCTOR COMPANY  2003-2005 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 *
 */
/* ------------------------------------------------------------------------- */
/*   Copyright (C) 1995-97 Simon G. Vogl
                   1998-99 Hans Berglund

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/* ------------------------------------------------------------------------- */

/* With some changes from Kyti Mkki <kmalkki@cc.hut.fi> and even
   Frodo Looijaard <frodol@dds.nl> */

/* Modifications by MontaVista Software, 2001
   Changes made to support the TC90400 I2C peripheral */


#ifndef I2C_ALGO_TC90400_H
#define I2C_ALGO_TC90400_H 1

#include <linux/i2c.h>

/* Example of a sequential read request:
	struct i2c_msg s_msg; 

	s_msg.addr=device_address;
	s_msg.len=length;
	s_msg.buf=buffer;
	s_msg.waddr=word_address;
	ioctl(file,I2C_SREAD, &s_msg);
 */
#define I2C_SREAD	0x780	/* SREAD ioctl command */

struct i2c_tc90400_msg {
	__u16 addr;	/* device address */
	__u16 waddr;	/* word address */
	short len;	/* msg length */
	char *buf;	/* pointer to msg data */
};

struct i2c_algo_tc90400_data {
	void  *data;		/* private data for lolevel routines	*/
	void  (*seti2c)		(int ctl, unsigned char val);
        unsigned char (*geti2c)	(int ctl);
	int   (*getown)		(struct i2c_algo_tc90400_data *adap);
	int   (*getclock)	(struct i2c_algo_tc90400_data *adap);
        int   (*setclock)	(struct i2c_algo_tc90400_data *adap, int value);
	void  (*waitforpin)	(struct i2c_algo_tc90400_data *adap, long timout);
	int   (*waitforbb)	(struct i2c_algo_tc90400_data *adap, long timout);
	int   (*waitforack)	(struct i2c_algo_tc90400_data *adap, unsigned short trx, long timout);
	int   (*waitforlrb)	(struct i2c_algo_tc90400_data *adap, long timout);

	/* local settings */
	void (*start)		(struct i2c_algo_tc90400_data *adap);
	void (*stop)		(struct i2c_algo_tc90400_data *adap);
	void (*reset)		(struct i2c_algo_tc90400_data *adap);
	int  (*send)		(struct i2c_algo_tc90400_data *adap);
	int  (*recv)		(struct i2c_algo_tc90400_data *adap);

	int udelay;
	int mdelay;
	int timeout;
	int ch;
};

int i2c_tc90400_add_bus(struct i2c_adapter *);
int i2c_tc90400_del_bus(struct i2c_adapter *);

#endif /* I2C_ALGO_TC90400_H */
